@@ -10,14 +10,13 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)cl_screen.c	10.48 (Berkeley) 9/15/96";
+static const char sccsid[] = "$Id: cl_screen.c,v 10.56 2002/05/03 19:59:44 skimo Exp $ (Berkeley) $Date: 2002/05/03 19:59:44 $";
 #endif /* not lint */
 
 #include <sys/types.h>
 #include <sys/queue.h>
 
 #include <bitstring.h>
-#include <curses.h>
 #include <errno.h>
 #include <signal.h>
 #include <stdio.h>
@@ -34,7 +33,7 @@ static int	cl_ex_init __P((SCR *));
 static void	cl_freecap __P((CL_PRIVATE *));
 static int	cl_vi_end __P((GS *));
 static int	cl_vi_init __P((SCR *));
-static int	cl_putenv __P((char *, char *, u_long));
+static int	cl_putenv __P((SCR *sp, char *, char *, u_long));
 
 /*
  * cl_screen --
@@ -43,26 +42,30 @@ static int	cl_putenv __P((char *, char *, u_long));
  * PUBLIC: int cl_screen __P((SCR *, u_int32_t));
  */
 int
-cl_screen(sp, flags)
-	SCR *sp;
-	u_int32_t flags;
+cl_screen(SCR *sp, u_int32_t flags)
 {
 	CL_PRIVATE *clp;
+	WINDOW *win;
 	GS *gp;
 
 	gp = sp->gp;
 	clp = CLP(sp);
+	win = CLSP(sp) ? CLSP(sp) : stdscr;
 
 	/* See if the current information is incorrect. */
 	if (F_ISSET(gp, G_SRESTART)) {
+		if (CLSP(sp)) {
+		    delwin(CLSP(sp));
+		    sp->cl_private = NULL;
+		}
 		if (cl_quit(gp))
 			return (1);
 		F_CLR(gp, G_SRESTART);
 	}
 	
 	/* See if we're already in the right mode. */
-	if (LF_ISSET(SC_EX) && F_ISSET(sp, SC_SCR_EX) ||
-	    LF_ISSET(SC_VI) && F_ISSET(sp, SC_SCR_VI))
+	if ((LF_ISSET(SC_EX) && F_ISSET(sp, SC_SCR_EX)) ||
+	    (LF_ISSET(SC_VI) && F_ISSET(sp, SC_SCR_VI)))
 		return (0);
 
 	/*
@@ -90,12 +93,12 @@ cl_screen(sp, flags)
 	if (F_ISSET(sp, SC_SCR_VI)) {
 		F_CLR(sp, SC_SCR_VI);
 
-		if (sp->q.cqe_next != (void *)&gp->dq) {
-			(void)move(RLNO(sp, sp->rows), 0);
-			clrtobot();
+		if (sp->q.cqe_next != (void *)&sp->wp->scrq) {
+			(void)wmove(win, RLNO(sp, sp->rows), 0);
+			wclrtobot(win);
 		}
-		(void)move(RLNO(sp, sp->rows) - 1, 0);
-		refresh();
+		(void)wmove(win, RLNO(sp, sp->rows) - 1, 0);
+		wrefresh(win);
 	}
 
 	/* Enter the requested mode. */
@@ -127,8 +130,7 @@ cl_screen(sp, flags)
  * PUBLIC: int cl_quit __P((GS *));
  */
 int
-cl_quit(gp)
-	GS *gp;
+cl_quit(GS *gp)
 {
 	CL_PRIVATE *clp;
 	int rval;
@@ -148,12 +150,12 @@ cl_quit(gp)
 		rval = 1;
 
 	/* Really leave vi mode. */
-	if (F_ISSET(gp, G_STDIN_TTY) &&
+	if (F_ISSET(clp, CL_STDIN_TTY) &&
 	    F_ISSET(clp, CL_SCR_VI_INIT) && cl_vi_end(gp))
 		rval = 1;
 
 	/* Really leave ex mode. */
-	if (F_ISSET(gp, G_STDIN_TTY) &&
+	if (F_ISSET(clp, CL_STDIN_TTY) &&
 	    F_ISSET(clp, CL_SCR_EX_INIT) && cl_ex_end(gp))
 		rval = 1;
 
@@ -178,8 +180,7 @@ cl_quit(gp)
  *	Initialize the curses vi screen.
  */
 static int
-cl_vi_init(sp)
-	SCR *sp;
+cl_vi_init(SCR *sp)
 {
 	CL_PRIVATE *clp;
 	GS *gp;
@@ -193,7 +194,7 @@ cl_vi_init(sp)
 		goto fast;
 
 	/* Curses vi always reads from (and writes to) a terminal. */
-	if (!F_ISSET(gp, G_STDIN_TTY) || !isatty(STDOUT_FILENO)) {
+	if (!F_ISSET(clp, CL_STDIN_TTY) || !isatty(STDOUT_FILENO)) {
 		msgq(sp, M_ERR,
 		    "016|Vi's standard input and output must be a terminal");
 		return (1);
@@ -222,11 +223,11 @@ cl_vi_init(sp)
 	 * the underlying function.
 	 */
 	o_term = getenv("TERM");
-	cl_putenv("TERM", ttype, 0);
+	cl_putenv(sp, "TERM", ttype, 0);
 	o_lines = getenv("LINES");
-	cl_putenv("LINES", NULL, (u_long)O_VAL(sp, O_LINES));
+	cl_putenv(sp, "LINES", NULL, (u_long)O_VAL(sp, O_LINES));
 	o_cols = getenv("COLUMNS");
-	cl_putenv("COLUMNS", NULL, (u_long)O_VAL(sp, O_COLUMNS));
+	cl_putenv(sp, "COLUMNS", NULL, (u_long)O_VAL(sp, O_COLUMNS));
 
 	/*
 	 * We don't care about the SCREEN reference returned by newterm, we
@@ -252,11 +253,11 @@ cl_vi_init(sp)
 	}
 
 	if (o_term == NULL)
-		unsetenv("TERM");
+		cl_unsetenv(sp, "TERM");
 	if (o_lines == NULL)
-		unsetenv("LINES");
+		cl_unsetenv(sp, "LINES");
 	if (o_cols == NULL)
-		unsetenv("COLUMNS");
+		cl_unsetenv(sp, "COLUMNS");
 
 	/*
 	 * XXX
@@ -368,6 +369,8 @@ cl_vi_init(sp)
 
 fast:	/* Set the terminal modes. */
 	if (tcsetattr(STDIN_FILENO, TCSASOFT | TCSADRAIN, &clp->vi_enter)) {
+		if (errno == EINTR)
+			goto fast;
 		msgq(sp, M_SYSERR, "tcsetattr");
 err:		(void)cl_vi_end(sp->gp);
 		return (1);
@@ -380,8 +383,7 @@ err:		(void)cl_vi_end(sp->gp);
  *	Shutdown the vi screen.
  */
 static int
-cl_vi_end(gp)
-	GS *gp;
+cl_vi_end(GS *gp)
 {
 	CL_PRIVATE *clp;
 
@@ -424,8 +426,7 @@ cl_vi_end(gp)
  *	Initialize the ex screen.
  */
 static int
-cl_ex_init(sp)
-	SCR *sp;
+cl_ex_init(SCR *sp)
 {
 	CL_PRIVATE *clp;
 
@@ -436,7 +437,7 @@ cl_ex_init(sp)
 		goto fast;
 
 	/* If not reading from a file, we're done. */
-	if (!F_ISSET(sp->gp, G_STDIN_TTY))
+	if (!F_ISSET(clp, CL_STDIN_TTY))
 		return (0);
 
 	/* Get the ex termcap/terminfo strings. */
@@ -486,6 +487,8 @@ cl_ex_init(sp)
 #endif
 
 fast:	if (tcsetattr(STDIN_FILENO, TCSADRAIN | TCSASOFT, &clp->ex_enter)) {
+		if (errno == EINTR)
+			goto fast;
 		msgq(sp, M_SYSERR, "tcsetattr");
 		return (1);
 	}
@@ -497,8 +500,7 @@ fast:	if (tcsetattr(STDIN_FILENO, TCSADRAIN | TCSASOFT, &clp->ex_enter)) {
  *	Shutdown the ex screen.
  */
 static int
-cl_ex_end(gp)
-	GS *gp;
+cl_ex_end(GS *gp)
 {
 	CL_PRIVATE *clp;
 
@@ -516,9 +518,7 @@ cl_ex_end(gp)
  * PUBLIC: int cl_getcap __P((SCR *, char *, char **));
  */
 int
-cl_getcap(sp, name, elementp)
-	SCR *sp;
-	char *name, **elementp;
+cl_getcap(SCR *sp, char *name, char **elementp)
 {
 	size_t len;
 	char *t;
@@ -536,8 +536,7 @@ cl_getcap(sp, name, elementp)
  *	Free any allocated termcap/terminfo strings.
  */
 static void
-cl_freecap(clp)
-	CL_PRIVATE *clp;
+cl_freecap(CL_PRIVATE *clp)
 {
 	if (clp->el != NULL) {
 		free(clp->el);
@@ -566,16 +565,13 @@ cl_freecap(clp)
  *	Put a value into the environment.
  */
 static int
-cl_putenv(name, str, value)
-	char *name, *str;
-	u_long value;
-
+cl_putenv(SCR *sp, char *name, char *str, u_long value)
 {
 	char buf[40];
 
 	if (str == NULL) {
 		(void)snprintf(buf, sizeof(buf), "%lu", value);
-		return (setenv(name, buf, 1));
+		return (cl_setenv(sp, name, buf));
 	} else
-		return (setenv(name, str, 1));
+		return (cl_setenv(sp, name, str));
 }
