@@ -10,7 +10,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)exf.c	10.48 (Berkeley) 9/15/96";
+static const char sccsid[] = "$Id: exf.c,v 10.72 2003/08/10 09:44:01 skimo Exp $ (Berkeley) $Date: 2003/08/10 09:44:01 $";
 #endif /* not lint */
 
 #include <sys/param.h>
@@ -34,6 +34,7 @@ static const char sccsid[] = "@(#)exf.c	10.48 (Berkeley) 9/15/96";
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <time.h>
 
 #include "common.h"
 
@@ -41,6 +42,7 @@ static int	file_backup __P((SCR *, char *, char *));
 static void	file_cinit __P((SCR *));
 static void	file_comment __P((SCR *));
 static int	file_spath __P((SCR *, FREF *, struct stat *, int *));
+static int   	db_setup __P((SCR *sp, EXF *ep));
 
 /*
  * file_add --
@@ -55,12 +57,10 @@ static int	file_spath __P((SCR *, FREF *, struct stat *, int *));
  * vi now remembers the last location in any file that it has ever edited,
  * not just the previously edited file.
  *
- * PUBLIC: FREF *file_add __P((SCR *, CHAR_T *));
+ * PUBLIC: FREF *file_add __P((SCR *, char *));
  */
 FREF *
-file_add(sp, name)
-	SCR *sp;
-	CHAR_T *name;
+file_add(SCR *sp, char *name)
 {
 	GS *gp;
 	FREF *frp, *tfrp;
@@ -123,20 +123,15 @@ file_add(sp, name)
  * PUBLIC: int file_init __P((SCR *, FREF *, char *, int));
  */
 int
-file_init(sp, frp, rcv_name, flags)
-	SCR *sp;
-	FREF *frp;
-	char *rcv_name;
-	int flags;
+file_init(SCR *sp, FREF *frp, char *rcv_name, int flags)
 {
 	EXF *ep;
-	RECNOINFO oinfo;
 	struct stat sb;
 	size_t psize;
-	int fd, exists, open_err, readonly;
+	int fd, exists, open_err, readonly, stolen;
 	char *oname, tname[MAXPATHLEN];
 
-	open_err = readonly = 0;
+	stolen = open_err = readonly = 0;
 
 	/*
 	 * If the file is a recovery file, let the recovery code handle it.
@@ -156,6 +151,30 @@ file_init(sp, frp, rcv_name, flags)
 	 */
 	F_CLR(frp, ~FR_CURSORSET);
 
+	/*
+	 * Scan the user's path to find the file that we're going to
+	 * try and open.
+	 */
+	if (file_spath(sp, frp, &sb, &exists))
+		return (1);
+
+	/*
+	 * Check whether we already have this file opened in some
+	 * other screen.
+	 */
+	if (exists) {
+		EXF *exfp;
+		for (exfp = sp->gp->exfq.cqh_first;
+		    exfp != (EXF *)&sp->gp->exfq; exfp = exfp->q.cqe_next) {
+			if (exfp->mdev == sb.st_dev &&
+			    exfp->minode == sb.st_ino && 
+			    (exfp != sp->ep || exfp->refcnt > 1)) {
+				ep = exfp;
+				goto postinit;
+			}
+		}
+	}
+
 	/*
 	 * Required EXF initialization:
 	 *	Flush the line caches.
@@ -163,17 +182,11 @@ file_init(sp, frp, rcv_name, flags)
 	 *	Set initial EXF flag bits.
 	 */
 	CALLOC_RET(sp, ep, EXF *, 1, sizeof(EXF));
-	ep->c_lno = ep->c_nlines = OOBLNO;
+	CIRCLEQ_INIT(&ep->scrq);
+	sp->c_lno = ep->c_nlines = OOBLNO;
 	ep->rcv_fd = ep->fcntl_fd = -1;
 	F_SET(ep, F_FIRSTMODIFY);
 
-	/*
-	 * Scan the user's path to find the file that we're going to
-	 * try and open.
-	 */
-	if (file_spath(sp, frp, &sb, &exists))
-		return (1);
-
 	/*
 	 * If no name or backing file, for whatever reason, create a backing
 	 * temporary file, saving the temp file name so we can later unlink
@@ -182,10 +195,10 @@ file_init(sp, frp, rcv_name, flags)
 	 */
 	oname = frp->name;
 	if (LF_ISSET(FS_OPENERR) || oname == NULL || !exists) {
-		if (opts_empty(sp, O_DIRECTORY, 0))
+		if (opts_empty(sp, O_TMP_DIRECTORY, 0))
 			goto err;
 		(void)snprintf(tname, sizeof(tname),
-		    "%s/vi.XXXXXX", O_STR(sp, O_DIRECTORY));
+		    "%s/vi.XXXXXX", O_STR(sp, O_TMP_DIRECTORY));
 		if ((fd = mkstemp(tname)) == -1) {
 			msgq(sp, M_SYSERR,
 			    "237|Unable to create temporary file");
@@ -196,9 +209,11 @@ file_init(sp, frp, rcv_name, flags)
 		if (frp->name == NULL)
 			F_SET(frp, FR_TMPFILE);
 		if ((frp->tname = strdup(tname)) == NULL ||
-		    frp->name == NULL && (frp->name = strdup(tname)) == NULL) {
-			if (frp->tname != NULL)
+		    (frp->name == NULL && 
+		     (frp->name = strdup(tname)) == NULL)) {
+			if (frp->tname != NULL) {
 				free(frp->tname);
+			}
 			msgq(sp, M_SYSERR, NULL);
 			(void)unlink(tname);
 			goto err;
@@ -235,29 +250,48 @@ file_init(sp, frp, rcv_name, flags)
 	}
 
 	/* Set up recovery. */
-	memset(&oinfo, 0, sizeof(RECNOINFO));
-	oinfo.bval = '\n';			/* Always set. */
-	oinfo.psize = psize;
-	oinfo.flags = F_ISSET(sp->gp, G_SNAPSHOT) ? R_SNAPSHOT : 0;
 	if (rcv_name == NULL) {
-		if (!rcv_tmp(sp, ep, frp->name))
-			oinfo.bfname = ep->rcv_path;
+		/* ep->rcv_path NULL if rcv_tmp fails */
+		rcv_tmp(sp, ep, frp->name);
 	} else {
 		if ((ep->rcv_path = strdup(rcv_name)) == NULL) {
 			msgq(sp, M_SYSERR, NULL);
 			goto err;
 		}
-		oinfo.bfname = ep->rcv_path;
 		F_SET(ep, F_MODIFIED);
 	}
 
+	if (db_setup(sp, ep))
+		goto err;
+
 	/* Open a db structure. */
-	if ((ep->db = dbopen(rcv_name == NULL ? oname : NULL,
-	    O_NONBLOCK | O_RDONLY,
-	    S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH,
-	    DB_RECNO, &oinfo)) == NULL) {
+	if ((sp->db_error = db_create(&ep->db, 0, 0)) != 0) {
+		msgq(sp, M_DBERR, "db_create");
+		goto err;
+	}
+
+	ep->db->set_re_delim(ep->db, '\n');		/* Always set. */
+	ep->db->set_pagesize(ep->db, psize);
+	ep->db->set_flags(ep->db, DB_RENUMBER | DB_SNAPSHOT);
+	if (rcv_name == NULL)
+		ep->db->set_re_source(ep->db, oname);
+
+/* 
+ * Don't let db use mmap when using fcntl for locking
+ */
+#ifdef HAVE_LOCK_FCNTL
+#define NOMMAPIFFCNTL DB_NOMMAP
+#else
+#define NOMMAPIFFCNTL 0
+#endif
+
+#define _DB_OPEN_MODE	S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH
+
+	if ((sp->db_error = db_open(ep->db, ep->rcv_path, DB_RECNO,
+	    ((rcv_name == 0) ? DB_TRUNCATE : 0) | VI_DB_THREAD | NOMMAPIFFCNTL,
+	    _DB_OPEN_MODE)) != 0) {
 		msgq_str(sp,
-		    M_SYSERR, rcv_name == NULL ? oname : rcv_name, "%s");
+		    M_DBERR, rcv_name == NULL ? oname : rcv_name, "%s");
 		/*
 		 * !!!
 		 * Historically, vi permitted users to edit files that couldn't
@@ -265,10 +299,33 @@ file_init(sp, frp, rcv_name, flags)
 		 * line, but it's quite useful for "vi *.c", since you can skip
 		 * past files that you can't read.
 		 */ 
+		ep->db = NULL; /* Don't close it; it wasn't opened */
+
+		if (LF_ISSET(FS_OPENERR)) 
+		    goto err;
+
 		open_err = 1;
 		goto oerr;
 	}
 
+	/* re_source is loaded into the database.
+	 * Close it and reopen it in the environment. 
+	 */
+	if ((sp->db_error = ep->db->close(ep->db, 0))) {
+		msgq(sp, M_DBERR, "close");
+		goto err;
+	}
+	if ((sp->db_error = db_create(&ep->db, ep->env, 0)) != 0) {
+		msgq(sp, M_DBERR, "db_create 2");
+		goto err;
+	}
+	if ((sp->db_error = db_open(ep->db, ep->rcv_path, DB_RECNO,
+	    VI_DB_THREAD | NOMMAPIFFCNTL, _DB_OPEN_MODE)) != 0) {
+		msgq_str(sp,
+		    M_DBERR, ep->rcv_path, "%s");
+		goto err;
+	}
+
 	/*
 	 * Do the remaining things that can cause failure of the new file,
 	 * mark and logging initialization.
@@ -276,6 +333,7 @@ file_init(sp, frp, rcv_name, flags)
 	if (mark_init(sp, ep) || log_init(sp, ep))
 		goto err;
 
+postinit:
 	/*
 	 * Set the alternate file name to be the file we're discarding.
 	 *
@@ -310,6 +368,7 @@ file_init(sp, frp, rcv_name, flags)
 			(void)file_end(sp, ep, 1);
 			goto err;
 		}
+		sp->ep = NULL;
 		F_CLR(frp, FR_DONTDELETE);
 	}
 
@@ -332,10 +391,13 @@ file_init(sp, frp, rcv_name, flags)
 	 * when locking is a little more reliable, this should change to be
 	 * an error.
 	 */
-	if (rcv_name == NULL)
-		switch (file_lock(sp, oname,
-		    &ep->fcntl_fd, ep->db->fd(ep->db), 0)) {
+	if (rcv_name == NULL && ep->refcnt == 0) {
+		if ((ep->fd = open(oname, O_RDWR)) == -1)
+		    goto no_lock;
+
+		switch (file_lock(sp, oname, &ep->fcntl_fd, ep->fd, 1)) {
 		case LOCK_FAILED:
+no_lock:
 			F_SET(frp, FR_UNLOCKED);
 			break;
 		case LOCK_UNAVAIL:
@@ -346,6 +408,7 @@ file_init(sp, frp, rcv_name, flags)
 		case LOCK_SUCCESS:
 			break;
 		}
+	}
 
 	/*
          * Historically, the readonly edit option was set per edit buffer in
@@ -391,24 +454,35 @@ file_init(sp, frp, rcv_name, flags)
 	 * probably isn't a problem for vi when it's running standalone.
 	 */
 	if (readonly || F_ISSET(sp, SC_READONLY) ||
-	    !F_ISSET(frp, FR_NEWFILE) &&
+	    (!F_ISSET(frp, FR_NEWFILE) &&
 	    (!(sb.st_mode & (S_IWUSR | S_IWGRP | S_IWOTH)) ||
-	    access(frp->name, W_OK)))
+	    access(frp->name, W_OK))))
 		O_SET(sp, O_READONLY);
 	else
 		O_CLR(sp, O_READONLY);
 
 	/* Switch... */
 	++ep->refcnt;
+	CIRCLEQ_INSERT_HEAD(&ep->scrq, sp, eq);
 	sp->ep = ep;
 	sp->frp = frp;
 
 	/* Set the initial cursor position, queue initial command. */
 	file_cinit(sp);
 
+	/* Report conversion errors again. */
+	F_CLR(sp, SC_CONV_ERROR);
+
 	/* Redraw the screen from scratch, schedule a welcome message. */
 	F_SET(sp, SC_SCR_REFORMAT | SC_STATUS);
 
+	if (frp->lno == OOBLNO)
+		F_SET(sp, SC_SCR_TOP);
+
+	/* Append into the chain of file structures. */
+	if (ep->refcnt == 1)
+		CIRCLEQ_INSERT_TAIL(&sp->gp->exfq, ep, q);
+
 	return (0);
 
 err:	if (frp->name != NULL) {
@@ -427,11 +501,13 @@ oerr:	if (F_ISSET(ep, F_RCV_ON))
 		free(ep->rcv_path);
 		ep->rcv_path = NULL;
 	}
-	if (ep->db != NULL)
-		(void)ep->db->close(ep->db);
+	if (ep->db != NULL) {
+		(void)ep->db->close(ep->db, DB_NOSYNC);
+		ep->db = NULL;
+	}
 	free(ep);
 
-	return (open_err ?
+	return (open_err && !LF_ISSET(FS_OPENERR) ?
 	    file_init(sp, frp, rcv_name, flags | FS_OPENERR) : 1);
 }
 
@@ -441,11 +517,7 @@ oerr:	if (F_ISSET(ep, F_RCV_ON))
  *	try and open.
  */
 static int
-file_spath(sp, frp, sbp, existsp)
-	SCR *sp;
-	FREF *frp;
-	struct stat *sbp;
-	int *existsp;
+file_spath(SCR *sp, FREF *frp, struct stat *sbp, int *existsp)
 {
 	CHAR_T savech;
 	size_t len;
@@ -461,8 +533,8 @@ file_spath(sp, frp, sbp, existsp)
 		*existsp = 0;
 		return (0);
 	}
-	if (name[0] == '/' || name[0] == '.' &&
-	    (name[1] == '/' || name[1] == '.' && name[2] == '/')) {
+	if (name[0] == '/' || (name[0] == '.' &&
+	    (name[1] == '/' || (name[1] == '.' && name[2] == '/')))) {
 		*existsp = !stat(name, sbp);
 		return (0);
 	}
@@ -508,13 +580,14 @@ file_spath(sp, frp, sbp, existsp)
  *	Set up the initial cursor position.
  */
 static void
-file_cinit(sp)
-	SCR *sp;
+file_cinit(SCR *sp)
 {
 	GS *gp;
 	MARK m;
 	size_t len;
 	int nb;
+	CHAR_T *wp;
+	size_t wlen;
 
 	/* Set some basic defaults. */
 	sp->lno = 1;
@@ -548,8 +621,9 @@ file_cinit(sp)
 			sp->lno = 1;
 			sp->cno = 0;
 		}
-		if (ex_run_str(sp,
-		    "-c option", gp->c_option, strlen(gp->c_option), 1, 1))
+		CHAR2INT(sp, gp->c_option, strlen(gp->c_option) + 1,
+			 wp, wlen);
+		if (ex_run_str(sp, "-c option", wp, wlen - 1, 1, 1))
 			return;
 		gp->c_option = NULL;
 	} else if (F_ISSET(sp, SC_EX)) {
@@ -617,10 +691,7 @@ file_cinit(sp)
  * PUBLIC: int file_end __P((SCR *, EXF *, int));
  */
 int
-file_end(sp, ep, force)
-	SCR *sp;
-	EXF *ep;
-	int force;
+file_end(SCR *sp, EXF *ep, int force)
 {
 	FREF *frp;
 
@@ -633,6 +704,7 @@ file_end(sp, ep, force)
 	 */
 	if (ep == NULL)
 		ep = sp->ep;
+	CIRCLEQ_REMOVE(&ep->scrq, sp, eq);
 	if (--ep->refcnt != 0)
 		return (0);
 
@@ -678,10 +750,15 @@ file_end(sp, ep, force)
 	 *
 	 * Close the db structure.
 	 */
-	if (ep->db->close != NULL && ep->db->close(ep->db) && !force) {
-		msgq_str(sp, M_SYSERR, frp->name, "241|%s: close");
-		++ep->refcnt;
-		return (1);
+	if (ep->db->close != NULL) {
+		if ((sp->db_error = ep->db->close(ep->db, DB_NOSYNC)) != 0 && 
+		    !force) {
+			msgq_str(sp, M_DBERR, frp->name, "241|%s: close");
+			CIRCLEQ_INSERT_HEAD(&ep->scrq, sp, eq);
+			++ep->refcnt;
+			return (1);
+		}
+		ep->db = NULL;
 	}
 
 	/* COMMITTED TO THE CLOSE.  THERE'S NO GOING BACK... */
@@ -692,6 +769,19 @@ file_end(sp, ep, force)
 	/* Free up any marks. */
 	(void)mark_end(sp, ep);
 
+	if (ep->env) {
+		DB_ENV *env;
+
+		ep->env->close(ep->env, 0);
+		ep->env = 0;
+		if ((sp->db_error = db_env_create(&env, 0)))
+			msgq(sp, M_DBERR, "env_create");
+		if ((sp->db_error = db_env_remove(env, ep->env_path, 0)))
+			msgq(sp, M_DBERR, "env->remove");
+		if (ep->env_path != NULL && rmdir(ep->env_path))
+			msgq_str(sp, M_SYSERR, ep->env_path, "242|%s: remove");
+	}
+
 	/*
 	 * Delete recovery files, close the open descriptor, free recovery
 	 * memory.  See recover.c for a description of the protocol.
@@ -707,10 +797,15 @@ file_end(sp, ep, force)
 		if (ep->rcv_mpath != NULL && unlink(ep->rcv_mpath))
 			msgq_str(sp, M_SYSERR, ep->rcv_mpath, "243|%s: remove");
 	}
+	CIRCLEQ_REMOVE(&sp->gp->exfq, ep, q);
+	if (ep->fd != -1)
+		(void)close(ep->fd);
 	if (ep->fcntl_fd != -1)
 		(void)close(ep->fcntl_fd);
 	if (ep->rcv_fd != -1)
 		(void)close(ep->rcv_fd);
+	if (ep->env_path != NULL)
+		free(ep->env_path);
 	if (ep->rcv_path != NULL)
 		free(ep->rcv_path);
 	if (ep->rcv_mpath != NULL)
@@ -729,11 +824,7 @@ file_end(sp, ep, force)
  * PUBLIC: int file_write __P((SCR *, MARK *, MARK *, char *, int));
  */
 int
-file_write(sp, fm, tm, name, flags)
-	SCR *sp;
-	MARK *fm, *tm;
-	char *name;
-	int flags;
+file_write(SCR *sp, MARK *fm, MARK *tm, char *name, int flags)
 {
 	enum { NEWFILE, OLDFILE } mtype;
 	struct stat sb;
@@ -807,8 +898,8 @@ file_write(sp, fm, tm, name, flags)
 		mtype = NEWFILE;
 	else {
 		if (noname && !LF_ISSET(FS_FORCE | FS_APPEND) &&
-		    (F_ISSET(ep, F_DEVSET) &&
-		    (sb.st_dev != ep->mdev || sb.st_ino != ep->minode) ||
+		    ((F_ISSET(ep, F_DEVSET) &&
+		    (sb.st_dev != ep->mdev || sb.st_ino != ep->minode)) ||
 		    sb.st_mtime != ep->mtime)) {
 			msgq_str(sp, M_ERR, name, LF_ISSET(FS_POSSIBLE) ?
 "250|%s: file modified more recently than this copy; use ! to override" :
@@ -889,7 +980,7 @@ file_write(sp, fm, tm, name, flags)
 	 * we re-init the time.  That way the user can clean up the disk
 	 * and rewrite without having to force it.
 	 */
-	if (noname)
+	if (noname) {
 		if (stat(name, &sb))
 			time(&ep->mtime);
 		else {
@@ -899,6 +990,7 @@ file_write(sp, fm, tm, name, flags)
 
 			ep->mtime = sb.st_mtime;
 		}
+	}
 
 	/*
 	 * If the write failed, complain loudly.  ex_writefp() has already
@@ -927,11 +1019,12 @@ file_write(sp, fm, tm, name, flags)
 	 */
 	if (LF_ISSET(FS_ALL) && !LF_ISSET(FS_APPEND)) {
 		F_CLR(ep, F_MODIFIED);
-		if (F_ISSET(frp, FR_TMPFILE))
+		if (F_ISSET(frp, FR_TMPFILE)) {
 			if (noname)
 				F_SET(frp, FR_TMPEXIT);
 			else
 				F_CLR(frp, FR_TMPEXIT);
+		}
 	}
 
 	p = msg_print(sp, name, &nf);
@@ -989,9 +1082,7 @@ file_write(sp, fm, tm, name, flags)
  * recreate the file.  So, let's not risk it.
  */
 static int
-file_backup(sp, name, bname)
-	SCR *sp;
-	char *name, *bname;
+file_backup(SCR *sp, char *name, char *bname)
 {
 	struct dirent *dp;
 	struct stat sb;
@@ -1001,6 +1092,10 @@ file_backup(sp, name, bname)
 	size_t blen;
 	int flags, maxnum, nr, num, nw, rfd, wfd, version;
 	char *bp, *estr, *p, *pct, *slash, *t, *wfname, buf[8192];
+	CHAR_T *wp;
+	size_t wlen;
+	size_t nlen;
+	char *d = NULL;
 
 	rfd = wfd = -1;
 	bp = estr = wfname = NULL;
@@ -1030,14 +1125,14 @@ file_backup(sp, name, bname)
 	 *
 	 * Shell and file name expand the option's value.
 	 */
-	argv_init(sp, &cmd);
-	ex_cinit(&cmd, 0, 0, 0, 0, 0, NULL);
+	ex_cinit(sp, &cmd, 0, 0, 0, 0, 0);
 	if (bname[0] == 'N') {
 		version = 1;
 		++bname;
 	} else
 		version = 0;
-	if (argv_exp2(sp, &cmd, bname, strlen(bname)))
+	CHAR2INT(sp, bname, strlen(bname) + 1, wp, wlen);
+	if (argv_exp2(sp, &cmd, wp, wlen - 1))
 		return (1);
 
 	/*
@@ -1061,9 +1156,13 @@ file_backup(sp, name, bname)
 	 * by one.
 	 */
 	if (version) {
-		GET_SPACE_GOTO(sp, bp, blen, cmd.argv[0]->len * 2 + 50);
-		for (t = bp, slash = NULL,
-		    p = cmd.argv[0]->bp; p[0] != '\0'; *t++ = *p++)
+		GET_SPACE_GOTOC(sp, bp, blen, cmd.argv[0]->len * 2 + 50);
+		INT2SYS(sp, cmd.argv[0]->bp, cmd.argv[0]->len + 1,
+			 p, nlen); 
+		d = strdup(p);
+		p = d;
+		for (t = bp, slash = NULL;
+		     p[0] != '\0'; *t++ = *p++)
 			if (p[0] == '%') {
 				if (p[1] != '%')
 					*t++ = '%';
@@ -1084,7 +1183,8 @@ file_backup(sp, name, bname)
 			p = slash + 1;
 		}
 		if (dirp == NULL) {
-			estr = cmd.argv[0]->bp;
+			INT2SYS(sp, cmd.argv[0]->bp, cmd.argv[0]->len + 1,
+				estr, nlen);
 			goto err;
 		}
 
@@ -1098,7 +1198,8 @@ file_backup(sp, name, bname)
 		wfname = bp;
 	} else {
 		bp = NULL;
-		wfname = cmd.argv[0]->bp;
+		INT2SYS(sp, cmd.argv[0]->bp, cmd.argv[0]->len + 1,
+			wfname, nlen);
 	}
 	
 	/* Open the backup file, avoiding lurkers. */
@@ -1158,6 +1259,8 @@ err:	if (rfd != -1)
 	}
 	if (estr)
 		msgq_str(sp, M_SYSERR, estr, "%s");
+	if (d != NULL)
+		free(d);
 	if (bp != NULL)
 		FREE_SPACE(sp, bp, blen);
 	return (1);
@@ -1168,12 +1271,11 @@ err:	if (rfd != -1)
  *	Skip the first comment.
  */
 static void
-file_comment(sp)
-	SCR *sp;
+file_comment(SCR *sp)
 {
-	recno_t lno;
+	db_recno_t lno;
 	size_t len;
-	char *p;
+	CHAR_T *p;
 
 	for (lno = 1; !db_get(sp, lno, 0, &p, &len) && len == 0; ++lno);
 	if (p == NULL)
@@ -1185,7 +1287,7 @@ file_comment(sp)
 				sp->lno = lno;
 				return;
 			}
-	} else if (len >= 1 && p[0] == '/' && p[1] == '*') {
+	} else if (len > 1 && p[0] == '/' && p[1] == '*') {
 		F_SET(sp, SC_SCR_TOP);
 		do {
 			for (; len > 1; --len, ++p)
@@ -1194,6 +1296,13 @@ file_comment(sp)
 					return;
 				}
 		} while (!db_get(sp, ++lno, 0, &p, &len));
+	} else if (len > 1 && p[0] == '/' && p[1] == '/') {
+		F_SET(sp, SC_SCR_TOP);
+		while (!db_get(sp, ++lno, 0, &p, &len))
+			if (len < 1 || p[0] != '/' || p[1] != '/') {
+				sp->lno = lno;
+				return;
+			}
 	}
 }
 
@@ -1205,9 +1314,7 @@ file_comment(sp)
  * PUBLIC: int file_m1 __P((SCR *, int, int));
  */
 int
-file_m1(sp, force, flags)
-	SCR *sp;
-	int force, flags;
+file_m1(SCR *sp, int force, int flags)
 {
 	EXF *ep;
 
@@ -1223,7 +1330,7 @@ file_m1(sp, force, flags)
 	 * unless force is also set.  Otherwise, we fail unless forced or
 	 * there's another open screen on this file.
 	 */
-	if (F_ISSET(ep, F_MODIFIED))
+	if (F_ISSET(ep, F_MODIFIED)) {
 		if (O_ISSET(sp, O_AUTOWRITE)) {
 			if (!force && file_aw(sp, flags))
 				return (1);
@@ -1233,6 +1340,7 @@ file_m1(sp, force, flags)
 "263|File modified since last complete write; write or use :edit! to override");
 			return (1);
 		}
+	}
 
 	return (file_m3(sp, force));
 }
@@ -1245,9 +1353,7 @@ file_m1(sp, force, flags)
  * PUBLIC: int file_m2 __P((SCR *, int));
  */
 int
-file_m2(sp, force)
-	SCR *sp;
-	int force;
+file_m2(SCR *sp, int force)
 {
 	EXF *ep;
 
@@ -1277,9 +1383,7 @@ file_m2(sp, force)
  * PUBLIC: int file_m3 __P((SCR *, int));
  */
 int
-file_m3(sp, force)
-	SCR *sp;
-	int force;
+file_m3(SCR *sp, int force)
 {
 	EXF *ep;
 
@@ -1313,9 +1417,7 @@ file_m3(sp, force)
  * PUBLIC: int file_aw __P((SCR *, int));
  */
 int
-file_aw(sp, flags)
-	SCR *sp;
-	int flags;
+file_aw(SCR *sp, int flags)
 {
 	if (!F_ISSET(sp->ep, F_MODIFIED))
 		return (0);
@@ -1374,9 +1476,7 @@ file_aw(sp, flags)
  * PUBLIC: void set_alt_name __P((SCR *, char *));
  */
 void
-set_alt_name(sp, name)
-	SCR *sp;
-	char *name;
+set_alt_name(SCR *sp, char *name)
 {
 	if (sp->alt_name != NULL)
 		free(sp->alt_name);
@@ -1388,7 +1488,7 @@ set_alt_name(sp, name)
 
 /*
  * file_lock --
- *	Get an exclusive lock on a file.
+ *	Get an exclusive lock on a file and set close-on-exec flag
  *
  * XXX
  * The default locking is flock(2) style, not fcntl(2).  The latter is
@@ -1410,11 +1510,10 @@ set_alt_name(sp, name)
  * PUBLIC: lockr_t file_lock __P((SCR *, char *, int *, int, int));
  */
 lockr_t
-file_lock(sp, name, fdp, fd, iswrite)
-	SCR *sp;
-	char *name;
-	int *fdp, fd, iswrite;
+file_lock(SCR *sp, char *name, int *fdp, int fd, int iswrite)
 {
+	fcntl(fd, F_SETFD, 1);
+
 	if (!O_ISSET(sp, O_LOCKFILES))
 		return (LOCK_SUCCESS);
 	
@@ -1485,3 +1584,59 @@ file_lock(sp, name, fdp, fd, iswrite)
 	return (LOCK_SUCCESS);
 #endif
 }
+
+#ifdef USE_DB4_LOGGING
+#define VI_DB_INIT_LOG	DB_INIT_LOG
+#else
+#define VI_DB_INIT_LOG	0
+#endif
+
+static int
+db_setup(SCR *sp, EXF *ep)
+{
+	char path[MAXPATHLEN];
+	int fd;
+	DB_ENV	*env;
+
+	(void)snprintf(path, sizeof(path), "%s/vi.XXXXXX", O_STR(sp, O_RECDIR));
+	if ((fd = mkstemp(path)) == -1) {
+		msgq(sp, M_SYSERR, "%s", path);
+		goto err;
+	}
+	(void)close(fd);
+	(void)unlink(path);
+	if (mkdir(path, S_IRWXU)) {
+		msgq(sp, M_SYSERR, "%s", path);
+		goto err;
+	}
+	if (db_env_create(&env, 0)) {
+		msgq(sp, M_ERR, "env_create");
+		goto err;
+	}
+#ifdef USE_DB4_LOGGING
+	if ((sp->db_error = vi_db_init_recover(env))) {
+		msgq(sp, M_DBERR, "init_recover");
+		goto err;
+	}
+	if ((sp->db_error = __vi_init_recover(env))) {
+		msgq(sp, M_DBERR, "init_recover");
+		goto err;
+	}
+#endif
+	if ((sp->db_error = db_env_open(env, path, 
+	    DB_PRIVATE | DB_CREATE | DB_INIT_MPOOL | VI_DB_THREAD 
+	    | VI_DB_INIT_LOG, 0)) != 0) {
+		msgq(sp, M_DBERR, "env->open");
+		goto err;
+	}
+
+	if ((ep->env_path = strdup(path)) == NULL) {
+		msgq(sp, M_SYSERR, NULL);
+		(void)rmdir(path);
+		goto err;
+	}
+	ep->env = env;
+	return 0;
+err:
+	return 1;
+}
