@@ -6,7 +6,7 @@
  *
  * See the LICENSE file for redistribution information.
  *
- *	@(#)exf.h	10.6 (Berkeley) 3/6/96
+ *	$Id: exf.h,v 10.19 2002/03/02 23:36:23 skimo Exp $ (Berkeley) $Date: 2002/03/02 23:36:23 $
  */
 					/* Undo direction. */
 /*
@@ -14,22 +14,27 @@
  *	The file structure.
  */
 struct _exf {
+	CIRCLEQ_ENTRY(_exf) q;		/* Linked list of file structures. */
 	int	 refcnt;		/* Reference count. */
 
+	CIRCLEQ_HEAD(_escrh, _scr)   scrq;   /* Attached screens */
 					/* Underlying database state. */
+	DB_ENV	*env;			/* The DB environment. */
+	char	*env_path;		/* DB environment directory. */
 	DB	*db;			/* File db structure. */
-	char	*c_lp;			/* Cached line. */
-	size_t	 c_len;			/* Cached line length. */
-	recno_t	 c_lno;			/* Cached line number. */
-	recno_t	 c_nlines;		/* Cached lines in the file. */
+	db_recno_t	 c_nlines;	/* Cached lines in the file. */
 
 	DB	*log;			/* Log db structure. */
-	char	*l_lp;			/* Log buffer. */
-	size_t	 l_len;			/* Log buffer length. */
-	recno_t	 l_high;		/* Log last + 1 record number. */
-	recno_t	 l_cur;			/* Log current record number. */
+	db_recno_t	 l_high;	/* Log last + 1 record number. */
+	db_recno_t	 l_cur;		/* Log current record number. */
+#ifdef USE_DB4_LOGGING
+	DB_LSN	lsn_first;
+	DB_LSN	lsn_high;		/* LSN of last record. */
+	DB_LSN	lsn_cur;		/* LSN of first record to undo. */
+#endif
 	MARK	 l_cursor;		/* Log cursor position. */
 	dir_t	 lundo;			/* Last undo direction. */
+	WIN	*l_win;			/* Window owning transaction. */
 
 	LIST_HEAD(_markh, _lmark) marks;/* Linked list of file MARK's. */
 
@@ -42,6 +47,7 @@ struct _exf {
 	time_t	 mtime;			/* Last modification time. */
 
 	int	 fcntl_fd;		/* Fcntl locking fd; see exf.c. */
+	int	 fd;			/* File descriptor */
 
 	/*
 	 * Recovery in general, and these fields specifically, are described
@@ -52,13 +58,16 @@ struct _exf {
 	char	*rcv_mpath;		/* Recover mail file name. */
 	int	 rcv_fd;		/* Locked mail file descriptor. */
 
-#define	F_FIRSTMODIFY	0x001		/* File not yet modified. */
-#define	F_MODIFIED	0x002		/* File is currently dirty. */
-#define	F_MULTILOCK	0x004		/* Multiple processes running, lock. */
-#define	F_NOLOG		0x008		/* Logging turned off. */
-#define	F_RCV_NORM	0x010		/* Don't delete recovery files. */
-#define	F_RCV_ON	0x020		/* Recovery is possible. */
-#define	F_UNDO		0x040		/* No change since last undo. */
+	void	*lock;			/* Lock for log. */
+
+#define	F_DEVSET	0x001		/* mdev/minode fields initialized. */
+#define	F_FIRSTMODIFY	0x002		/* File not yet modified. */
+#define	F_MODIFIED	0x004		/* File is currently dirty. */
+#define	F_MULTILOCK	0x008		/* Multiple processes running, lock. */
+#define	F_NOLOG		0x010		/* Logging turned off. */
+#define	F_RCV_NORM	0x020		/* Don't delete recovery files. */
+#define	F_RCV_ON	0x040		/* Recovery is possible. */
+#define	F_UNDO		0x080		/* No change since last undo. */
 	u_int8_t flags;
 };
 
