@@ -6,7 +6,7 @@
  *
  * See the LICENSE file for redistribution information.
  *
- *	@(#)gs.h	10.33 (Berkeley) 7/12/96
+ *	$Id: gs.h,v 10.55 2001/11/01 10:28:25 skimo Exp $ (Berkeley) $Date: 2001/11/01 10:28:25 $
  */
 
 #define	TEMPORARY_FILE_STRING	"/tmp"	/* Default temporary file name. */
@@ -23,7 +23,7 @@ struct _fref {
 	char	*name;			/* File name. */
 	char	*tname;			/* Backing temporary file name. */
 
-	recno_t	 lno;			/* 1-N: file cursor line. */
+	db_recno_t	 lno;			/* 1-N: file cursor line. */
 	size_t	 cno;			/* 0-N: file cursor column. */
 
 #define	FR_CURSORSET	0x0001		/* If lno/cno values valid. */
@@ -56,20 +56,19 @@ struct _gs {
 	char	*progname;		/* Programe name. */
 
 	int	 id;			/* Last allocated screen id. */
-	CIRCLEQ_HEAD(_dqh, _scr) dq;	/* Displayed screens. */
+	CIRCLEQ_HEAD(_dqh, _win) dq;	/* Displayed windows. */
 	CIRCLEQ_HEAD(_hqh, _scr) hq;	/* Hidden screens. */
 
-	SCR	*ccl_sp;		/* Colon command-line screen. */
-
 	void	*perl_interp;		/* Perl interpreter. */
 	void	*tcl_interp;		/* Tcl_Interp *: Tcl interpreter. */
 
 	void	*cl_private;		/* Curses support private area. */
-	void	*ip_private;		/* IP support private area. */
 	void	*tk_private;		/* Tk/Tcl support private area. */
 
 					/* File references. */
 	CIRCLEQ_HEAD(_frefh, _fref) frefq;
+ 					/* File structures. */
+ 	CIRCLEQ_HEAD(_exfh, _exf) exfq;
 
 #define	GO_COLUMNS	0		/* Global options: columns. */
 #define	GO_LINES	1		/* Global options: lines. */
@@ -82,54 +81,32 @@ struct _gs {
 #define	DEFAULT_NOPRINT	'\1'		/* Emergency non-printable character. */
 	CHAR_T	 noprint;		/* Cached, unprintable character. */
 
-	char	*tmp_bp;		/* Temporary buffer. */
-	size_t	 tmp_blen;		/* Temporary buffer size. */
-
-	/*
-	 * Ex command structures (EXCMD).  Defined here because ex commands
-	 * exist outside of any particular screen or file.
-	 */
-#define	EXCMD_RUNNING(gp)	((gp)->ecq.lh_first->clen != 0)
-	LIST_HEAD(_excmdh, _excmd) ecq;	/* Ex command linked list. */
-	EXCMD	 excmd;			/* Default ex command structure. */
-	char	 *if_name;		/* Current associated file. */
-	recno_t	  if_lno;		/* Current associated line number. */
-
 	char	*c_option;		/* Ex initial, command-line command. */
 
 #ifdef DEBUG
 	FILE	*tracefp;		/* Trace file pointer. */
 #endif
 
-	EVENT	*i_event;		/* Array of input events. */
-	size_t	 i_nelem;		/* Number of array elements. */
-	size_t	 i_cnt;			/* Count of events. */
-	size_t	 i_next;		/* Offset of next event. */
-
-	CB	*dcbp;			/* Default cut buffer pointer. */
-	CB	 dcb_store;		/* Default cut buffer storage. */
-	LIST_HEAD(_cuth, _cb) cutq;	/* Linked list of cut buffers. */
-
 #define	MAX_BIT_SEQ	128		/* Max + 1 fast check character. */
 	LIST_HEAD(_seqh, _seq) seqq;	/* Linked list of maps, abbrevs. */
 	bitstr_t bit_decl(seqb, MAX_BIT_SEQ);
 
-#define	MAX_FAST_KEY	254		/* Max fast check character.*/
+#define	MAX_FAST_KEY	255		/* Max fast check character.*/
 #define	KEY_LEN(sp, ch)							\
-	((unsigned char)(ch) <= MAX_FAST_KEY ?				\
+	((UCHAR_T)(ch) <= MAX_FAST_KEY ?			\
 	    sp->gp->cname[(unsigned char)ch].len : v_key_len(sp, ch))
 #define	KEY_NAME(sp, ch)						\
-	((unsigned char)(ch) <= MAX_FAST_KEY ?				\
+	((UCHAR_T)(ch) <= MAX_FAST_KEY ?				\
 	    sp->gp->cname[(unsigned char)ch].name : v_key_name(sp, ch))
 	struct {
-		CHAR_T	 name[MAX_CHARACTER_COLUMNS + 1];
+		u_char	 name[MAX_CHARACTER_COLUMNS + 1];
 		u_int8_t len;
 	} cname[MAX_FAST_KEY + 1];	/* Fast lookup table. */
 
 #define	KEY_VAL(sp, ch)							\
-	((unsigned char)(ch) <= MAX_FAST_KEY ? 				\
-	    sp->gp->special_key[(unsigned char)ch] :			\
-	    (unsigned char)(ch) > sp->gp->max_special ? 0 : v_key_val(sp,ch))
+	((UCHAR_T)(ch) <= MAX_FAST_KEY ? 				\
+	    sp->gp->special_key[(UCHAR_T)ch] :				\
+	    (UCHAR_T)(ch) > sp->gp->max_special ? K_NOTUSED : v_key_val(sp,ch))
 	CHAR_T	 max_special;		/* Max special character. */
 	u_char				/* Fast lookup table. */
 	    special_key[MAX_FAST_KEY + 1];
@@ -139,16 +116,17 @@ struct _gs {
 #define	G_BELLSCHED	0x0002		/* Bell scheduled. */
 #define	G_INTERRUPTED	0x0004		/* Interrupted. */
 #define	G_RECOVER_SET	0x0008		/* Recover system initialized. */
-#define	G_SCRIPT	0x0010		/* Scripting windows running. */
-#define	G_SNAPSHOT	0x0020		/* Always snapshot files. */
-#define	G_SRESTART	0x0040		/* Screen restarted. */
-#define	G_STDIN_TTY	0x0080		/* Standard input is a tty. */
-#define	G_TMP_INUSE	0x0100		/* Temporary buffer in use. */
+#define	G_SCRIPTED	0x0010		/* Ex script session. */
+#define	G_SCRWIN	0x0020		/* Scripting windows running. */
+#define	G_SNAPSHOT	0x0040		/* Always snapshot files. */
+#define	G_SRESTART	0x0080		/* Screen restarted. */
 	u_int32_t flags;
 
 	/* Screen interface functions. */
 					/* Add a string to the screen. */
 	int	(*scr_addstr) __P((SCR *, const char *, size_t));
+					/* Add a string to the screen. */
+	int	(*scr_waddstr) __P((SCR *, const CHAR_T *, size_t));
 					/* Toggle a screen attribute. */
 	int	(*scr_attr) __P((SCR *, scr_attr_t, int));
 					/* Terminal baud rate. */
@@ -157,12 +135,16 @@ struct _gs {
 	int	(*scr_bell) __P((SCR *));
 					/* Display a busy message. */
 	void	(*scr_busy) __P((SCR *, const char *, busy_t));
+					/* Prepare child. */
+	int	(*scr_child) __P((SCR *));
 					/* Clear to the end of the line. */
 	int	(*scr_clrtoeol) __P((SCR *));
 					/* Return the cursor location. */
 	int	(*scr_cursor) __P((SCR *, size_t *, size_t *));
 					/* Delete a line. */
 	int	(*scr_deleteln) __P((SCR *));
+					/* Discard a screen. */
+	int	(*scr_discard) __P((SCR *, SCR **));
 					/* Get a keyboard event. */
 	int	(*scr_event) __P((SCR *, EVENT *, u_int32_t, int));
 					/* Ex: screen adjustment routine. */
@@ -177,18 +159,38 @@ struct _gs {
 	int	(*scr_optchange) __P((SCR *, int, char *, u_long *));
 					/* Move the cursor. */
 	int	(*scr_move) __P((SCR *, size_t, size_t));
-					/* Message or ex output. */
-	void	(*scr_msg) __P((SCR *, mtype_t, char *, size_t));
 					/* Refresh the screen. */
 	int	(*scr_refresh) __P((SCR *, int));
 					/* Rename the file. */
 	int	(*scr_rename) __P((SCR *, char *, int));
+					/* Reply to an event. */
+	int	(*scr_reply) __P((SCR *, int, char *));
 					/* Set the screen type. */
 	int	(*scr_screen) __P((SCR *, u_int32_t));
+					/* Split the screen. */
+	int	(*scr_split) __P((SCR *, SCR *));
 					/* Suspend the editor. */
 	int	(*scr_suspend) __P((SCR *, int *));
 					/* Print usage message. */
 	void	(*scr_usage) __P((void));
+
+	/* Threading stuff */
+	void	*th_private;
+
+	int	(*run) __P((WIN *, void *(*)(void*), void *));
+
+	int	(*lock_init) __P((WIN *, void **));
+#define LOCK_INIT(wp,s)							    \
+	wp->gp->lock_init(wp, &s->lock)
+	int	(*lock_try) __P((WIN *, void **));
+#define LOCK_TRY(wp,s)							    \
+	wp->gp->lock_try(wp, &s->lock)
+	int	(*lock_unlock) __P((WIN *, void **));
+#define LOCK_UNLOCK(wp,s)						    \
+	wp->gp->lock_unlock(wp, &s->lock)
+	int	(*lock_end) __P((WIN *, void **));
+#define LOCK_END(wp,s)							    \
+	wp->gp->lock_end(wp, &s->lock)
 };
 
 /*
