@@ -10,7 +10,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)key.c	10.32 (Berkeley) 6/30/96";
+static const char sccsid[] = "$Id: key.c,v 10.48 2001/06/25 15:19:10 skimo Exp $ (Berkeley) $Date: 2001/06/25 15:19:10 $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -100,8 +100,7 @@ static int nkeylist =
  * PUBLIC: int v_key_init __P((SCR *));
  */
 int
-v_key_init(sp)
-	SCR *sp;
+v_key_init(SCR *sp)
 {
 	CHAR_T ch;
 	GS *gp;
@@ -138,15 +137,15 @@ v_key_init(sp)
 
 	/* Initialize the fast lookup table. */
 	for (gp->max_special = 0, kp = keylist, cnt = nkeylist; cnt--; ++kp) {
-		if (gp->max_special < kp->value)
-			gp->max_special = kp->value;
+		if (gp->max_special < kp->ch)
+			gp->max_special = kp->ch;
 		if (kp->ch <= MAX_FAST_KEY)
 			gp->special_key[kp->ch] = kp->value;
 	}
 
 	/* Find a non-printable character to use as a message separator. */
 	for (ch = 1; ch <= MAX_CHAR_T; ++ch)
-		if (!isprint(ch)) {
+		if (!ISPRINT(ch)) {
 			gp->noprint = ch;
 			break;
 		}
@@ -166,10 +165,7 @@ v_key_init(sp)
  * in the table, so we check for that first.
  */
 static void
-v_keyval(sp, val, name)
-	SCR *sp;
-	int val;
-	scr_keyval_t name;
+v_keyval(SCR *sp, int val, scr_keyval_t name)
 {
 	KEYLIST *kp;
 	CHAR_T ch;
@@ -203,17 +199,20 @@ v_keyval(sp, val, name)
  * PUBLIC: void v_key_ilookup __P((SCR *));
  */
 void
-v_key_ilookup(sp)
-	SCR *sp;
+v_key_ilookup(SCR *sp)
 {
-	CHAR_T ch, *p, *t;
+	UCHAR_T ch;
+	char *p, *t;
 	GS *gp;
 	size_t len;
 
-	for (gp = sp->gp, ch = 0; ch <= MAX_FAST_KEY; ++ch)
+	for (gp = sp->gp, ch = 0;; ++ch) {
 		for (p = gp->cname[ch].name, t = v_key_name(sp, ch),
 		    len = gp->cname[ch].len = sp->clen; len--;)
 			*p++ = *t++;
+		if (ch == MAX_FAST_KEY)
+			break;
+	}
 }
 
 /*
@@ -224,9 +223,7 @@ v_key_ilookup(sp)
  * PUBLIC: size_t v_key_len __P((SCR *, ARG_CHAR_T));
  */
 size_t
-v_key_len(sp, ch)
-	SCR *sp;
-	ARG_CHAR_T ch;
+v_key_len(SCR *sp, ARG_CHAR_T ch)
 {
 	(void)v_key_name(sp, ch);
 	return (sp->clen);
@@ -237,18 +234,17 @@ v_key_len(sp, ch)
  *	Return the string that will display the key.  This routine
  *	is the backup for the KEY_NAME() macro.
  *
- * PUBLIC: CHAR_T *v_key_name __P((SCR *, ARG_CHAR_T));
+ * PUBLIC: u_char *v_key_name __P((SCR *, ARG_CHAR_T));
  */
-CHAR_T *
-v_key_name(sp, ach)
-	SCR *sp;
-	ARG_CHAR_T ach;
+u_char *
+v_key_name(SCR *sp, ARG_CHAR_T ach)
 {
-	static const CHAR_T hexdigit[] = "0123456789abcdef";
-	static const CHAR_T octdigit[] = "01234567";
-	CHAR_T ch, *chp, mask;
+	static const char hexdigit[] = "0123456789abcdef";
+	static const char octdigit[] = "01234567";
+	CHAR_T ch, mask;
 	size_t len;
 	int cnt, shift;
+	char *chp;
 
 	ch = ach;
 
@@ -281,12 +277,12 @@ v_key_name(sp, ach)
 	 * NB: There's an assumption here that all printable characters take
 	 * up a single column on the screen.  This is not always correct.
 	 */
-	if (isprint(ch)) {
+	if (ISPRINT(ch)) {
 pr:		sp->cname[0] = ch;
 		len = 1;
 		goto done;
 	}
-nopr:	if (iscntrl(ch) && (ch < 0x20 || ch == 0x7f)) {
+nopr:	if (ISCNTRL(ch) && (ch < 0x20 || ch == 0x7f)) {
 		sp->cname[0] = '^';
 		sp->cname[1] = ch == 0x7f ? '?' : '@' + ch;
 		len = 2;
@@ -304,7 +300,11 @@ nopr:	if (iscntrl(ch) && (ch < 0x20 || ch == 0x7f)) {
 		sp->cname[0] = '\\';
 		sp->cname[1] = 'x';
 		for (len = 2, chp = (u_int8_t *)&ch,
-		    cnt = sizeof(CHAR_T); cnt-- > 0; ++chp) {
+		    /* sizeof(CHAR_T) conflict with MAX_CHARACTER_COLUMNS
+		     * and code depends on big endian
+		     * and might not be needed in the long run
+		     */
+		    cnt = /*sizeof(CHAR_T)*/1; cnt-- > 0; ++chp) {
 			sp->cname[len++] = hexdigit[(*chp & 0xf0) >> 4];
 			sp->cname[len++] = hexdigit[*chp & 0x0f];
 		}
@@ -321,9 +321,7 @@ done:	sp->cname[sp->clen = len] = '\0';
  * PUBLIC: int v_key_val __P((SCR *, ARG_CHAR_T));
  */
 int
-v_key_val(sp, ch)
-	SCR *sp;
-	ARG_CHAR_T ch;
+v_key_val(SCR *sp, ARG_CHAR_T ch)
 {
 	KEYLIST k, *kp;
 
@@ -345,23 +343,25 @@ v_key_val(sp, ch)
  * PUBLIC: int v_event_push __P((SCR *, EVENT *, CHAR_T *, size_t, u_int));
  */
 int
-v_event_push(sp, p_evp, p_s, nitems, flags)
-	SCR *sp;
-	EVENT *p_evp;			/* Push event. */
-	CHAR_T *p_s;			/* Push characters. */
-	size_t nitems;			/* Number of items to push. */
-	u_int flags;			/* CH_* flags. */
+v_event_push(SCR *sp, EVENT *p_evp, CHAR_T *p_s, size_t nitems, u_int flags)
+	        
+	             			/* Push event. */
+	            			/* Push characters. */
+	              			/* Number of items to push. */
+	            			/* CH_* flags. */
 {
 	EVENT *evp;
 	GS *gp;
+	WIN *wp;
 	size_t total;
 
 	/* If we have room, stuff the items into the buffer. */
 	gp = sp->gp;
-	if (nitems <= gp->i_next ||
-	    (gp->i_event != NULL && gp->i_cnt == 0 && nitems <= gp->i_nelem)) {
-		if (gp->i_cnt != 0)
-			gp->i_next -= nitems;
+	wp = sp->wp;
+	if (nitems <= wp->i_next ||
+	    (wp->i_event != NULL && wp->i_cnt == 0 && nitems <= wp->i_nelem)) {
+		if (wp->i_cnt != 0)
+			wp->i_next -= nitems;
 		goto copy;
 	}
 
@@ -371,24 +371,24 @@ v_event_push(sp, p_evp, p_s, nitems, flags)
 	 * extra.
 	 */
 #define	TERM_PUSH_SHIFT	30
-	total = gp->i_cnt + gp->i_next + nitems + TERM_PUSH_SHIFT;
-	if (total >= gp->i_nelem && v_event_grow(sp, MAX(total, 64)))
+	total = wp->i_cnt + wp->i_next + nitems + TERM_PUSH_SHIFT;
+	if (total >= wp->i_nelem && v_event_grow(sp, MAX(total, 64)))
 		return (1);
-	if (gp->i_cnt)
-		MEMMOVE(gp->i_event + TERM_PUSH_SHIFT + nitems,
-		    gp->i_event + gp->i_next, gp->i_cnt);
-	gp->i_next = TERM_PUSH_SHIFT;
+	if (wp->i_cnt)
+		MEMMOVE(wp->i_event + TERM_PUSH_SHIFT + nitems,
+		    wp->i_event + wp->i_next, wp->i_cnt);
+	wp->i_next = TERM_PUSH_SHIFT;
 
 	/* Put the new items into the queue. */
-copy:	gp->i_cnt += nitems;
-	for (evp = gp->i_event + gp->i_next; nitems--; ++evp) {
+copy:	wp->i_cnt += nitems;
+	for (evp = wp->i_event + wp->i_next; nitems--; ++evp) {
 		if (p_evp != NULL)
 			*evp = *p_evp++;
 		else {
 			evp->e_event = E_CHARACTER;
 			evp->e_c = *p_s++;
 			evp->e_value = KEY_VAL(sp, evp->e_c);
-			F_INIT(&evp->e_ch, flags);
+			FL_INIT(evp->e_flags, flags);
 		}
 	}
 	return (0);
@@ -399,23 +399,21 @@ copy:	gp->i_cnt += nitems;
  *	Append events onto the tail of the buffer.
  */
 static int
-v_event_append(sp, argp)
-	SCR *sp;
-	EVENT *argp;
+v_event_append(SCR *sp, EVENT *argp)
 {
 	CHAR_T *s;			/* Characters. */
 	EVENT *evp;
-	GS *gp;
+	WIN *wp;
 	size_t nevents;			/* Number of events. */
 
 	/* Grow the buffer as necessary. */
 	nevents = argp->e_event == E_STRING ? argp->e_len : 1;
-	gp = sp->gp;
-	if (gp->i_event == NULL ||
-	    nevents > gp->i_nelem - (gp->i_next + gp->i_cnt))
+	wp = sp->wp;
+	if (wp->i_event == NULL ||
+	    nevents > wp->i_nelem - (wp->i_next + wp->i_cnt))
 		v_event_grow(sp, MAX(nevents, 64));
-	evp = gp->i_event + gp->i_next + gp->i_cnt;
-	gp->i_cnt += nevents;
+	evp = wp->i_event + wp->i_next + wp->i_cnt;
+	wp->i_cnt += nevents;
 
 	/* Transform strings of characters into single events. */
 	if (argp->e_event == E_STRING)
@@ -432,10 +430,10 @@ v_event_append(sp, argp)
 
 /* Remove events from the queue. */
 #define	QREM(len) {							\
-	if ((gp->i_cnt -= len) == 0)					\
-		gp->i_next = 0;						\
+	if ((wp->i_cnt -= len) == 0)					\
+		wp->i_next = 0;						\
 	else								\
-		gp->i_next += len;					\
+		wp->i_next += len;					\
 }
 
 /*
@@ -526,18 +524,16 @@ v_event_append(sp, argp)
  * PUBLIC: int v_event_get __P((SCR *, EVENT *, int, u_int32_t));
  */
 int
-v_event_get(sp, argp, timeout, flags)
-	SCR *sp;
-	EVENT *argp;
-	int timeout;
-	u_int32_t flags;
+v_event_get(SCR *sp, EVENT *argp, int timeout, u_int32_t flags)
 {
 	EVENT *evp, ev;
 	GS *gp;
 	SEQ *qp;
 	int init_nomap, ispartial, istimeout, remap_cnt;
+	WIN *wp;
 
 	gp = sp->gp;
+	wp = sp->wp;
 
 	/* If simply checking for interrupts, argp may be NULL. */
 	if (argp == NULL)
@@ -549,19 +545,19 @@ retry:	istimeout = remap_cnt = 0;
 	 * If the queue isn't empty and we're timing out for characters,
 	 * return immediately.
 	 */
-	if (gp->i_cnt != 0 && LF_ISSET(EC_TIMEOUT))
+	if (wp->i_cnt != 0 && LF_ISSET(EC_TIMEOUT))
 		return (0);
 
 	/*
 	 * If the queue is empty, we're checking for interrupts, or we're
 	 * timing out for characters, get more events.
 	 */
-	if (gp->i_cnt == 0 || LF_ISSET(EC_INTERRUPT | EC_TIMEOUT)) {
+	if (wp->i_cnt == 0 || LF_ISSET(EC_INTERRUPT | EC_TIMEOUT)) {
 		/*
 		 * If we're reading new characters, check any scripting
 		 * windows for input.
 		 */
-		if (F_ISSET(gp, G_SCRIPT) && sscr_input(sp))
+		if (F_ISSET(gp, G_SCRWIN) && sscr_input(sp))
 			return (1);
 loop:		if (gp->scr_event(sp, argp,
 		    LF_ISSET(EC_INTERRUPT | EC_QUOTED | EC_RAW), timeout))
@@ -606,7 +602,7 @@ append:			if (v_event_append(sp, argp))
 	if (LF_ISSET(EC_INTERRUPT | EC_TIMEOUT))
 		return (0);
 	 
-newmap:	evp = &gp->i_event[gp->i_next];
+newmap:	evp = &wp->i_event[wp->i_next];
 
 	/* 
 	 * If the next event in the queue isn't a character event, return
@@ -628,13 +624,13 @@ newmap:	evp = &gp->i_event[gp->i_next];
 	 *
 	 * return it forthwith.
 	 */
-	if (istimeout || F_ISSET(&evp->e_ch, CH_NOMAP) ||
+	if (istimeout || FL_ISSET(evp->e_flags, CH_NOMAP) ||
 	    !LF_ISSET(EC_MAPCOMMAND | EC_MAPINPUT) ||
 	    evp->e_c < MAX_BIT_SEQ && !bit_test(gp->seqb, evp->e_c))
 		goto nomap;
 
 	/* Search the map. */
-	qp = seq_find(sp, NULL, evp, NULL, gp->i_cnt,
+	qp = seq_find(sp, NULL, evp, NULL, wp->i_cnt,
 	    LF_ISSET(EC_MAPCOMMAND) ? SEQ_COMMAND : SEQ_INPUT, &ispartial);
 
 	/*
@@ -664,7 +660,7 @@ newmap:	evp = &gp->i_event[gp->i_next];
 
 	/* If no map, return the character. */
 	if (qp == NULL) {
-nomap:		if (!isdigit(evp->e_c) && LF_ISSET(EC_MAPNODIGIT))
+nomap:		if (!ISDIGIT(evp->e_c) && LF_ISSET(EC_MAPNODIGIT))
 			goto not_digit;
 		*argp = *evp;
 		QREM(1);
@@ -676,16 +672,16 @@ nomap:		if (!isdigit(evp->e_c) && LF_ISSET(EC_MAPNODIGIT))
 	 * of the map is it, pretend we haven't seen the character.
 	 */
 	if (LF_ISSET(EC_MAPNODIGIT) &&
-	    qp->output != NULL && !isdigit(qp->output[0])) {
+	    qp->output != NULL && !ISDIGIT(qp->output[0])) {
 not_digit:	argp->e_c = CH_NOT_DIGIT;
 		argp->e_value = K_NOTUSED;
 		argp->e_event = E_CHARACTER;
-		F_INIT(&argp->e_ch, 0);
+		FL_INIT(argp->e_flags, 0);
 		return (0);
 	}
 
 	/* Find out if the initial segments are identical. */
-	init_nomap = !e_memcmp(qp->output, &gp->i_event[gp->i_next], qp->ilen);
+	init_nomap = !e_memcmp(qp->output, &wp->i_event[wp->i_next], qp->ilen);
 
 	/* Delete the mapped characters from the queue. */
 	QREM(qp->ilen);
@@ -724,7 +720,7 @@ not_digit:	argp->e_c = CH_NOT_DIGIT;
 			if (v_event_push(sp, NULL,
 			    qp->output, qp->ilen, CH_NOMAP | CH_MAPPED))
 				return (1);
-			evp = &gp->i_event[gp->i_next];
+			evp = &wp->i_event[wp->i_next];
 			goto nomap;
 		}
 		if (v_event_push(sp, NULL, qp->output, qp->olen, CH_MAPPED))
@@ -744,14 +740,16 @@ not_digit:	argp->e_c = CH_NOT_DIGIT;
  *	Walk the screen lists, sync'ing files to their backup copies.
  */
 static void
-v_sync(sp, flags)
-	SCR *sp;
-	int flags;
+v_sync(SCR *sp, int flags)
 {
 	GS *gp;
+	WIN *wp;
 
 	gp = sp->gp;
-	for (sp = gp->dq.cqh_first; sp != (void *)&gp->dq; sp = sp->q.cqe_next)
+	for (wp = gp->dq.cqh_first; wp != (void *)&gp->dq; 
+	    wp = wp->q.cqe_next)
+		for (sp = wp->scrq.cqh_first; sp != (void *)&wp->scrq; 
+		    sp = sp->q.cqe_next)
 		rcv_sync(sp, flags);
 	for (sp = gp->hq.cqh_first; sp != (void *)&gp->hq; sp = sp->q.cqe_next)
 		rcv_sync(sp, flags);
@@ -764,9 +762,7 @@ v_sync(sp, flags)
  * PUBLIC: void v_event_err __P((SCR *, EVENT *));
  */
 void
-v_event_err(sp, evp)
-	SCR *sp;
-	EVENT *evp;
+v_event_err(SCR *sp, EVENT *evp)
 {
 	switch (evp->e_event) {
 	case E_CHARACTER:
@@ -778,8 +774,8 @@ v_event_err(sp, evp)
 	case E_INTERRUPT:
 		msgq(sp, M_ERR, "279|Unexpected interrupt event");
 		break;
-	case E_QUIT:
-		msgq(sp, M_ERR, "280|Unexpected quit event");
+	case E_IPCOMMAND:
+		msgq(sp, M_ERR, "318|Unexpected command or input");
 		break;
 	case E_REPAINT:
 		msgq(sp, M_ERR, "281|Unexpected repaint event");
@@ -793,9 +789,6 @@ v_event_err(sp, evp)
 	case E_WRESIZE:
 		msgq(sp, M_ERR, "316|Unexpected resize event");
 		break;
-	case E_WRITE:
-		msgq(sp, M_ERR, "287|Unexpected write event");
-		break;
 
 	/*
 	 * Theoretically, none of these can occur, as they're handled at the
@@ -807,10 +800,6 @@ v_event_err(sp, evp)
 	default:
 		abort();
 	}
-
-	/* Free any allocated memory. */
-	if (evp->e_asp != NULL)
-		free(evp->e_asp);
 }
 
 /*
@@ -820,15 +809,13 @@ v_event_err(sp, evp)
  * PUBLIC: int v_event_flush __P((SCR *, u_int));
  */
 int
-v_event_flush(sp, flags)
-	SCR *sp;
-	u_int flags;
+v_event_flush(SCR *sp, u_int flags)
 {
-	GS *gp;
+	WIN *wp;
 	int rval;
 
-	for (rval = 0, gp = sp->gp; gp->i_cnt != 0 &&
-	    F_ISSET(&gp->i_event[gp->i_next].e_ch, flags); rval = 1)
+	for (rval = 0, wp = sp->wp; wp->i_cnt != 0 &&
+	    FL_ISSET(wp->i_event[wp->i_next].e_flags, flags); rval = 1)
 		QREM(1);
 	return (rval);
 }
@@ -838,18 +825,16 @@ v_event_flush(sp, flags)
  *	Grow the terminal queue.
  */
 static int
-v_event_grow(sp, add)
-	SCR *sp;
-	int add;
+v_event_grow(SCR *sp, int add)
 {
-	GS *gp;
+	WIN *wp;
 	size_t new_nelem, olen;
 
-	gp = sp->gp;
-	new_nelem = gp->i_nelem + add;
-	olen = gp->i_nelem * sizeof(gp->i_event[0]);
-	BINC_RET(sp, gp->i_event, olen, new_nelem * sizeof(gp->i_event[0]));
-	gp->i_nelem = olen / sizeof(gp->i_event[0]);
+	wp = sp->wp;
+	new_nelem = wp->i_nelem + add;
+	olen = wp->i_nelem * sizeof(wp->i_event[0]);
+	BINC_RET(sp, EVENT, wp->i_event, olen, new_nelem * sizeof(EVENT));
+	wp->i_nelem = olen / sizeof(wp->i_event[0]);
 	return (0);
 }
 
@@ -858,8 +843,7 @@ v_event_grow(sp, add)
  *	Compare two keys for sorting.
  */
 static int
-v_key_cmp(ap, bp)
-	const void *ap, *bp;
+v_key_cmp(const void *ap, const void *bp)
 {
 	return (((KEYLIST *)ap)->ch - ((KEYLIST *)bp)->ch);
 }
