@@ -11,14 +11,14 @@
 
 #ifndef lint
 static const char copyright[] =
-"@(#) Copyright (c) 1992, 1993, 1994\n\
+"%Z% Copyright (c) 1992, 1993, 1994\n\
 	The Regents of the University of California.  All rights reserved.\n\
-@(#) Copyright (c) 1992, 1993, 1994, 1995, 1996\n\
+%Z% Copyright (c) 1992, 1993, 1994, 1995, 1996\n\
 	Keith Bostic.  All rights reserved.\n";
 #endif /* not lint */
 
 #ifndef lint
-static const char sccsid[] = "@(#)main.c	10.48 (Berkeley) 10/11/96";
+static const char sccsid[] = "$Id: main.c,v 10.63 2001/11/01 15:24:43 skimo Exp $ (Berkeley) $Date: 2001/11/01 15:24:43 $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -47,13 +47,10 @@ static int	 v_obsolete __P((char *, char *[]));
  * editor --
  *	Main editor routine.
  *
- * PUBLIC: int editor __P((GS *, int, char *[]));
+ * PUBLIC: int editor __P((WIN *, int, char *[]));
  */
 int
-editor(gp, argc, argv)
-	GS *gp;
-	int argc;
-	char *argv[];
+editor(WIN *wp, int argc, char **argv)
 {
 	extern int optind;
 	extern char *optarg;
@@ -61,30 +58,22 @@ editor(gp, argc, argv)
 	EVENT ev;
 	FREF *frp;
 	SCR *sp;
+	GS *gp;
 	size_t len;
 	u_int flags;
 	int ch, flagchk, lflag, secure, startup, readonly, rval, silent;
 	char *tag_f, *wsizearg, path[256];
+	CHAR_T *w;
+	size_t wlen;
+
+	gp = wp->gp;
 
 	/* Initialize the busy routine, if not defined by the screen. */
 	if (gp->scr_busy == NULL)
 		gp->scr_busy = vs_busy;
 	/* Initialize the message routine, if not defined by the screen. */
-	if (gp->scr_msg == NULL)
-		gp->scr_msg = vs_msg;
-
-	/* Common global structure initialization. */
-	CIRCLEQ_INIT(&gp->dq);
-	CIRCLEQ_INIT(&gp->hq);
-	LIST_INIT(&gp->ecq);
-	LIST_INSERT_HEAD(&gp->ecq, &gp->excmd, q);
-	gp->noprint = DEFAULT_NOPRINT;
-
-	/* Structures shared by screens so stored in the GS structure. */
-	CIRCLEQ_INIT(&gp->frefq);
-	CIRCLEQ_INIT(&gp->dcb_store.textq);
-	LIST_INIT(&gp->cutq);
-	LIST_INIT(&gp->seqq);
+	if (wp->scr_msg == NULL)
+		wp->scr_msg = vs_msg;
 
 	/* Set initial screen type and mode based on the program name. */
 	readonly = 0;
@@ -152,7 +141,8 @@ editor(gp, argc, argv)
 			LF_SET(SC_EX);
 			break;
 		case 'F':		/* No snapshot. */
-			F_CLR(gp, G_SNAPSHOT);
+			v_estr(gp->progname, 0, 
+			    "-F option no longer supported.");
 			break;
 		case 'l':		/* Set lisp, showmatch options. */
 			lflag = 1;
@@ -174,14 +164,9 @@ editor(gp, argc, argv)
 		case 's':
 			silent = 1;
 			break;
-#ifdef DEBUG
+#ifdef TRACE
 		case 'T':		/* Trace. */
-			if ((gp->tracefp = fopen(optarg, "w")) == NULL) {
-				v_estr(gp->progname, errno, optarg);
-				goto err;
-			}
-			(void)fprintf(gp->tracefp,
-			    "\n===\ntrace: open %s\n", optarg);
+			(void)vtrace_init(optarg);
 			break;
 #endif
 		case 't':		/* Tag. */
@@ -235,12 +220,15 @@ editor(gp, argc, argv)
 	 * Everything we do until we go interactive is done in ex mode.
 	 */
 	if (screen_init(gp, NULL, &sp)) {
-		if (sp != NULL)
-			CIRCLEQ_INSERT_HEAD(&gp->dq, sp, q);
+		if (sp != NULL) {
+			CIRCLEQ_INSERT_HEAD(&wp->scrq, sp, q);
+			sp->wp = wp;
+		}
 		goto err;
 	}
 	F_SET(sp, SC_EX);
-	CIRCLEQ_INSERT_HEAD(&gp->dq, sp, q);
+	CIRCLEQ_INSERT_HEAD(&wp->scrq, sp, q);
+	sp->wp = wp;
 
 	if (v_key_init(sp))		/* Special key initialization. */
 		goto err;
@@ -331,8 +319,11 @@ editor(gp, argc, argv)
 	}
 
 	/* Open a tag file if specified. */
-	if (tag_f != NULL && ex_tag_first(sp, tag_f))
-		goto err;
+	if (tag_f != NULL) {
+		CHAR2INT(sp, tag_f, strlen(tag_f) + 1, w, wlen);
+		if (ex_tag_first(sp, w))
+			goto err;
+	}
 
 	/*
 	 * Append any remaining arguments as file names.  Files are recovery
@@ -366,7 +357,7 @@ editor(gp, argc, argv)
 			if ((frp = file_add(sp, NULL)) == NULL)
 				goto err;
 		} else  {
-			if ((frp = file_add(sp, (CHAR_T *)sp->argv[0])) == NULL)
+			if ((frp = file_add(sp, sp->argv[0])) == NULL)
 				goto err;
 			if (F_ISSET(sp, SC_ARGRECOVER))
 				F_SET(frp, FR_RECOVER);
@@ -374,7 +365,7 @@ editor(gp, argc, argv)
 
 		if (file_init(sp, frp, NULL, 0))
 			goto err;
-		if (EXCMD_RUNNING(gp)) {
+		if (EXCMD_RUNNING(wp)) {
 			(void)ex_cmd(sp);
 			if (F_ISSET(sp, SC_EXIT | SC_EXIT_FORCE)) {
 				if (screen_end(sp))
@@ -400,8 +391,8 @@ editor(gp, argc, argv)
 			if (v_event_get(sp, &ev, 0, 0))
 				goto err;
 			if (ev.e_event == E_INTERRUPT ||
-			    ev.e_event == E_CHARACTER &&
-			    (ev.e_value == K_CR || ev.e_value == K_NL))
+			    (ev.e_event == E_CHARACTER &&
+			     (ev.e_value == K_CR || ev.e_value == K_NL)))
 				break;
 			(void)gp->scr_bell(sp);
 		}
@@ -423,109 +414,15 @@ done:	rval = 0;
 	if (0)
 err:		rval = 1;
 
-	/* Clean out the global structure. */
-	v_end(gp);
-
 	return (rval);
 }
 
-/*
- * v_end --
- *	End the program, discarding screens and most of the global area.
- *
- * PUBLIC: void v_end __P((GS *));
- */
-void
-v_end(gp)
-	GS *gp;
-{
-	MSGS *mp;
-	SCR *sp;
-
-	/* If there are any remaining screens, kill them off. */
-	if (gp->ccl_sp != NULL) {
-		(void)file_end(gp->ccl_sp, NULL, 1);
-		(void)screen_end(gp->ccl_sp);
-	}
-	while ((sp = gp->dq.cqh_first) != (void *)&gp->dq)
-		(void)screen_end(sp);
-	while ((sp = gp->hq.cqh_first) != (void *)&gp->hq)
-		(void)screen_end(sp);
-
-#ifdef HAVE_PERL_INTERP
-	perl_end(gp);
-#endif
-
-#if defined(DEBUG) || defined(PURIFY) || defined(LIBRARY)
-	{ FREF *frp;
-		/* Free FREF's. */
-		while ((frp = gp->frefq.cqh_first) != (FREF *)&gp->frefq) {
-			CIRCLEQ_REMOVE(&gp->frefq, frp, q);
-			if (frp->name != NULL)
-				free(frp->name);
-			if (frp->tname != NULL)
-				free(frp->tname);
-			free(frp);
-		}
-	}
-
-	/* Free key input queue. */
-	if (gp->i_event != NULL)
-		free(gp->i_event);
-
-	/* Free cut buffers. */
-	cut_close(gp);
-
-	/* Free map sequences. */
-	seq_close(gp);
-
-	/* Free default buffer storage. */
-	(void)text_lfree(&gp->dcb_store.textq);
-
-	/* Close message catalogs. */
-	msg_close(gp);
-#endif
-
-	/* Ring the bell if scheduled. */
-	if (F_ISSET(gp, G_BELLSCHED))
-		(void)fprintf(stderr, "\07");		/* \a */
-
-	/*
-	 * Flush any remaining messages.  If a message is here, it's almost
-	 * certainly the message about the event that killed us (although
-	 * it's possible that the user is sourcing a file that exits from the
-	 * editor).
-	 */
-	while ((mp = gp->msgq.lh_first) != NULL) {
-		(void)fprintf(stderr, "%s%.*s",
-		    mp->mtype == M_ERR ? "ex/vi: " : "", (int)mp->len, mp->buf);
-		LIST_REMOVE(mp, q);
-#if defined(DEBUG) || defined(PURIFY) || defined(LIBRARY)
-		free(mp->buf);
-		free(mp);
-#endif
-	}
-
-#if defined(DEBUG) || defined(PURIFY) || defined(LIBRARY)
-	/* Free any temporary space. */
-	if (gp->tmp_bp != NULL)
-		free(gp->tmp_bp);
-
-#if defined(DEBUG)
-	/* Close debugging file descriptor. */
-	if (gp->tracefp != NULL)
-		(void)fclose(gp->tracefp);
-#endif
-#endif
-}
-
 /*
  * v_obsolete --
  *	Convert historic arguments into something getopt(3) will like.
  */
 static int
-v_obsolete(name, argv)
-	char *name, *argv[];
+v_obsolete(char *name, char **argv)
 {
 	size_t len;
 	char *p;
@@ -560,7 +457,7 @@ v_obsolete(name, argv)
 				argv[0][1] = 'c';
 				(void)strcpy(argv[0] + 2, p + 1);
 			}
-		} else if (argv[0][0] == '-')
+		} else if (argv[0][0] == '-') {
 			if (argv[0][1] == '\0') {
 				MALLOC_NOMSG(NULL, argv[0], char *, 3);
 				if (argv[0] == NULL) {
@@ -573,13 +470,13 @@ nomem:					v_estr(name, errno, NULL);
 				    argv[0][1] == 't' || argv[0][1] == 'w') &&
 				    argv[0][2] == '\0')
 					++argv;
+		}
 	return (0);
 }
 
 #ifdef DEBUG
 static void
-attach(gp)
-	GS *gp;
+attach(GS *gp)
 {
 	int fd;
 	char ch;
@@ -604,9 +501,7 @@ attach(gp)
 #endif
 
 static void
-v_estr(name, eno, msg)
-	char *name, *msg;
-	int eno;
+v_estr(char *name, int eno, char *msg)
 {
 	(void)fprintf(stderr, "%s", name);
 	if (msg != NULL)
