@@ -10,7 +10,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)msg.c	10.39 (Berkeley) 6/20/96";
+static const char sccsid[] = "$Id: msg.c,v 10.61 2003/07/18 23:17:30 skimo Exp $ (Berkeley) $Date: 2003/07/18 23:17:30 $";
 #endif /* not lint */
 
 #include <sys/param.h>
@@ -68,6 +68,7 @@ msgq(sp, mt, fmt, va_alist)
 	static int reenter;		/* STATIC: Re-entrancy check. */
 	CHAR_T ch;
 	GS *gp;
+	WIN *wp;
 	size_t blen, cnt1, cnt2, len, mlen, nlen, soff;
 	const char *p, *t, *u;
 	char *bp, *mp, *rbp, *s_rbp;
@@ -87,6 +88,7 @@ msgq(sp, mt, fmt, va_alist)
 			mt = M_INFO;
 	} else {
 		gp = sp->gp;
+		wp = sp->wp;
 		switch (mt) {
 		case M_BERR:
 			if (F_ISSET(sp, SC_VI) && !O_ISSET(sp, O_VERBOSE)) {
@@ -106,6 +108,7 @@ msgq(sp, mt, fmt, va_alist)
 			break;
 		case M_ERR:
 		case M_SYSERR:
+		case M_DBERR:
 			break;
 		default:
 			abort();
@@ -130,7 +133,7 @@ retry:		FREE_SPACE(sp, bp, blen);
 	}
 	bp = NULL;
 	blen = 0;
-	GET_SPACE_GOTO(sp, bp, blen, nlen);
+	GET_SPACE_GOTOC(sp, bp, blen, nlen);
 
 	/*
 	 * Error prefix.
@@ -142,11 +145,11 @@ retry:		FREE_SPACE(sp, bp, blen);
 #define	REM	(blen - mlen)
 	mp = bp;
 	mlen = 0;
-	if (mt == M_SYSERR) {
+	if (mt == M_SYSERR || mt == M_DBERR) {
 		p = msg_cat(sp, "020|Error: ", &len);
 		if (REM < len)
 			goto retry;
-		memmove(mp, p, len);
+		memcpy(mp, p, len);
 		mp += len;
 		mlen += len;
 	}
@@ -156,14 +159,14 @@ retry:		FREE_SPACE(sp, bp, blen);
 	 * the file name and line number prefix.
 	 */
 	if ((mt == M_ERR || mt == M_SYSERR) &&
-	    sp != NULL && gp != NULL && gp->if_name != NULL) {
-		for (p = gp->if_name; *p != '\0'; ++p) {
+	    sp != NULL && wp != NULL && wp->if_name != NULL) {
+		for (p = wp->if_name; *p != '\0'; ++p) {
 			len = snprintf(mp, REM, "%s", KEY_NAME(sp, *p));
 			mp += len;
 			if ((mlen += len) > blen)
 				goto retry;
 		}
-		len = snprintf(mp, REM, ", %d: ", gp->if_lno);
+		len = snprintf(mp, REM, ", %d: ", wp->if_lno);
 		mp += len;
 		if ((mlen += len) > blen)
 			goto retry;
@@ -333,16 +336,23 @@ nofmt:	mp += len;
 			goto retry;
 		mt = M_ERR;
 	}
+	if (mt == M_DBERR) {
+		len = snprintf(mp, REM, ": %s", db_strerror(sp->db_error));
+		mp += len;
+		if ((mlen += len) > blen)
+			goto retry;
+		mt = M_ERR;
+	}
 
 	/* Add trailing newline. */
 	if ((mlen += 1) > blen)
 		goto retry;
 	*mp = '\n';
 
-	if (sp != NULL)
+	if (sp != NULL && sp->ep != NULL)
 		(void)ex_fflush(sp);
-	if (gp != NULL)
-		gp->scr_msg(sp, mt, bp, mlen);
+	if (wp != NULL)
+		wp->scr_msg(sp, mt, bp, mlen);
 	else
 		(void)fprintf(stderr, "%.*s", (int)mlen, bp);
 
@@ -352,6 +362,26 @@ alloc_err:
 	reenter = 0;
 }
 
+/*
+ * msgq_str --
+ *	Display a message with an embedded string.
+ *
+ * PUBLIC: void msgq_wstr __P((SCR *, mtype_t, CHAR_T *, char *));
+ */
+void
+msgq_wstr(SCR *sp, mtype_t mtype, CHAR_T *str, char *fmt)
+{
+	size_t nlen;
+	char *nstr;
+
+	if (str == NULL) {
+		msgq(sp, mtype, fmt);
+		return;
+	}
+	INT2CHAR(sp, str, STRLEN(str) + 1, nstr, nlen);
+	msgq_str(sp, mtype, nstr, fmt);
+}
+
 /*
  * msgq_str --
  *	Display a message with an embedded string.
@@ -359,10 +389,7 @@ alloc_err:
  * PUBLIC: void msgq_str __P((SCR *, mtype_t, char *, char *));
  */
 void
-msgq_str(sp, mtype, str, fmt)
-	SCR *sp;
-	mtype_t mtype;
-	char *str, *fmt;
+msgq_str(SCR *sp, mtype_t mtype, char *str, char *fmt)
 {
 	int nf, sv_errno;
 	char *p;
@@ -401,8 +428,7 @@ msgq_str(sp, mtype, str, fmt)
  * PUBLIC: void mod_rpt __P((SCR *));
  */
 void
-mod_rpt(sp)
-	SCR *sp;
+mod_rpt(SCR *sp)
 {
 	static char * const action[] = {
 		"293|added",
@@ -417,7 +443,7 @@ mod_rpt(sp)
 		"300|line",
 		"301|lines",
 	};
-	recno_t total;
+	db_recno_t total;
 	u_long rptval;
 	int first, cnt;
 	size_t blen, len, tlen;
@@ -461,7 +487,7 @@ mod_rpt(sp)
 	}
 
 	/* Build and display the message. */
-	GET_SPACE_GOTO(sp, bp, blen, sizeof(action) * MAXNUM + 1);
+	GET_SPACE_GOTOC(sp, bp, blen, sizeof(action) * MAXNUM + 1);
 	for (p = bp, first = 1, tlen = 0,
 	    ap = action, cnt = 0; cnt < ARSIZE(action); ++ap, ++cnt)
 		if (sp->rptlines[cnt] != 0) {
@@ -472,18 +498,19 @@ mod_rpt(sp)
 				*p++ = ' ';
 				tlen += 2;
 			}
-			len = snprintf(p, MAXNUM, "%lu ", sp->rptlines[cnt]);
+			len = snprintf(p, MAXNUM, "%lu ", 
+				(unsigned long) sp->rptlines[cnt]);
 			p += len;
 			tlen += len;
 			t = msg_cat(sp,
 			    lines[sp->rptlines[cnt] == 1 ? 0 : 1], &len);
-			memmove(p, t, len);
+			memcpy(p, t, len);
 			p += len;
 			tlen += len;
 			*p++ = ' ';
 			++tlen;
 			t = msg_cat(sp, *ap, &len);
-			memmove(p, t, len);
+			memcpy(p, t, len);
 			p += len;
 			tlen += len;
 			sp->rptlines[cnt] = 0;
@@ -494,7 +521,7 @@ mod_rpt(sp)
 	++tlen;
 
 	(void)ex_fflush(sp);
-	sp->gp->scr_msg(sp, M_INFO, bp, tlen);
+	sp->wp->scr_msg(sp, M_INFO, bp, tlen);
 
 	FREE_SPACE(sp, bp, blen);
 alloc_err:
@@ -508,30 +535,45 @@ alloc_err:
  * msgq_status --
  *	Report on the file's status.
  *
- * PUBLIC: void msgq_status __P((SCR *, recno_t, u_int));
+ * PUBLIC: void msgq_status __P((SCR *, db_recno_t, u_int));
  */
 void
-msgq_status(sp, lno, flags)
-	SCR *sp;
-	recno_t lno;
-	u_int flags;
+msgq_status(SCR *sp, db_recno_t lno, u_int flags)
 {
-	recno_t last;
-	const char *t;
-	char *bp, *np, *p, *s;
-	int needsep;
+	db_recno_t last;
 	size_t blen, len;
+	int cnt, needsep;
+	const char *t;
+	char **ap, *bp, *np, *p, *s;
 
+	/* Get sufficient memory. */
 	len = strlen(sp->frp->name);
-	GET_SPACE_GOTO(sp, bp, blen, len + 128);
+	GET_SPACE_GOTOC(sp, bp, blen, len * MAX_CHARACTER_COLUMNS + 128);
 	p = bp;
 
-	memmove(p, sp->frp->name, len);
-	p += len;
+	/* Copy in the filename. */
+	for (p = bp, t = sp->frp->name; *t != '\0'; ++t) {
+		len = KEY_LEN(sp, *t);
+		memcpy(p, KEY_NAME(sp, *t), len);
+		p += len;
+	}
 	np = p;
 	*p++ = ':';
 	*p++ = ' ';
 
+	/* Copy in the argument count. */
+	if (F_ISSET(sp, SC_STATUS_CNT) && sp->argv != NULL) {
+		for (cnt = 0, ap = sp->argv; *ap != NULL; ++ap, ++cnt);
+		if (cnt > 1) {
+			(void)sprintf(p,
+			    msg_cat(sp, "317|%d files to edit", NULL), cnt);
+			p += strlen(p);
+			*p++ = ':';
+			*p++ = ' ';
+		}
+		F_CLR(sp, SC_STATUS_CNT);
+	}
+
 	/*
 	 * See nvi/exf.c:file_init() for a description of how and when the
 	 * read-only bit is set.
@@ -543,13 +585,13 @@ msgq_status(sp, lno, flags)
 	if (F_ISSET(sp->frp, FR_NEWFILE)) {
 		F_CLR(sp->frp, FR_NEWFILE);
 		t = msg_cat(sp, "021|new file", &len);
-		memmove(p, t, len);
+		memcpy(p, t, len);
 		p += len;
 		needsep = 1;
 	} else {
 		if (F_ISSET(sp->frp, FR_NAMECHANGE)) {
 			t = msg_cat(sp, "022|name changed", &len);
-			memmove(p, t, len);
+			memcpy(p, t, len);
 			p += len;
 			needsep = 1;
 		}
@@ -561,7 +603,7 @@ msgq_status(sp, lno, flags)
 			t = msg_cat(sp, "023|modified", &len);
 		else
 			t = msg_cat(sp, "024|unmodified", &len);
-		memmove(p, t, len);
+		memcpy(p, t, len);
 		p += len;
 		needsep = 1;
 	}
@@ -571,7 +613,7 @@ msgq_status(sp, lno, flags)
 			*p++ = ' ';
 		}
 		t = msg_cat(sp, "025|UNLOCKED", &len);
-		memmove(p, t, len);
+		memcpy(p, t, len);
 		p += len;
 		needsep = 1;
 	}
@@ -581,7 +623,7 @@ msgq_status(sp, lno, flags)
 			*p++ = ' ';
 		}
 		t = msg_cat(sp, "026|readonly", &len);
-		memmove(p, t, len);
+		memcpy(p, t, len);
 		p += len;
 		needsep = 1;
 	}
@@ -594,7 +636,7 @@ msgq_status(sp, lno, flags)
 			return;
 		if (last == 0) {
 			t = msg_cat(sp, "028|empty file", &len);
-			memmove(p, t, len);
+			memcpy(p, t, len);
 			p += len;
 		} else {
 			t = msg_cat(sp, "027|line %lu of %lu [%ld%%]", &len);
@@ -611,6 +653,7 @@ msgq_status(sp, lno, flags)
 	p += strlen(p);
 #endif
 	*p++ = '\n';
+	len = p - bp;
 
 	/*
 	 * There's a nasty problem with long path names.  Cscope and tags files
@@ -619,26 +662,31 @@ msgq_status(sp, lno, flags)
 	 * has already typed ahead, and chaos results.  If we assume that the
 	 * characters in the filenames and informational messages only take a
 	 * single screen column each, we can trim the filename.
+	 *
+	 * XXX
+	 * Status lines get put up at fairly awkward times.  For example, when
+	 * you do a filter read (e.g., :read ! echo foo) in the top screen of a
+	 * split screen, we have to repaint the status lines for all the screens
+	 * below the top screen.  We don't want users having to enter continue
+	 * characters for those screens.  Make it really hard to screw this up.
 	 */
 	s = bp;
-	if (LF_ISSET(MSTAT_TRUNCATE))
-		if ((p - s) >= sp->cols) {
-			for (; s < np &&
-			    (*s != '/' || (p - s) > sp->cols - 3); ++s);
-			if (s == np)
-				s = bp;
-			else {
-				*--s = '.';
-				*--s = '.';
-				*--s = '.';
-			}
+	if (LF_ISSET(MSTAT_TRUNCATE) && len > sp->cols) {
+		for (; s < np && (*s != '/' || (p - s) > sp->cols - 3); ++s);
+		if (s == np) {
+			s = p - (sp->cols - 5);
+			*--s = ' ';
 		}
-	len = p - s;
+		*--s = '.';
+		*--s = '.';
+		*--s = '.';
+		len = p - s;
+	}
 
 	/* Flush any waiting ex messages. */
 	(void)ex_fflush(sp);
 
-	sp->gp->scr_msg(sp, M_INFO, s, len);
+	sp->wp->scr_msg(sp, M_INFO, s, len);
 
 	FREE_SPACE(sp, bp, blen);
 alloc_err:
@@ -652,9 +700,7 @@ alloc_err:
  * PUBLIC: int msg_open __P((SCR *, char *));
  */
 int
-msg_open(sp, file)
-	SCR *sp;
-	char *file;
+msg_open(SCR *sp, char *file)
 {
 	/*
 	 * !!!
@@ -668,23 +714,24 @@ msg_open(sp, file)
 	static int first = 1;
 	DB *db;
 	DBT data, key;
-	recno_t msgno;
+	db_recno_t msgno;
 	char *p, *t, buf[MAXPATHLEN];
 
 	if ((p = strrchr(file, '/')) != NULL && p[1] == '\0' &&
-	    ((t = getenv("LC_MESSAGES")) != NULL && t[0] != '\0' ||
-	    (t = getenv("LANG")) != NULL && t[0] != '\0')) {
+	    (((t = getenv("LC_MESSAGES")) != NULL && t[0] != '\0') ||
+	    ((t = getenv("LANG")) != NULL && t[0] != '\0'))) {
 		(void)snprintf(buf, sizeof(buf), "%s%s", file, t);
 		p = buf;
 	} else
 		p = file;
-	if ((db = dbopen(p,
-	    O_NONBLOCK | O_RDONLY, 0, DB_RECNO, NULL)) == NULL) {
+	if ((sp->db_error = db_create(&db, 0, 0)) != 0 ||
+	    (sp->db_error = db->set_re_source(db, p)) != 0 ||
+	    (sp->db_error = db_open(db, NULL, DB_RECNO, 0, 0)) != 0) {
 		if (first) {
 			first = 0;
 			return (1);
 		}
-		msgq_str(sp, M_SYSERR, p, "%s");
+		msgq_str(sp, M_DBERR, p, "%s");
 		return (1);
 	}
 
@@ -693,25 +740,27 @@ msg_open(sp, file)
 	 * the message catalog build finds it.
 	 */
 #define	VMC	"VI_MESSAGE_CATALOG"
+	memset(&key, 0, sizeof(key));
 	key.data = &msgno;
-	key.size = sizeof(recno_t);
+	key.size = sizeof(db_recno_t);
+	memset(&data, 0, sizeof(data));
 	msgno = 1;
-	if (db->get(db, &key, &data, 0) != 0 ||
+	if ((sp->db_error = db->get(db, NULL, &key, &data, 0)) != 0 ||
 	    data.size != sizeof(VMC) - 1 ||
 	    memcmp(data.data, VMC, sizeof(VMC) - 1)) {
-		(void)db->close(db);
+		(void)db->close(db, DB_NOSYNC);
 		if (first) {
 			first = 0;
 			return (1);
 		}
-		msgq_str(sp, M_ERR, p,
+		msgq_str(sp, M_DBERR, p,
 		    "030|The file %s is not a message catalog");
 		return (1);
 	}
 	first = 0;
 
 	if (sp->gp->msg != NULL)
-		(void)sp->gp->msg->close(sp->gp->msg);
+		(void)sp->gp->msg->close(sp->gp->msg, DB_NOSYNC);
 	sp->gp->msg = db;
 	return (0);
 }
@@ -723,11 +772,12 @@ msg_open(sp, file)
  * PUBLIC: void msg_close __P((GS *));
  */
 void
-msg_close(gp)
-	GS *gp;
+msg_close(GS *gp)
 {
-	if (gp->msg != NULL)
-		(void)gp->msg->close(gp->msg);
+	if (gp->msg != NULL) {
+		(void)gp->msg->close(gp->msg, 0);
+		gp->msg = NULL;
+	}
 }
 
 /*
@@ -737,10 +787,7 @@ msg_close(gp)
  * PUBLIC: const char *msg_cmsg __P((SCR *, cmsg_t, size_t *));
  */
 const char *
-msg_cmsg(sp, which, lenp)
-	SCR *sp;
-	cmsg_t which;
-	size_t *lenp;
+msg_cmsg(SCR *sp, cmsg_t which, size_t *lenp)
 {
 	switch (which) {
 	case CMSG_CONF:
@@ -775,14 +822,11 @@ msg_cmsg(sp, which, lenp)
  * PUBLIC: const char *msg_cat __P((SCR *, const char *, size_t *));
  */
 const char *
-msg_cat(sp, str, lenp)
-	SCR *sp;
-	const char *str;
-	size_t *lenp;
+msg_cat(SCR *sp, const char *str, size_t *lenp)
 {
 	GS *gp;
 	DBT data, key;
-	recno_t msgno;
+	db_recno_t msgno;
 
 	/*
 	 * If it's not a catalog message, i.e. has doesn't have a leading
@@ -790,8 +834,10 @@ msg_cat(sp, str, lenp)
 	 */
 	if (isdigit(str[0]) &&
 	    isdigit(str[1]) && isdigit(str[2]) && str[3] == '|') {
+		memset(&key, 0, sizeof(key));
 		key.data = &msgno;
-		key.size = sizeof(recno_t);
+		key.size = sizeof(db_recno_t);
+		memset(&data, 0, sizeof(data));
 		msgno = atoi(str);
 
 		/*
@@ -804,7 +850,7 @@ msg_cat(sp, str, lenp)
 		 */
 		gp = sp == NULL ? NULL : sp->gp;
 		if (gp != NULL && gp->msg != NULL &&
-		    gp->msg->get(gp->msg, &key, &data, 0) == 0 &&
+		    gp->msg->get(gp->msg, NULL, &key, &data, 0) == 0 &&
 		    data.size != 0) {
 			if (lenp != NULL)
 				*lenp = data.size - 1;
@@ -825,10 +871,7 @@ msg_cat(sp, str, lenp)
  * PUBLIC: char *msg_print __P((SCR *, const char *, int *));
  */
 char *
-msg_print(sp, s, needfree)
-	SCR *sp;
-	const char *s;
-	int *needfree;
+msg_print(SCR *sp, const char *s, int *needfree)
 {
 	size_t blen, nlen;
 	const char *cp;
@@ -848,14 +891,14 @@ retry:		if (sp == NULL)
 			free(bp);
 		else
 			FREE_SPACE(sp, bp, blen);
-		needfree = 0;
+		*needfree = 0;
 	}
 	nlen += 256;
 	if (sp == NULL) {
 		if ((bp = malloc(nlen)) == NULL)
 			goto alloc_err;
 	} else
-		GET_SPACE_GOTO(sp, bp, blen, nlen);
+		GET_SPACE_GOTOC(sp, bp, blen, nlen);
 	if (0) {
 alloc_err:	return ("");
 	}
