@@ -10,7 +10,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)put.c	10.10 (Berkeley) 9/15/96";
+static const char sccsid[] = "$Id: put.c,v 10.18 2001/06/25 15:19:11 skimo Exp $ (Berkeley) $Date: 2001/06/25 15:19:11 $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -32,23 +32,19 @@ static const char sccsid[] = "@(#)put.c	10.10 (Berkeley) 9/15/96";
  * PUBLIC: int put __P((SCR *, CB *, CHAR_T *, MARK *, MARK *, int));
  */
 int
-put(sp, cbp, namep, cp, rp, append)
-	SCR *sp;
-	CB *cbp;
-	CHAR_T *namep;
-	MARK *cp, *rp;
-	int append;
+put(SCR *sp, CB *cbp, CHAR_T *namep, MARK *cp, MARK *rp, int append)
 {
 	CHAR_T name;
 	TEXT *ltp, *tp;
-	recno_t lno;
+	db_recno_t lno;
 	size_t blen, clen, len;
 	int rval;
-	char *bp, *p, *t;
+	CHAR_T *bp, *t;
+	CHAR_T *p;
 
 	if (cbp == NULL)
 		if (namep == NULL) {
-			cbp = sp->gp->dcbp;
+			cbp = sp->wp->dcbp;
 			if (cbp == NULL) {
 				msgq(sp, M_ERR,
 				    "053|The default buffer is empty");
@@ -120,22 +116,24 @@ put(sp, cbp, namep, cp, rp, append)
 	if (db_get(sp, lno, DBG_FATAL, &p, &len))
 		return (1);
 
-	GET_SPACE_RET(sp, bp, blen, tp->len + len + 1);
+	GET_SPACE_RETW(sp, bp, blen, tp->len + len + 1);
 	t = bp;
 
 	/* Original line, left of the split. */
 	if (len > 0 && (clen = cp->cno + (append ? 1 : 0)) > 0) {
-		memcpy(bp, p, clen);
+		MEMCPYW(bp, p, clen);
 		p += clen;
 		t += clen;
 	}
 
 	/* First line from the CB. */
-	memcpy(t, tp->lb, tp->len);
-	t += tp->len;
+	if (tp->len != 0) {
+		MEMCPYW(t, tp->lb, tp->len);
+		t += tp->len;
+	}
 
 	/* Calculate length left in the original line. */
-	clen = len != 0 ? len - cp->cno - (append ? 1 : 0) : 0;
+	clen = len == 0 ? 0 : len - (cp->cno + (append ? 1 : 0));
 
 	/*
 	 * !!!
@@ -151,7 +149,7 @@ put(sp, cbp, namep, cp, rp, append)
 	 * behavior, and expect POSIX.2 to do so as well.
 	 */
 	rp->lno = lno;
-	rp->cno = len == 0 ? 0 : sp->cno + (append ? 1 : 0);
+	rp->cno = len == 0 ? 0 : sp->cno + (append && tp->len ? 1 : 0);
 
 	/*
 	 * If no more lines in the CB, append the rest of the original
@@ -161,7 +159,7 @@ put(sp, cbp, namep, cp, rp, append)
 	 */
 	if (tp->q.cqe_next == (void *)&cbp->textq) {
 		if (clen > 0) {
-			memcpy(t, p, clen);
+			MEMCPYW(t, p, clen);
 			t += clen;
 		}
 		if (db_set(sp, lno, bp, t - bp))
@@ -183,13 +181,13 @@ put(sp, cbp, namep, cp, rp, append)
 		 */
 		ltp = cbp->textq.cqh_last;
 		len = t - bp;
-		ADD_SPACE_RET(sp, bp, blen, ltp->len + clen);
+		ADD_SPACE_RETW(sp, bp, blen, ltp->len + clen);
 		t = bp + len;
 
 		/* Add in last part of the CB. */
-		memcpy(t, ltp->lb, ltp->len);
+		MEMCPYW(t, ltp->lb, ltp->len);
 		if (clen)
-			memcpy(t + ltp->len, p, clen);
+			MEMCPYW(t + ltp->len, p, clen);
 		clen += ltp->len;
 
 		/*
@@ -224,6 +222,6 @@ put(sp, cbp, namep, cp, rp, append)
 	if (0)
 err:		rval = 1;
 
-	FREE_SPACE(sp, bp, blen);
+	FREE_SPACEW(sp, bp, blen);
 	return (rval);
 }
