@@ -10,7 +10,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)recover.c	10.21 (Berkeley) 9/15/96";
+static const char sccsid[] = "$Id: recover.c,v 10.31 2001/11/01 15:24:44 skimo Exp $ (Berkeley) $Date: 2001/11/01 15:24:44 $";
 #endif /* not lint */
 
 #include <sys/param.h>
@@ -122,14 +122,11 @@ static int	 rcv_mktemp __P((SCR *, char *, char *, int));
  * PUBLIC: int rcv_tmp __P((SCR *, EXF *, char *));
  */
 int
-rcv_tmp(sp, ep, name)
-	SCR *sp;
-	EXF *ep;
-	char *name;
+rcv_tmp(SCR *sp, EXF *ep, char *name)
 {
 	struct stat sb;
 	int fd;
-	char *dp, *p, path[MAXPATHLEN];
+	char *dp, path[MAXPATHLEN];
 
 	/*
 	 * !!!
@@ -154,12 +151,11 @@ rcv_tmp(sp, ep, name)
 	}
 
 	/* Newlines delimit the mail messages. */
-	for (p = name; *p; ++p)
-		if (*p == '\n') {
-			msgq(sp, M_ERR,
+	if (strchr(name, '\n')) {
+		msgq(sp, M_ERR,
 		    "055|Files with newlines in the name are unrecoverable");
-			goto err;
-		}
+		goto err;
+	}
 
 	(void)snprintf(path, sizeof(path), "%s/vi.XXXXXX", dp);
 	if ((fd = rcv_mktemp(sp, path, dp, S_IRWXU)) == -1)
@@ -186,11 +182,10 @@ err:		msgq(sp, M_ERR,
  * PUBLIC: int rcv_init __P((SCR *));
  */
 int
-rcv_init(sp)
-	SCR *sp;
+rcv_init(SCR *sp)
 {
 	EXF *ep;
-	recno_t lno;
+	db_recno_t lno;
 
 	ep = sp->ep;
 
@@ -217,7 +212,7 @@ rcv_init(sp)
 		/* Turn on a busy message, and sync it to backing store. */
 		sp->gp->scr_busy(sp,
 		    "057|Copying file for recovery...", BUSY_ON);
-		if (ep->db->sync(ep->db, R_RECNOSYNC)) {
+		if (ep->db->sync(ep->db, 0)) {
 			msgq_str(sp, M_SYSERR, ep->rcv_path,
 			    "058|Preservation failed: %s");
 			sp->gp->scr_busy(sp, NULL, BUSY_OFF);
@@ -249,9 +244,7 @@ err:	msgq(sp, M_ERR,
  * PUBLIC: int rcv_sync __P((SCR *, u_int));
  */
 int
-rcv_sync(sp, flags)
-	SCR *sp;
-	u_int flags;
+rcv_sync(SCR *sp, u_int flags)
 {
 	EXF *ep;
 	int fd, rval;
@@ -264,15 +257,12 @@ rcv_sync(sp, flags)
 
 	/* Sync the file if it's been modified. */
 	if (F_ISSET(ep, F_MODIFIED)) {
-		SIGBLOCK;
-		if (ep->db->sync(ep->db, R_RECNOSYNC)) {
+		if (ep->db->sync(ep->db, 0)) {
 			F_CLR(ep, F_RCV_ON | F_RCV_NORM);
 			msgq_str(sp, M_SYSERR,
 			    ep->rcv_path, "060|File backup failed: %s");
-			SIGUNBLOCK;
 			return (1);
 		}
-		SIGUNBLOCK;
 
 		/* REQUEST: don't remove backing file on exit. */
 		if (LF_ISSET(RCV_PRESERVE))
@@ -327,10 +317,7 @@ err:		rval = 1;
  *	Build the file to mail to the user.
  */
 static int
-rcv_mailfile(sp, issync, cp_path)
-	SCR *sp;
-	int issync;
-	char *cp_path;
+rcv_mailfile(SCR *sp, int issync, char *cp_path)
 {
 	EXF *ep;
 	GS *gp;
@@ -488,8 +475,7 @@ err:	if (!issync)
  * PUBLIC: int rcv_list __P((SCR *));
  */
 int
-rcv_list(sp)
-	SCR *sp;
+rcv_list(SCR *sp)
 {
 	struct dirent *dp;
 	struct stat sb;
@@ -591,9 +577,7 @@ next:		(void)fclose(fp);
  * PUBLIC: int rcv_read __P((SCR *, FREF *));
  */
 int
-rcv_read(sp, frp)
-	SCR *sp;
-	FREF *frp;
+rcv_read(SCR *sp, FREF *frp)
 {
 	struct dirent *dp;
 	struct stat sb;
@@ -777,10 +761,7 @@ next:			(void)close(fd);
  *	Copy a recovery file.
  */
 static int
-rcv_copy(sp, wfd, fname)
-	SCR *sp;
-	int wfd;
-	char *fname;
+rcv_copy(SCR *sp, int wfd, char *fname)
 {
 	int nr, nw, off, rfd;
 	char buf[8 * 1024];
@@ -803,10 +784,7 @@ err:	msgq_str(sp, M_SYSERR, fname, "%s");
  *	Fgets(3) for a file descriptor.
  */
 static char *
-rcv_gets(buf, len, fd)
-	char *buf;
-	size_t len;
-	int fd;
+rcv_gets(char *buf, size_t len, int fd)
 {
 	int nr;
 	char *p;
@@ -824,10 +802,7 @@ rcv_gets(buf, len, fd)
  *	Paranoid make temporary file routine.
  */
 static int
-rcv_mktemp(sp, path, dname, perms)
-	SCR *sp;
-	char *path, *dname;
-	int perms;
+rcv_mktemp(SCR *sp, char *path, char *dname, int perms)
 {
 	int fd;
 
@@ -853,9 +828,7 @@ rcv_mktemp(sp, path, dname, perms)
  *	Send email.
  */
 static void
-rcv_email(sp, fname)
-	SCR *sp;
-	char *fname;
+rcv_email(SCR *sp, char *fname)
 {
 	struct stat sb;
 	char buf[MAXPATHLEN * 2 + 20];
