@@ -10,7 +10,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)util.c	10.10 (Berkeley) 3/6/96";
+static const char sccsid[] = "$Id: util.c,v 10.22 2001/06/25 15:19:12 skimo Exp $ (Berkeley) $Date: 2001/06/25 15:19:12 $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -33,10 +33,10 @@ static const char sccsid[] = "@(#)util.c	10.10 (Berkeley) 3/6/96";
  * PUBLIC: void *binc __P((SCR *, void *, size_t *, size_t));
  */
 void *
-binc(sp, bp, bsizep, min)
-	SCR *sp;			/* sp MAY BE NULL!!! */
-	void *bp;
-	size_t *bsizep, min;
+binc(SCR *sp, void *bp, size_t *bsizep, size_t min)
+	        			/* sp MAY BE NULL!!! */
+	         
+	                    
 {
 	size_t csize;
 
@@ -70,15 +70,12 @@ binc(sp, bp, bsizep, min)
  *	including or after the starting column.  On error, set
  *	the column to 0, it's safest.
  *
- * PUBLIC: int nonblank __P((SCR *, recno_t, size_t *));
+ * PUBLIC: int nonblank __P((SCR *, db_recno_t, size_t *));
  */
 int
-nonblank(sp, lno, cnop)
-	SCR *sp;
-	recno_t lno;
-	size_t *cnop;
+nonblank(SCR *sp, db_recno_t lno, size_t *cnop)
 {
-	char *p;
+	CHAR_T *p;
 	size_t cnt, len, off;
 	int isempty;
 
@@ -109,8 +106,7 @@ nonblank(sp, lno, cnop)
  * PUBLIC: char *tail __P((char *));
  */
 char *
-tail(path)
-	char *path;
+tail(char *path)
 {
 	char *p;
 
@@ -123,20 +119,36 @@ tail(path)
  * v_strdup --
  *	Strdup for wide character strings with an associated length.
  *
- * PUBLIC: CHAR_T *v_strdup __P((SCR *, const CHAR_T *, size_t));
+ * PUBLIC: char *v_strdup __P((SCR *, const char *, size_t));
+ */
+char *
+v_strdup(SCR *sp, const char *str, size_t len)
+{
+	char *copy;
+
+	MALLOC(sp, copy, char *, (len + 1));
+	if (copy == NULL)
+		return (NULL);
+	memcpy(copy, str, len);
+	copy[len] = '\0';
+	return (copy);
+}
+
+/*
+ * v_strdup --
+ *	Strdup for wide character strings with an associated length.
+ *
+ * PUBLIC: CHAR_T *v_wstrdup __P((SCR *, const CHAR_T *, size_t));
  */
 CHAR_T *
-v_strdup(sp, str, len)
-	SCR *sp;
-	const CHAR_T *str;
-	size_t len;
+v_wstrdup(SCR *sp, const CHAR_T *str, size_t len)
 {
 	CHAR_T *copy;
 
-	MALLOC(sp, copy, CHAR_T *, len + 1);
+	MALLOC(sp, copy, CHAR_T *, (len + 1) * sizeof(CHAR_T));
 	if (copy == NULL)
 		return (NULL);
-	memmove(copy, str, len * sizeof(CHAR_T));
+	MEMCPYW(copy, str, len);
 	copy[len] = '\0';
 	return (copy);
 }
@@ -145,17 +157,13 @@ v_strdup(sp, str, len)
  * nget_uslong --
  *      Get an unsigned long, checking for overflow.
  *
- * PUBLIC: enum nresult nget_uslong __P((u_long *, const char *, char **, int));
+ * PUBLIC: enum nresult nget_uslong __P((SCR *, u_long *, const CHAR_T *, CHAR_T **, int));
  */
 enum nresult
-nget_uslong(valp, p, endp, base)
-	u_long *valp;
-	const char *p;
-	char **endp;
-	int base;
+nget_uslong(SCR *sp, u_long *valp, const CHAR_T *p, CHAR_T **endp, int base)
 {
 	errno = 0;
-	*valp = strtoul(p, endp, base);
+	*valp = STRTOUL(p, endp, base);
 	if (errno == 0)
 		return (NUM_OK);
 	if (errno == ERANGE && *valp == ULONG_MAX)
@@ -167,17 +175,13 @@ nget_uslong(valp, p, endp, base)
  * nget_slong --
  *      Convert a signed long, checking for overflow and underflow.
  *
- * PUBLIC: enum nresult nget_slong __P((long *, const char *, char **, int));
+ * PUBLIC: enum nresult nget_slong __P((SCR *, long *, const CHAR_T *, CHAR_T **, int));
  */
 enum nresult
-nget_slong(valp, p, endp, base)
-	long *valp;
-	const char *p;
-	char **endp;
-	int base;
+nget_slong(SCR *sp, long int *valp, const CHAR_T *p, CHAR_T **endp, int base)
 {
 	errno = 0;
-	*valp = strtol(p, endp, base);
+	*valp = STRTOL(p, endp, base);
 	if (errno == 0)
 		return (NUM_OK);
 	if (errno == ERANGE) {
@@ -188,43 +192,3 @@ nget_slong(valp, p, endp, base)
 	}
 	return (NUM_ERR);
 }
-
-#ifdef DEBUG
-#ifdef __STDC__
-#include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
-
-/*
- * TRACE --
- *	debugging trace routine.
- *
- * PUBLIC: void TRACE __P((SCR *, const char *, ...));
- */
-void
-#ifdef __STDC__
-TRACE(SCR *sp, const char *fmt, ...)
-#else
-TRACE(sp, fmt, va_alist)
-	SCR *sp;
-	char *fmt;
-	va_dcl
-#endif
-{
-	FILE *tfp;
-	va_list ap;
-
-	if ((tfp = sp->gp->tracefp) == NULL)
-		return;
-#ifdef __STDC__
-	va_start(ap, fmt);
-#else
-	va_start(ap);
-#endif
-	(void)vfprintf(tfp, fmt, ap);
-	va_end(ap);
-
-	(void)fflush(tfp);
-}
-#endif
