@@ -10,7 +10,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)ex.c	10.54 (Berkeley) 9/15/96";
+static const char sccsid[] = "$Id: ex.c,v 10.75 2004/03/16 14:13:35 skimo Exp $ (Berkeley) $Date: 2004/03/16 14:13:35 $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -35,11 +35,11 @@ static const char sccsid[] = "@(#)ex.c	10.54 (Berkeley) 9/15/96";
 static void	ex_comlog __P((SCR *, EXCMD *));
 #endif
 static EXCMDLIST const *
-		ex_comm_search __P((char *, size_t));
+		ex_comm_search __P((SCR *, CHAR_T *, size_t));
 static int	ex_discard __P((SCR *));
 static int	ex_line __P((SCR *, EXCMD *, MARK *, int *, int *));
 static int	ex_load __P((SCR *));
-static void	ex_unknown __P((SCR *, char *, size_t));
+static void	ex_unknown __P((SCR *, CHAR_T *, size_t));
 
 /*
  * ex --
@@ -48,17 +48,18 @@ static void	ex_unknown __P((SCR *, char *, size_t));
  * PUBLIC: int ex __P((SCR **));
  */
 int
-ex(spp)
-	SCR **spp;
+ex(SCR **spp)
 {
 	EX_PRIVATE *exp;
 	GS *gp;
+	WIN *wp;
 	MSGS *mp;
 	SCR *sp;
 	TEXT *tp;
 	u_int32_t flags;
 
 	sp = *spp;
+	wp = sp->wp;
 	gp = sp->gp;
 	exp = EXP(sp);
 
@@ -68,16 +69,16 @@ ex(spp)
 
 	/* Flush any saved messages. */
 	while ((mp = gp->msgq.lh_first) != NULL) {
-		gp->scr_msg(sp, mp->mtype, mp->buf, mp->len);
+		wp->scr_msg(sp, mp->mtype, mp->buf, mp->len);
 		LIST_REMOVE(mp, q);
 		free(mp->buf);
 		free(mp);
 	}
 
 	/* If reading from a file, errors should have name and line info. */
-	if (!F_ISSET(gp, G_STDIN_TTY)) {
-		gp->excmd.if_lno = 1;
-		gp->excmd.if_name = "script";
+	if (F_ISSET(gp, G_SCRIPTED)) {
+		wp->excmd.if_lno = 1;
+		wp->excmd.if_name = "script";
 	}
 
 	/*
@@ -88,7 +89,7 @@ ex(spp)
 	 * "^H discarded", that was displayed.  We don't bother.
 	 */
 	LF_INIT(TXT_BACKSLASH | TXT_CNTRLD | TXT_CR);
-	for (;; ++gp->excmd.if_lno) {
+	for (;; ++wp->excmd.if_lno) {
 		/* Display status line and flush. */
 		if (F_ISSET(sp, SC_STATUS)) {
 			if (!F_ISSET(sp, SC_EX_SILENT))
@@ -114,7 +115,7 @@ ex(spp)
 		}
 
 		/* Initialize the command structure. */
-		CLEAR_EX_PARSER(&gp->excmd);
+		CLEAR_EX_PARSER(&wp->excmd);
 
 		/*
 		 * If the user entered a single carriage return, send
@@ -122,15 +123,16 @@ ex(spp)
 		 */
 		tp = sp->tiq.cqh_first;
 		if (tp->len == 0) {
-			gp->excmd.cp = " ";	/* __TK__ why not |? */
-			gp->excmd.clen = 1;
+			static CHAR_T space = ' ';
+			wp->excmd.cp = &space;	/* __TK__ why not |? */
+			wp->excmd.clen = 1;
 		} else {
-			gp->excmd.cp = tp->lb;
-			gp->excmd.clen = tp->len;
+			wp->excmd.cp = tp->lb;
+			wp->excmd.clen = tp->len;
 		}
-		F_INIT(&gp->excmd, E_NRSEP);
+		F_INIT(&wp->excmd, E_NRSEP);
 
-		if (ex_cmd(sp) && !F_ISSET(gp, G_STDIN_TTY))
+		if (ex_cmd(sp) && F_ISSET(gp, G_SCRIPTED))
 			return (1);
 
 		if (INTERRUPTED(sp)) {
@@ -192,24 +194,28 @@ ex(spp)
  * PUBLIC: int ex_cmd __P((SCR *));
  */
 int
-ex_cmd(sp)
-	SCR *sp;
+ex_cmd(SCR *sp)
 {
 	enum nresult nret;
 	EX_PRIVATE *exp;
 	EXCMD *ecp;
 	GS *gp;
+	WIN *wp;
 	MARK cur;
-	recno_t lno;
-	size_t arg1_len, len;
+	db_recno_t lno;
+	size_t arg1_len, discard, len;
 	u_int32_t flags;
 	long ltmp;
 	int at_found, gv_found;
-	int ch, cnt, delim, isaddr, namelen;
+	int cnt, delim, isaddr, namelen;
 	int newscreen, notempty, tmp, vi_address;
-	char *arg1, *p, *s, *t;
+	CHAR_T *arg1, *s, *p, *t;
+	CHAR_T ch;
+	CHAR_T *n;
+	char *np;
 
 	gp = sp->gp;
+	wp = sp->wp;
 	exp = EXP(sp);
 
 	/*
@@ -217,12 +223,12 @@ ex_cmd(sp)
 	 * This means that *everything* must be resolved when we leave
 	 * this function for any reason.
 	 */
-loop:	ecp = gp->ecq.lh_first;
+loop:	ecp = wp->ecq.lh_first;
 
 	/* If we're reading a command from a file, set up error information. */
 	if (ecp->if_name != NULL) {
-		gp->if_lno = ecp->if_lno;
-		gp->if_name = ecp->if_name;
+		wp->if_lno = ecp->if_lno;
+		wp->if_name = ecp->if_name;
 	}
 
 	/*
@@ -238,7 +244,7 @@ loop:	ecp = gp->ecq.lh_first;
 
 	/* If we found a newline, increment the count now. */
 	if (F_ISSET(ecp, E_NEWLINE)) {
-		++gp->if_lno;
+		++wp->if_lno;
 		++ecp->if_lno;
 		F_CLR(ecp, E_NEWLINE);
 	}
@@ -257,9 +263,9 @@ loop:	ecp = gp->ecq.lh_first;
 	/* Skip <blank>s, empty lines.  */
 	for (notempty = 0; ecp->clen > 0; ++ecp->cp, --ecp->clen)
 		if ((ch = *ecp->cp) == '\n') {
-			++gp->if_lno;
+			++wp->if_lno;
 			++ecp->if_lno;
-		} else if (isblank(ch))
+		} else if (ISBLANK(ch))
 			notempty = 1;
 		else
 			break;
@@ -298,7 +304,7 @@ loop:	ecp = gp->ecq.lh_first;
 	/* Skip whitespace. */
 	for (; ecp->clen > 0; ++ecp->cp, --ecp->clen) {
 		ch = *ecp->cp;
-		if (!isblank(ch))
+		if (!ISBLANK(ch))
 			break;
 	}
 
@@ -321,7 +327,7 @@ loop:	ecp = gp->ecq.lh_first;
 	    (!notempty || F_ISSET(sp, SC_VI) || F_ISSET(ecp, E_BLIGNORE))) {
 		if (ex_load(sp))
 			goto rfail;
-		ecp = gp->ecq.lh_first;
+		ecp = wp->ecq.lh_first;
 		if (ecp->clen == 0)
 			goto rsuccess;
 		goto loop;
@@ -353,7 +359,7 @@ loop:	ecp = gp->ecq.lh_first;
 	 */
 	for (; ecp->clen > 0; ++ecp->cp, --ecp->clen) {
 		ch = *ecp->cp;
-		if (!isblank(ch) && ch != ':')
+		if (!ISBLANK(ch) && ch != ':')
 			break;
 	}
 
@@ -388,7 +394,7 @@ loop:	ecp = gp->ecq.lh_first;
 		} else {
 			for (p = ecp->cp;
 			    ecp->clen > 0; --ecp->clen, ++ecp->cp)
-				if (!isalpha(*ecp->cp))
+				if (!ISALPHA(*ecp->cp))
 					break;
 			if ((namelen = ecp->cp - p) == 0) {
 				msgq(sp, M_ERR, "080|Unknown command name");
@@ -415,7 +421,7 @@ loop:	ecp = gp->ecq.lh_first;
 		switch (p[0]) {
 		case 'd':
 			for (s = p,
-			    t = cmds[C_DELETE].name; *s == *t; ++s, ++t);
+			    n = cmds[C_DELETE].name; *s == *n; ++s, ++n);
 			if (s[0] == 'l' || s[0] == 'p' || s[0] == '+' ||
 			    s[0] == '-' || s[0] == '^' || s[0] == '#') {
 				len = (ecp->cp - p) - (s - p);
@@ -451,10 +457,10 @@ loop:	ecp = gp->ecq.lh_first;
 		 * follows the 's', but we limit the choices here to "cgr" so
 		 * that we get unknown command messages for wrong combinations.
 		 */
-		if ((ecp->cmd = ex_comm_search(p, namelen)) == NULL)
+		if ((ecp->cmd = ex_comm_search(sp, p, namelen)) == NULL)
 			switch (p[0]) {
 			case 'k':
-				if (p[1] && !p[2]) {
+				if (namelen == 2) {
 					ecp->cp -= namelen - 1;
 					ecp->clen += namelen - 1;
 					ecp->cmd = &cmds[C_K];
@@ -510,7 +516,7 @@ skip_srch:	if (ecp->cmd == &cmds[C_VISUAL_EX] && F_ISSET(sp, SC_VI))
 
 		/* Secure means no shell access. */
 		if (F_ISSET(ecp->cmd, E_SECURE) && O_ISSET(sp, O_SECURE)) {
-			ex_emsg(sp, ecp->cmd->name, EXM_SECURE);
+			ex_wemsg(sp, ecp->cmd->name, EXM_SECURE);
 			goto err;
 		}
 
@@ -625,10 +631,12 @@ skip_srch:	if (ecp->cmd == &cmds[C_VISUAL_EX] && F_ISSET(sp, SC_VI))
 	 * special cases we move past their special argument(s).  Then, we
 	 * do normal command processing on whatever is left.  Barf-O-Rama.
 	 */
+	discard = 0;		/* Characters discarded from the command. */
 	arg1_len = 0;
 	ecp->save_cmd = ecp->cp;
 	if (ecp->cmd == &cmds[C_EDIT] || ecp->cmd == &cmds[C_EX] ||
-	    ecp->cmd == &cmds[C_NEXT] || ecp->cmd == &cmds[C_VISUAL_VI]) {
+	    ecp->cmd == &cmds[C_NEXT] || ecp->cmd == &cmds[C_VISUAL_VI] ||
+	    ecp->cmd == &cmds[C_VSPLIT]) {
 		/*
 		 * Move to the next non-whitespace character.  A '!'
 		 * immediately following the command is eaten as a
@@ -643,7 +651,7 @@ skip_srch:	if (ecp->cmd == &cmds[C_VISUAL_EX] && F_ISSET(sp, SC_VI))
 			ecp->save_cmd = ecp->cp;
 		}
 		for (; ecp->clen > 0; --ecp->clen, ++ecp->cp)
-			if (!isblank(*ecp->cp))
+			if (!ISBLANK(*ecp->cp))
 				break;
 		/*
 		 * QUOTING NOTE:
@@ -662,9 +670,10 @@ skip_srch:	if (ecp->cmd == &cmds[C_VISUAL_EX] && F_ISSET(sp, SC_VI))
 				ch = *ecp->cp;
 				if (IS_ESCAPE(sp, ecp, ch) &&
 				    ecp->clen > 1) {
+					++discard;
 					--ecp->clen;
 					ch = *++ecp->cp;
-				} else if (isblank(ch))
+				} else if (ISBLANK(ch))
 					break;
 				*p++ = ch;
 			}
@@ -675,9 +684,28 @@ skip_srch:	if (ecp->cmd == &cmds[C_VISUAL_EX] && F_ISSET(sp, SC_VI))
 		}
 	} else if (ecp->cmd == &cmds[C_BANG] ||
 	    ecp->cmd == &cmds[C_GLOBAL] || ecp->cmd == &cmds[C_V]) {
-		for (; ecp->clen > 0; --ecp->clen, ++ecp->cp)
-			if (ecp->cp[0] == '\n')
+		/*
+		 * QUOTING NOTE:
+		 *
+		 * We use backslashes to escape <newline> characters, although
+		 * this wasn't historic practice for the bang command.  It was
+		 * for the global and v commands, and it's common usage when
+		 * doing text insert during the command.  Escaping characters
+		 * are stripped as no longer useful.
+		 */
+		for (p = ecp->cp; ecp->clen > 0; --ecp->clen, ++ecp->cp) {
+			ch = *ecp->cp;
+			if (ch == '\\' && ecp->clen > 1 && ecp->cp[1] == '\n') {
+				++discard;
+				--ecp->clen;
+				ch = *++ecp->cp;
+
+				++wp->if_lno;
+				++ecp->if_lno;
+			} else if (ch == '\n')
 				break;
+			*p++ = ch;
+		}
 	} else if (ecp->cmd == &cmds[C_READ] || ecp->cmd == &cmds[C_WRITE]) {
 		/*
 		 * For write commands, if the next character is a <blank>, and
@@ -689,7 +717,7 @@ skip_srch:	if (ecp->cmd == &cmds[C_VISUAL_EX] && F_ISSET(sp, SC_VI))
 		 */
 		for (tmp = 0; ecp->clen > 0; --ecp->clen, ++ecp->cp) {
 			ch = *ecp->cp;
-			if (isblank(ch))
+			if (ISBLANK(ch))
 				tmp = 1;
 			else
 				break;
@@ -707,10 +735,10 @@ skip_srch:	if (ecp->cmd == &cmds[C_VISUAL_EX] && F_ISSET(sp, SC_VI))
 		 * into something like ":s g", so use the special s command.
 		 */
 		for (; ecp->clen > 0; --ecp->clen, ++ecp->cp)
-			if (!isblank(ecp->cp[0]))
+			if (!ISBLANK(ecp->cp[0]))
 				break;
 
-		if (isalnum(ecp->cp[0]) || ecp->cp[0] == '|') {
+		if (ISALNUM(ecp->cp[0]) || ecp->cp[0] == '|') {
 			ecp->rcmd = cmds[C_SUBSTITUTE];
 			ecp->rcmd.fn = ex_subagain;
 			ecp->cmd = &ecp->rcmd;
@@ -751,18 +779,18 @@ skip_srch:	if (ecp->cmd == &cmds[C_VISUAL_EX] && F_ISSET(sp, SC_VI))
 	 * no longer useful.
 	 */
 	vi_address = ecp->clen != 0 && ecp->cp[0] != '\n';
-	for (cnt = 0, p = ecp->cp; ecp->clen > 0; --ecp->clen, ++ecp->cp) {
+	for (p = ecp->cp; ecp->clen > 0; --ecp->clen, ++ecp->cp) {
 		ch = ecp->cp[0];
 		if (IS_ESCAPE(sp, ecp, ch) && ecp->clen > 1) {
-			tmp = ecp->cp[1];
+			CHAR_T tmp = ecp->cp[1];
 			if (tmp == '\n' || tmp == '|') {
 				if (tmp == '\n') {
-					++gp->if_lno;
+					++wp->if_lno;
 					++ecp->if_lno;
 				}
+				++discard;
 				--ecp->clen;
 				++ecp->cp;
-				++cnt;
 				ch = tmp;
 			}
 		} else if (ch == '\n' || ch == '|') {
@@ -782,7 +810,7 @@ skip_srch:	if (ecp->cmd == &cmds[C_VISUAL_EX] && F_ISSET(sp, SC_VI))
 	ecp->cp = ecp->save_cmd;
 	ecp->save_cmd = p;
 	ecp->save_cmdlen = ecp->clen;
-	ecp->clen = ((ecp->save_cmd - ecp->cp) - 1) - cnt;
+	ecp->clen = ((ecp->save_cmd - ecp->cp) - 1) - discard;
 
 	/*
 	 * QUOTING NOTE:
@@ -884,7 +912,7 @@ two_addr:	switch (ecp->addrcnt) {
 			ecp->addr1.cno = ecp->addr2.cno = sp->cno;
 			break;
 		case 1:				/* Default to first address. */
-			ecp->addrcnt = 2;
+			//ecp->addrcnt = 2;	/* XXX Was this needed ??? */
 			ecp->addr2 = ecp->addr1;
 			break;
 		case 2:
@@ -914,13 +942,13 @@ two_addr:	switch (ecp->addrcnt) {
 	}
 
 	ecp->flagoff = 0;
-	for (p = ecp->cmd->syntax; *p != '\0'; ++p) {
+	for (np = ecp->cmd->syntax; *np != '\0'; ++np) {
 		/*
 		 * The force flag is sensitive to leading whitespace, i.e.
 		 * "next !" is different from "next!".  Handle it before
 		 * skipping leading <blank>s.
 		 */
-		if (*p == '!') {
+		if (*np == '!') {
 			if (ecp->clen > 0 && *ecp->cp == '!') {
 				++ecp->cp;
 				--ecp->clen;
@@ -931,12 +959,12 @@ two_addr:	switch (ecp->addrcnt) {
 
 		/* Skip leading <blank>s. */
 		for (; ecp->clen > 0; --ecp->clen, ++ecp->cp)
-			if (!isblank(*ecp->cp))
+			if (!ISBLANK(*ecp->cp))
 				break;
 		if (ecp->clen == 0)
 			break;
 
-		switch (*p) {
+		switch (*np) {
 		case '1':				/* +, -, #, l, p */
 			/*
 			 * !!!
@@ -991,7 +1019,7 @@ end_case1:		break;
 					FL_SET(ecp->iflags, E_C_CARAT);
 					break;
 				case '=':
-					if (*p == '3') {
+					if (*np == '3') {
 						FL_SET(ecp->iflags, E_C_EQUAL);
 						break;
 					}
@@ -1011,7 +1039,7 @@ end_case23:		break;
 			 */
 			if ((ecp->cp[0] == '+' || ecp->cp[0] == '-' ||
 			    ecp->cp[0] == '^' || ecp->cp[0] == '#') &&
-			    strchr(p, '1') != NULL)
+			    strchr(np, '1') != NULL)
 				break;
 			/*
 			 * !!!
@@ -1019,7 +1047,7 @@ end_case23:		break;
 			 * command "d2" would be a delete into buffer '2', and
 			 * not a two-line deletion.
 			 */
-			if (!isdigit(ecp->cp[0])) {
+			if (!ISDIGIT(ecp->cp[0])) {
 				ecp->buffer = *ecp->cp;
 				++ecp->cp;
 				--ecp->clen;
@@ -1027,9 +1055,9 @@ end_case23:		break;
 			}
 			break;
 		case 'c':				/* count [01+a] */
-			++p;
+			++np;
 			/* Validate any signed value. */
-			if (!isdigit(*ecp->cp) && (*p != '+' ||
+			if (!ISDIGIT(*ecp->cp) && (*np != '+' ||
 			    (*ecp->cp != '+' && *ecp->cp != '-')))
 				break;
 			/* If a signed value, set appropriate flags. */
@@ -1038,11 +1066,11 @@ end_case23:		break;
 			else if (*ecp->cp == '+')
 				FL_SET(ecp->iflags, E_C_COUNT_POS);
 			if ((nret =
-			    nget_slong(&ltmp, ecp->cp, &t, 10)) != NUM_OK) {
+			    nget_slong(sp, &ltmp, ecp->cp, &t, 10)) != NUM_OK) {
 				ex_badaddr(sp, NULL, A_NOTSET, nret);
 				goto err;
 			}
-			if (ltmp == 0 && *p != '0') {
+			if (ltmp == 0 && *np != '0') {
 				msgq(sp, M_ERR, "083|Count may not be zero");
 				goto err;
 			}
@@ -1058,7 +1086,7 @@ end_case23:		break;
 			 * join) do different things with counts than with
 			 * line addresses.
 			 */
-			if (*p == 'a') {
+			if (*np == 'a') {
 				ecp->addr1 = ecp->addr2;
 				ecp->addr2.lno = ecp->addr1.lno + ltmp - 1;
 			} else
@@ -1085,7 +1113,7 @@ end_case23:		break;
 
 			/* Line specifications are always required. */
 			if (!isaddr) {
-				msgq_str(sp, M_ERR, ecp->cp,
+				msgq_wstr(sp, M_ERR, ecp->cp,
 				     "084|%s: bad line specification");
 				goto err;
 			}
@@ -1128,7 +1156,7 @@ end_case23:		break;
 				    ecp, ch) && ecp->clen > 1) {
 					--ecp->clen;
 					*p++ = *++ecp->cp;
-				} else if (isblank(ch)) {
+				} else if (ISBLANK(ch)) {
 					++ecp->cp;
 					--ecp->clen;
 					break;
@@ -1142,7 +1170,7 @@ end_case23:		break;
 			for (; ecp->clen > 0;
 			    --ecp->clen, ++ecp->cp) {
 				ch = *ecp->cp;
-				if (!isblank(ch))
+				if (!ISBLANK(ch))
 					break;
 			}
 			if (ecp->clen == 0)
@@ -1165,14 +1193,14 @@ end_case23:		break;
 		case 'w':				/* word */
 			if (argv_exp3(sp, ecp, ecp->cp, ecp->clen))
 				goto err;
-arg_cnt_chk:		if (*++p != 'N') {		/* N */
+arg_cnt_chk:		if (*++np != 'N') {		/* N */
 				/*
 				 * If a number is specified, must either be
 				 * 0 or that number, if optional, and that
 				 * number, if required.
 				 */
-				tmp = *p - '0';
-				if ((*++p != 'o' || exp->argsoff != 0) &&
+				tmp = *np - '0';
+				if ((*++np != 'o' || exp->argsoff != 0) &&
 				    exp->argsoff != tmp)
 					goto usage;
 			}
@@ -1180,14 +1208,14 @@ arg_cnt_chk:		if (*++p != 'N') {		/* N */
 		default:
 			msgq(sp, M_ERR,
 			    "085|Internal syntax table error (%s: %s)",
-			    ecp->cmd->name, KEY_NAME(sp, *p));
+			    ecp->cmd->name, KEY_NAME(sp, *np));
 		}
 	}
 
 	/* Skip trailing whitespace. */
 	for (; ecp->clen > 0; --ecp->clen) {
 		ch = *ecp->cp++;
-		if (!isblank(ch))
+		if (!ISBLANK(ch))
 			break;
 	}
 
@@ -1195,7 +1223,7 @@ arg_cnt_chk:		if (*++p != 'N') {		/* N */
 	 * There shouldn't be anything left, and no more required fields,
 	 * i.e neither 'l' or 'r' in the syntax string.
 	 */
-	if (ecp->clen != 0 || strpbrk(p, "lr")) {
+	if (ecp->clen != 0 || strpbrk(np, "lr")) {
 usage:		msgq(sp, M_ERR, "086|Usage: %s", ecp->cmd->usage);
 		goto err;
 	}
@@ -1227,7 +1255,7 @@ addr_verify:
 				ex_badaddr(sp, ecp->cmd, A_ZERO, NUM_OK);
 				goto err;
 			}
-		} else if (!db_exist(sp, ecp->addr2.lno))
+		} else if (!db_exist(sp, ecp->addr2.lno)) {
 			if (FL_ISSET(ecp->iflags, E_C_COUNT)) {
 				if (db_last(sp, &lno))
 					goto err;
@@ -1236,6 +1264,7 @@ addr_verify:
 				ex_badaddr(sp, NULL, A_EOF, NUM_OK);
 				goto err;
 			}
+		}
 		/* FALLTHROUGH */
 	case 1:
 		if (ecp->addr1.lno == 0) {
@@ -1336,7 +1365,7 @@ addr_verify:
 	 */
 	if (F_ISSET(ecp, E_NRSEP)) {
 		if (sp->ep != NULL &&
-		    F_ISSET(sp, SC_EX) && F_ISSET(gp, G_STDIN_TTY) &&
+		    F_ISSET(sp, SC_EX) && !F_ISSET(gp, G_SCRIPTED) &&
 		    (F_ISSET(ecp, E_USELASTCMD) || ecp->cmd == &cmds[C_SCROLL]))
 			gp->scr_ex_adjust(sp, EX_TERM_SCROLL);
 		F_CLR(ecp, E_NRSEP);
@@ -1349,15 +1378,15 @@ addr_verify:
 	 * Interrupts behave like errors, for now.
 	 */
 	if (ecp->cmd->fn(sp, ecp) || INTERRUPTED(sp)) {
-		if (!F_ISSET(gp, G_STDIN_TTY))
+		if (F_ISSET(gp, G_SCRIPTED))
 			F_SET(sp, SC_EXIT_FORCE);
 		goto err;
 	}
 
 #ifdef DEBUG
 	/* Make sure no function left global temporary space locked. */
-	if (F_ISSET(gp, G_TMP_INUSE)) {
-		F_CLR(gp, G_TMP_INUSE);
+	if (F_ISSET(wp, W_TMP_INUSE)) {
+		F_CLR(wp, W_TMP_INUSE);
 		msgq(sp, M_ERR, "087|%s: temporary buffer not released",
 		    ecp->cmd->name);
 	}
@@ -1395,7 +1424,7 @@ addr_verify:
 				goto err;
 			}
 		} else {
-			if (!NPFITS(MAX_REC_NUMBER, sp->lno, ecp->flagoff)) {
+			if (!NPFITS(DB_MAX_RECORDS, sp->lno, ecp->flagoff)) {
 				ex_badaddr(sp, NULL, A_NOTSET, NUM_OVER);
 				goto err;
 			}
@@ -1472,7 +1501,7 @@ addr_verify:
 
 		ecp->save_cmd -= arg1_len;
 		ecp->save_cmdlen += arg1_len;
-		memcpy(ecp->save_cmd, arg1, arg1_len);
+		MEMCPYW(ecp->save_cmd, arg1, arg1_len);
 
 		/*
 		 * Any commands executed from a +cmd are executed starting at
@@ -1506,7 +1535,7 @@ addr_verify:
 	 */
 	if (F_ISSET(sp, SC_EXIT | SC_EXIT_FORCE | SC_FSWITCH | SC_SSWITCH)) {
 		at_found = gv_found = 0;
-		for (ecp = sp->gp->ecq.lh_first;
+		for (ecp = wp->ecq.lh_first;
 		    ecp != NULL; ecp = ecp->q.le_next)
 			switch (ecp->agv_flags) {
 			case 0:
@@ -1559,7 +1588,7 @@ err:	/*
 				break;
 			}
 		}
-	if (ecp->save_cmdlen != 0 || gp->ecq.lh_first != &gp->excmd) {
+	if (ecp->save_cmdlen != 0 || wp->ecq.lh_first != &wp->excmd) {
 discard:	msgq(sp, M_BERR,
 		    "092|Ex command failed: pending commands discarded");
 		ex_discard(sp);
@@ -1573,7 +1602,7 @@ rfail:	tmp = 1;
 rsuccess:	tmp = 0;
 
 	/* Turn off any file name error information. */
-	gp->if_name = NULL;
+	wp->if_name = NULL;
 
 	/* Turn off the global bit. */
 	F_CLR(sp, SC_EX_GLOBAL);
@@ -1588,10 +1617,7 @@ rsuccess:	tmp = 0;
  * PUBLIC: int ex_range __P((SCR *, EXCMD *, int *));
  */
 int
-ex_range(sp, ecp, errp)
-	SCR *sp;
-	EXCMD *ecp;
-	int *errp;
+ex_range(SCR *sp, EXCMD *ecp, int *errp)
 {
 	enum { ADDR_FOUND, ADDR_NEED, ADDR_NONE } addr;
 	GS *gp;
@@ -1790,19 +1816,15 @@ ret:	if (F_ISSET(ecp, E_VISEARCH))
  * it's fairly close.
  */
 static int
-ex_line(sp, ecp, mp, isaddrp, errp)
-	SCR *sp;
-	EXCMD *ecp;
-	MARK *mp;
-	int *isaddrp, *errp;
+ex_line(SCR *sp, EXCMD *ecp, MARK *mp, int *isaddrp, int *errp)
 {
 	enum nresult nret;
 	EX_PRIVATE *exp;
 	GS *gp;
 	long total, val;
 	int isneg;
-	int (*sf) __P((SCR *, MARK *, MARK *, char *, size_t, char **, u_int));
-	char *endp;
+	int (*sf) __P((SCR *, MARK *, MARK *, CHAR_T *, size_t, CHAR_T **, u_int));
+	CHAR_T *endp;
 
 	gp = sp->gp;
 	exp = EXP(sp);
@@ -1833,12 +1855,12 @@ ex_line(sp, ecp, mp, isaddrp, errp)
 		*isaddrp = 1;
 		F_SET(ecp, E_ABSMARK);
 
-		if ((nret = nget_slong(&val, ecp->cp, &endp, 10)) != NUM_OK) {
+		if ((nret = nget_slong(sp, &val, ecp->cp, &endp, 10)) != NUM_OK) {
 			ex_badaddr(sp, NULL, A_NOTSET, nret);
 			*errp = 1;
 			return (0);
 		}
-		if (!NPFITS(MAX_REC_NUMBER, 0, val)) {
+		if (!NPFITS(DB_MAX_RECORDS, 0, val)) {
 			ex_badaddr(sp, NULL, A_NOTSET, NUM_OVER);
 			*errp = 1;
 			return (0);
@@ -1922,7 +1944,7 @@ search:		mp->lno = sp->lno;
 		 * the '+' could be omitted.  (This feature is found in ed
 		 * as well.)
 		 */
-		if (ecp->clen > 1 && isdigit(ecp->cp[1]))
+		if (ecp->clen > 1 && ISDIGIT(ecp->cp[1]))
 			*ecp->cp = '+';
 		else {
 			++ecp->cp;
@@ -1933,14 +1955,14 @@ search:		mp->lno = sp->lno;
 
 	/* Skip trailing <blank>s. */
 	for (; ecp->clen > 0 &&
-	    isblank(ecp->cp[0]); ++ecp->cp, --ecp->clen);
+	    ISBLANK(ecp->cp[0]); ++ecp->cp, --ecp->clen);
 
 	/*
 	 * Evaluate any offset.  If no address yet found, the offset
 	 * is relative to ".".
 	 */
 	total = 0;
-	if (ecp->clen != 0 && (isdigit(ecp->cp[0]) ||
+	if (ecp->clen != 0 && (ISDIGIT(ecp->cp[0]) ||
 	    ecp->cp[0] == '+' || ecp->cp[0] == '-' ||
 	    ecp->cp[0] == '^')) {
 		if (!*isaddrp) {
@@ -1976,14 +1998,14 @@ search:		mp->lno = sp->lno;
 		 */
 		F_SET(ecp, E_DELTA);
 		for (;;) {
-			for (; ecp->clen > 0 && isblank(ecp->cp[0]);
+			for (; ecp->clen > 0 && ISBLANK(ecp->cp[0]);
 			    ++ecp->cp, --ecp->clen);
-			if (ecp->clen == 0 || !isdigit(ecp->cp[0]) &&
+			if (ecp->clen == 0 || !ISDIGIT(ecp->cp[0]) &&
 			    ecp->cp[0] != '+' && ecp->cp[0] != '-' &&
 			    ecp->cp[0] != '^')
 				break;
-			if (!isdigit(ecp->cp[0]) &&
-			    !isdigit(ecp->cp[1])) {
+			if (!ISDIGIT(ecp->cp[0]) &&
+			    !ISDIGIT(ecp->cp[1])) {
 				total += ecp->cp[0] == '+' ? 1 : -1;
 				--ecp->clen;
 				++ecp->cp;
@@ -1997,7 +2019,7 @@ search:		mp->lno = sp->lno;
 					isneg = 0;
 
 				/* Get a signed long, add it to the total. */
-				if ((nret = nget_slong(&val,
+				if ((nret = nget_slong(sp, &val,
 				    ecp->cp, &endp, 10)) != NUM_OK ||
 				    (nret = NADD_SLONG(sp,
 				    total, val)) != NUM_OK) {
@@ -2014,7 +2036,7 @@ search:		mp->lno = sp->lno;
 
 	/*
 	 * Any value less than 0 is an error.  Make sure that the new value
-	 * will fit into a recno_t.
+	 * will fit into a db_recno_t.
 	 */
 	if (*isaddrp && total != 0) {
 		if (total < 0) {
@@ -2025,7 +2047,7 @@ search:		mp->lno = sp->lno;
 				return (0);
 			}
 		} else
-			if (!NPFITS(MAX_REC_NUMBER, mp->lno, total)) {
+			if (!NPFITS(DB_MAX_RECORDS, mp->lno, total)) {
 				ex_badaddr(sp, NULL, A_NOTSET, NUM_OVER);
 				*errp = 1;
 				return (0);
@@ -2041,10 +2063,9 @@ search:		mp->lno = sp->lno;
  *	Load up the next command, which may be an @ buffer or global command.
  */
 static int
-ex_load(sp)
-	SCR *sp;
+ex_load(SCR *sp)
 {
-	GS *gp;
+	WIN *wp;
 	EXCMD *ecp;
 	RANGE *rp;
 
@@ -2054,13 +2075,13 @@ ex_load(sp)
 	 * Lose any exhausted commands.  We know that the first command
 	 * can't be an AGV command, which makes things a bit easier.
 	 */
-	for (gp = sp->gp;;) {
+	for (wp = sp->wp;;) {
 		/*
 		 * If we're back to the original structure, leave it around,
 		 * but discard any allocated source name, we've returned to
 		 * the beginning of the command stack.
 		 */
-		if ((ecp = gp->ecq.lh_first) == &gp->excmd) {
+		if ((ecp = wp->ecq.lh_first) == &wp->excmd) {
 			if (F_ISSET(ecp, E_NAMEDISCARD)) {
 				free(ecp->if_name);
 				ecp->if_name = NULL;
@@ -2121,7 +2142,7 @@ ex_load(sp)
 	 * so we have play games.
 	 */
 	ecp->cp = ecp->o_cp;
-	memcpy(ecp->cp, ecp->cp + ecp->o_clen, ecp->o_clen);
+	MEMCPYW(ecp->cp, ecp->cp + ecp->o_clen, ecp->o_clen);
 	ecp->clen = ecp->o_clen;
 	ecp->range_lno = sp->lno = rp->start++;
 
@@ -2135,10 +2156,9 @@ ex_load(sp)
  *	Discard any pending ex commands.
  */
 static int
-ex_discard(sp)
-	SCR *sp;
+ex_discard(SCR *sp)
 {
-	GS *gp;
+	WIN *wp;
 	EXCMD *ecp;
 	RANGE *rp;
 
@@ -2146,7 +2166,7 @@ ex_discard(sp)
 	 * We know the first command can't be an AGV command, so we don't
 	 * process it specially.  We do, however, nail the command itself.
 	 */
-	for (gp = sp->gp; (ecp = gp->ecq.lh_first) != &gp->excmd;) {
+	for (wp = sp->wp; (ecp = wp->ecq.lh_first) != &wp->excmd;) {
 		if (FL_ISSET(ecp->agv_flags, AGV_ALL)) {
 			while ((rp = ecp->rq.cqh_first) != (void *)&ecp->rq) {
 				CIRCLEQ_REMOVE(&ecp->rq, rp, q);
@@ -2157,7 +2177,7 @@ ex_discard(sp)
 		LIST_REMOVE(ecp, q);
 		free(ecp);
 	}
-	gp->ecq.lh_first->clen = 0;
+	wp->ecq.lh_first->clen = 0;
 	return (0);
 }
 
@@ -2166,19 +2186,16 @@ ex_discard(sp)
  *	Display an unknown command name.
  */
 static void
-ex_unknown(sp, cmd, len)
-	SCR *sp;
-	char *cmd;
-	size_t len;
+ex_unknown(SCR *sp, CHAR_T *cmd, size_t len)
 {
 	size_t blen;
-	char *bp;
+	CHAR_T *bp;
 
-	GET_SPACE_GOTO(sp, bp, blen, len + 1);
+	GET_SPACE_GOTOW(sp, bp, blen, len + 1);
 	bp[len] = '\0';
-	memcpy(bp, cmd, len);
-	msgq_str(sp, M_ERR, bp, "098|The %s command is unknown");
-	FREE_SPACE(sp, bp, blen);
+	MEMCPYW(bp, cmd, len);
+	msgq_wstr(sp, M_ERR, bp, "098|The %s command is unknown");
+	FREE_SPACEW(sp, bp, blen);
 
 alloc_err:
 	return;
@@ -2190,16 +2207,14 @@ alloc_err:
  *	[un]abbreviate command, so it can turn off abbreviations.  See
  *	the usual ranting in the vi/v_txt_ev.c:txt_abbrev() routine.
  *
- * PUBLIC: int ex_is_abbrev __P((char *, size_t));
+ * PUBLIC: int ex_is_abbrev __P((SCR *, CHAR_T *, size_t));
  */
 int
-ex_is_abbrev(name, len)
-	char *name;
-	size_t len;
+ex_is_abbrev(SCR *sp, CHAR_T *name, size_t len)
 {
 	EXCMDLIST const *cp;
 
-	return ((cp = ex_comm_search(name, len)) != NULL &&
+	return ((cp = ex_comm_search(sp, name, len)) != NULL &&
 	    (cp == &cmds[C_ABBR] || cp == &cmds[C_UNABBREVIATE]));
 }
 
@@ -2209,12 +2224,10 @@ ex_is_abbrev(name, len)
  *	unmap command, so it can turn off input mapping.  See the usual
  *	ranting in the vi/v_txt_ev.c:txt_unmap() routine.
  *
- * PUBLIC: int ex_is_unmap __P((char *, size_t));
+ * PUBLIC: int ex_is_unmap __P((SCR *, CHAR_T *, size_t));
  */
 int
-ex_is_unmap(name, len)
-	char *name;
-	size_t len;
+ex_is_unmap(SCR *sp, CHAR_T *name, size_t len)
 {
 	EXCMDLIST const *cp;
 
@@ -2225,7 +2238,7 @@ ex_is_unmap(name, len)
 	if (name[len - 1] != '!')
 		return (0);
 	--len;
-	return ((cp = ex_comm_search(name, len)) != NULL &&
+	return ((cp = ex_comm_search(sp, name, len)) != NULL &&
 	    cp == &cmds[C_UNMAP]);
 }
 
@@ -2234,9 +2247,7 @@ ex_is_unmap(name, len)
  *	Search for a command name.
  */
 static EXCMDLIST const *
-ex_comm_search(name, len)
-	char *name;
-	size_t len;
+ex_comm_search(SCR *sp, CHAR_T *name, size_t len)
 {
 	EXCMDLIST const *cp;
 
@@ -2245,7 +2256,7 @@ ex_comm_search(name, len)
 			return (NULL);
 		if (cp->name[0] != name[0])
 			continue;
-		if (!memcmp(name, cp->name, len))
+		if (!MEMCMP(name, cp->name, len))
 			return (cp);
 	}
 	return (NULL);
@@ -2259,13 +2270,9 @@ ex_comm_search(name, len)
  * PUBLIC:    __P((SCR *, EXCMDLIST const *, enum badaddr, enum nresult));
  */
 void
-ex_badaddr(sp, cp, ba, nret)
-	SCR *sp;
-	EXCMDLIST const *cp;
-	enum badaddr ba;
-	enum nresult nret;
+ex_badaddr(SCR *sp, const EXCMDLIST *cp, enum badaddr ba, enum nresult nret)
 {
-	recno_t lno;
+	db_recno_t lno;
 
 	switch (nret) {
 	case NUM_OK:
@@ -2286,7 +2293,7 @@ ex_badaddr(sp, cp, ba, nret)
 	 * underlying file, that's the real problem.
 	 */
 	if (sp->ep == NULL) {
-		ex_emsg(sp, cp->name, EXM_NOFILEYET);
+		ex_wemsg(sp, cp ? cp->name : NULL, EXM_NOFILEYET);
 		return;
 	}
 
@@ -2329,21 +2336,21 @@ ex_comlog(sp, ecp)
 	SCR *sp;
 	EXCMD *ecp;
 {
-	TRACE(sp, "ecmd: %s", ecp->cmd->name);
+	vtrace(sp, "ecmd: %s", ecp->cmd->name);
 	if (ecp->addrcnt > 0) {
-		TRACE(sp, " a1 %d", ecp->addr1.lno);
+		vtrace(sp, " a1 %d", ecp->addr1.lno);
 		if (ecp->addrcnt > 1)
-			TRACE(sp, " a2: %d", ecp->addr2.lno);
+			vtrace(sp, " a2: %d", ecp->addr2.lno);
 	}
 	if (ecp->lineno)
-		TRACE(sp, " line %d", ecp->lineno);
+		vtrace(sp, " line %d", ecp->lineno);
 	if (ecp->flags)
-		TRACE(sp, " flags 0x%x", ecp->flags);
+		vtrace(sp, " flags 0x%x", ecp->flags);
 	if (F_ISSET(&exc, E_BUFFER))
-		TRACE(sp, " buffer %c", ecp->buffer);
+		vtrace(sp, " buffer %c", ecp->buffer);
 	if (ecp->argc)
 		for (cnt = 0; cnt < ecp->argc; ++cnt)
-			TRACE(sp, " arg %d: {%s}", cnt, ecp->argv[cnt]->bp);
-	TRACE(sp, "\n");
+			vtrace(sp, " arg %d: {%s}", cnt, ecp->argv[cnt]->bp);
+	vtrace(sp, "\n");
 }
 #endif
