@@ -10,7 +10,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)ex_at.c	10.11 (Berkeley) 6/30/96";
+static const char sccsid[] = "$Id: ex_at.c,v 10.16 2001/06/25 15:19:14 skimo Exp $ (Berkeley) $Date: 2001/06/25 15:19:14 $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -34,9 +34,7 @@ static const char sccsid[] = "@(#)ex_at.c	10.11 (Berkeley) 6/30/96";
  * PUBLIC: int ex_at __P((SCR *, EXCMD *));
  */
 int
-ex_at(sp, cmdp)
-	SCR *sp;
-	EXCMD *cmdp;
+ex_at(SCR *sp, EXCMD *cmdp)
 {
 	CB *cbp;
 	CHAR_T name;
@@ -44,7 +42,7 @@ ex_at(sp, cmdp)
 	RANGE *rp;
 	TEXT *tp;
 	size_t len;
-	char *p;
+	CHAR_T *p;
 
 	/*
 	 * !!!
@@ -108,7 +106,7 @@ ex_at(sp, cmdp)
 	    tp != (void *)&cbp->textq; tp = tp->q.cqe_prev)
 		len += tp->len + 1;
 
-	MALLOC_RET(sp, ecp->cp, char *, len * 2);
+	MALLOC_RET(sp, ecp->cp, CHAR_T *, len * 2 * sizeof(CHAR_T));
 	ecp->o_cp = ecp->cp;
 	ecp->o_clen = len;
 	ecp->cp[len] = '\0';
@@ -116,11 +114,11 @@ ex_at(sp, cmdp)
 	/* Copy the buffer into the command space. */
 	for (p = ecp->cp + len, tp = cbp->textq.cqh_last;
 	    tp != (void *)&cbp->textq; tp = tp->q.cqe_prev) {
-		memmove(p, tp->lb, tp->len);
+		MEMCPYW(p, tp->lb, tp->len);
 		p += tp->len;
 		*p++ = '\n';
 	}
 
-	LIST_INSERT_HEAD(&sp->gp->ecq, ecp, q);
+	LIST_INSERT_HEAD(&sp->wp->ecq, ecp, q);
 	return (0);
 }
