@@ -10,7 +10,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)ex_cscope.c	10.8 (Berkeley) 6/30/96";
+static const char sccsid[] = "$Id: ex_cscope.c,v 10.21 2003/11/05 17:11:54 skimo Exp $ (Berkeley) $Date: 2003/11/05 17:11:54 $";
 #endif /* not lint */
 
 #include <sys/param.h>
@@ -25,7 +25,6 @@ static const char sccsid[] = "@(#)ex_cscope.c	10.8 (Berkeley) 6/30/96";
 #include <errno.h>
 #include <fcntl.h>
 #include <limits.h>
-#include <signal.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,6 +33,7 @@ static const char sccsid[] = "@(#)ex_cscope.c	10.8 (Berkeley) 6/30/96";
 #include <unistd.h>
 
 #include "../common/common.h"
+#include "pathnames.h"
 #include "tag.h"
 
 #define	CSCOPE_DBFILE		"cscope.out"
@@ -62,22 +62,22 @@ find c|d|e|f|g|i|s|t buffer|pattern\n\
       s: find all uses of name\n\
       t: find assignments to name"
 
-static int cscope_add __P((SCR *, EXCMD *, char *));
-static int cscope_find __P((SCR *, EXCMD*, char *));
-static int cscope_help __P((SCR *, EXCMD *, char *));
-static int cscope_kill __P((SCR *, EXCMD *, char *));
-static int cscope_reset __P((SCR *, EXCMD *, char *));
+static int cscope_add __P((SCR *, EXCMD *, CHAR_T *));
+static int cscope_find __P((SCR *, EXCMD*, CHAR_T *));
+static int cscope_help __P((SCR *, EXCMD *, CHAR_T *));
+static int cscope_kill __P((SCR *, EXCMD *, CHAR_T *));
+static int cscope_reset __P((SCR *, EXCMD *, CHAR_T *));
 
 typedef struct _cc {
 	char	 *name;
-	int	(*function) __P((SCR *, EXCMD *, char *));
+	int	(*function) __P((SCR *, EXCMD *, CHAR_T *));
 	char	 *help_msg;
 	char	 *usage_msg;
 } CC;
 
 static CC const cscope_cmds[] = {
 	{ "add",   cscope_add,
-	   "Add a new cscope database", "add db-name" },
+	  "Add a new cscope database", "add file | directory" },
 	{ "find",  cscope_find,
 	  "Query the databases for a pattern", FINDHELP },
 	{ "help",  cscope_help,
@@ -97,7 +97,7 @@ static int	 get_paths __P((SCR *, CSC *));
 static CC const	*lookup_ccmd __P((char *));
 static int	 parse __P((SCR *, CSC *, TAGQ *, int *));
 static int	 read_prompt __P((SCR *, CSC *));
-static int	 run_cscope __P((SCR *, CSC *));
+static int	 run_cscope __P((SCR *, CSC *, char *));
 static int	 start_cscopes __P((SCR *, EXCMD *));
 static int	 terminate __P((SCR *, CSC *, int));
 
@@ -108,14 +108,15 @@ static int	 terminate __P((SCR *, CSC *, int));
  * PUBLIC: int ex_cscope __P((SCR *, EXCMD *));
  */
 int
-ex_cscope(sp, cmdp)
-	SCR *sp;
-	EXCMD *cmdp;
+ex_cscope(SCR *sp, EXCMD *cmdp)
 {
 	CC const *ccp;
 	EX_PRIVATE *exp;
 	int i;
-	char *cmd, *p;
+	CHAR_T *cmd;
+	CHAR_T *p;
+	char *np;
+	size_t nlen;
 
 	/* Initialize the default cscope directories. */
 	exp = EXP(sp);
@@ -139,7 +140,8 @@ ex_cscope(sp, cmdp)
 		for (; *p && isspace(*p); ++p);
 	}
 
-	if ((ccp = lookup_ccmd(cmd)) == NULL) {
+	INT2CHAR(sp, cmd, STRLEN(cmd) + 1, np, nlen);
+	if ((ccp = lookup_ccmd(np)) == NULL) {
 usage:		msgq(sp, M_ERR, "309|Use \"cscope help\" for help");
 		return (1);
 	}
@@ -153,12 +155,12 @@ usage:		msgq(sp, M_ERR, "309|Use \"cscope help\" for help");
  *	Initialize the cscope package.
  */
 static int
-start_cscopes(sp, cmdp)
-	SCR *sp;
-	EXCMD *cmdp;
+start_cscopes(SCR *sp, EXCMD *cmdp)
 {
 	size_t blen, len;
-	char *bp, *cscopes, *p;
+	char *bp, *cscopes, *p, *t;
+	CHAR_T *wp;
+	size_t wlen;
 
 	/*
 	 * EXTENSION #1:
@@ -175,12 +177,16 @@ start_cscopes(sp, cmdp)
 	if ((cscopes = getenv("CSCOPE_DIRS")) == NULL)
 		return (0);
 	len = strlen(cscopes);
-	GET_SPACE_RET(sp, bp, blen, len);
+	GET_SPACE_RETC(sp, bp, blen, len);
 	memcpy(bp, cscopes, len + 1);
 
-	for (cscopes = bp; (p = strsep(&bp, "\t ")) != NULL;)
-		if (*p != '\0')
-			(void)cscope_add(sp, cmdp, p);
+	for (cscopes = t = bp; (p = strsep(&t, "\t :")) != NULL;)
+		if (*p != '\0') {
+			CHAR2INT(sp, p, strlen(p) + 1, wp, wlen);
+			(void)cscope_add(sp, cmdp, wp);
+		}
+
+	FREE_SPACE(sp, bp, blen);
 	return (0);
 }
 
@@ -189,59 +195,73 @@ start_cscopes(sp, cmdp)
  *	The cscope add command.
  */
 static int
-cscope_add(sp, cmdp, dname)
-	SCR *sp;
-	EXCMD *cmdp;
-	char *dname;
+cscope_add(SCR *sp, EXCMD *cmdp, CHAR_T *dname)
 {
 	struct stat sb;
 	EX_PRIVATE *exp;
 	CSC *csc;
 	size_t len;
-	char path[MAXPATHLEN];
+	int cur_argc;
+	char *dbname, path[MAXPATHLEN];
+	char *np;
+	size_t nlen;
 
 	exp = EXP(sp);
 
-	if (argv_exp2(sp, cmdp, dname, strlen(dname)))
-		return (1);
 	/*
-	 *  0 args: impossible.
-	 *  1 args: usage.
-	 *  2 args: matched a directory.
-	 * >2 args: object, too many args.
-	 *
-	 * The 1 args case depends on the argv_sexp() function refusing
-	 * to return success without at least one non-blank character.
+	 *  0 additional args: usage.
+	 *  1 additional args: matched a file.
+	 * >1 additional args: object, too many args.
 	 */
-	switch (cmdp->argc) {
-	case 0:
-		abort();
-		/* NOTREACHED */
-	case 1:
+	cur_argc = cmdp->argc;
+	if (argv_exp2(sp, cmdp, dname, STRLEN(dname))) {
+		return (1);
+	}
+	if (cmdp->argc == cur_argc) {
 		(void)csc_help(sp, "add");
 		return (1);
-	case 2:
-		dname = cmdp->argv[1]->bp;
-		break;
-	default:
-		ex_emsg(sp, dname, EXM_FILECOUNT);
+	}
+	if (cmdp->argc == cur_argc + 1)
+		dname = cmdp->argv[cur_argc]->bp;
+	else {
+		ex_emsg(sp, np, EXM_FILECOUNT);
 		return (1);
 	}
 
-	/* If the database file doesn't exist, we're done. */
-	(void)snprintf(path, sizeof(path),
-	    "%s/%s", cmdp->argv[1]->bp, CSCOPE_DBFILE);
-	if (stat(path, &sb)) {
-		msgq(sp, M_SYSERR, path);
+	INT2CHAR(sp, dname, STRLEN(dname)+1, np, nlen);
+
+	/*
+	 * The user can specify a specific file (so they can have multiple
+	 * Cscope databases in a single directory) or a directory.  If the
+	 * file doesn't exist, we're done.  If it's a directory, append the
+	 * standard database file name and try again.  Store the directory
+	 * name regardless so that we can use it as a base for searches.
+	 */
+	if (stat(np, &sb)) {
+		msgq(sp, M_SYSERR, np);
 		return (1);
 	}
+	if (S_ISDIR(sb.st_mode)) {
+		(void)snprintf(path, sizeof(path),
+		    "%s/%s", np, CSCOPE_DBFILE);
+		if (stat(path, &sb)) {
+			msgq(sp, M_SYSERR, path);
+			return (1);
+		}
+		dbname = CSCOPE_DBFILE;
+	} else if ((dbname = strrchr(np, '/')) != NULL)
+		*dbname++ = '\0';
+	else {
+		dbname = np;
+		np = ".";
+	}
 
 	/* Allocate a cscope connection structure and initialize its fields. */
-	len = strlen(dname);
+	len = strlen(np);
 	CALLOC_RET(sp, csc, CSC *, 1, sizeof(CSC) + len);
 	csc->dname = csc->buf;
 	csc->dlen = len;
-	memcpy(csc->dname, dname, len);
+	memcpy(csc->dname, np, len);
 	csc->mtime = sb.st_mtime;
 
 	/* Get the search paths for the cscope. */
@@ -249,7 +269,7 @@ cscope_add(sp, cmdp, dname)
 		goto err;
 
 	/* Start the cscope process. */
-	if (run_cscope(sp, csc))
+	if (run_cscope(sp, csc, dbname))
 		goto err;
 
 	/*
@@ -260,12 +280,7 @@ cscope_add(sp, cmdp, dname)
 	LIST_INSERT_HEAD(&exp->cscq, csc, q);
 
 	/* Read the initial prompt from the cscope to make sure it's okay. */
-	if (read_prompt(sp, csc)) {
-		terminate(sp, csc, 0);
-		return (1);
-	}
-
-	return (0);
+	return read_prompt(sp, csc);
 
 err:	free(csc);
 	return (1);
@@ -277,9 +292,7 @@ err:	free(csc);
  *	cscope database.
  */
 static int
-get_paths(sp, csc)
-	SCR *sp;
-	CSC *csc;
+get_paths(SCR *sp, CSC *csc)
 {
 	struct stat sb;
 	int fd, nentries;
@@ -352,9 +365,7 @@ alloc_err:
  *	Fork off the cscope process.
  */
 static int
-run_cscope(sp, csc)
-	SCR *sp;
-	CSC *csc;
+run_cscope(SCR *sp, CSC *csc, char *dbname)
 {
 	int to_cs[2], from_cs[2];
 	char cmd[MAXPATHLEN * 2];
@@ -390,9 +401,10 @@ err:		if (to_cs[0] != -1)
 		(void)close(from_cs[0]);
 
 		/* Run the cscope command. */
-#define	CSCOPE_CMD_FMT		"cd '%s' && exec cscope -dl"
-		(void)snprintf(cmd, sizeof(cmd), CSCOPE_CMD_FMT, csc->dname);
-		(void)execl("/bin/sh", "sh", "-c", cmd, NULL);
+#define	CSCOPE_CMD_FMT		"cd '%s' && exec cscope -dl -f %s"
+		(void)snprintf(cmd, sizeof(cmd),
+		    CSCOPE_CMD_FMT, csc->dname, dbname);
+		(void)execl(_PATH_BSHELL, "sh", "-c", cmd, (char *)NULL);
 		msgq_str(sp, M_SYSERR, cmd, "execl: %s");
 		_exit (127);
 		/* NOTREACHED */
@@ -419,19 +431,18 @@ err:		if (to_cs[0] != -1)
  *	The cscope find command.
  */
 static int
-cscope_find(sp, cmdp, pattern)
-	SCR *sp;
-	EXCMD *cmdp;
-	char *pattern;
+cscope_find(SCR *sp, EXCMD *cmdp, CHAR_T *pattern)
 {
 	CSC *csc, *csc_next;
 	EX_PRIVATE *exp;
 	FREF *frp;
 	TAGQ *rtqp, *tqp;
 	TAG *rtp;
-	recno_t lno;
+	db_recno_t lno;
 	size_t cno, search;
 	int force, istmp, matches;
+	char *np = NULL;
+	size_t nlen;
 
 	exp = EXP(sp);
 
@@ -460,7 +471,9 @@ cscope_find(sp, cmdp, pattern)
 	}
 
 	/* Create the cscope command. */
-	if ((tqp = create_cs_cmd(sp, pattern, &search)) == NULL)
+	INT2CHAR(sp, pattern, STRLEN(pattern) + 1, np, nlen);
+	np = strdup(np);
+	if ((tqp = create_cs_cmd(sp, np, &search)) == NULL)
 		goto err;
 
 	/*
@@ -558,6 +571,8 @@ alloc_err:
 		free(rtqp);
 	if (rtp != NULL)
 		free(rtp);
+	if (np != NULL)
+		free(np);
 	return (1);
 }
 
@@ -566,10 +581,7 @@ alloc_err:
  *	Build a cscope command, creating and initializing the base TAGQ.
  */
 static TAGQ *
-create_cs_cmd(sp, pattern, searchp)
-	SCR *sp;
-	char *pattern;
-	size_t *searchp;
+create_cs_cmd(SCR *sp, char *pattern, size_t *searchp)
 {
 	CB *cbp;
 	TAGQ *tqp;
@@ -613,8 +625,8 @@ usage:		(void)csc_help(sp, "find");
 	if (p[0] == '"' && p[1] != '\0' && p[2] == '\0')
 		CBNAME(sp, cbp, p[1]);
 	if (cbp != NULL) {
-		p = cbp->textq.cqh_first->lb;
-		tlen = cbp->textq.cqh_first->len;
+		INT2CHAR(sp, cbp->textq.cqh_first->lb,
+			cbp->textq.cqh_first->len, p, tlen);
 	} else
 		tlen = strlen(p);
 
@@ -639,16 +651,12 @@ usage:		(void)csc_help(sp, "find");
  *	Parse the cscope output.
  */
 static int
-parse(sp, csc, tqp, matchesp)
-	SCR *sp;
-	CSC *csc;
-	TAGQ *tqp;
-	int *matchesp;
+parse(SCR *sp, CSC *csc, TAGQ *tqp, int *matchesp)
 {
 	TAG *tp;
-	recno_t slno;
+	db_recno_t slno;
 	size_t dlen, nlen, slen;
-	int ch, i, isnewer, nlines;
+	int ch, i, isolder, nlines;
 	char *dname, *name, *search, *p, *t, dummy[2], buf[2048];
 
 	for (;;) {
@@ -698,7 +706,7 @@ parse(sp, csc, tqp, matchesp)
 			case 1:			/* Context. */
 				break;
 			case 2:			/* Line number. */
-				slno = (recno_t)atol(t);
+				slno = (db_recno_t)atol(t);
 				break;
 			}
 		if (i != 3 || p == NULL || t == NULL)
@@ -709,14 +717,14 @@ parse(sp, csc, tqp, matchesp)
 		slen = strlen(p);
 
 		/* Resolve the file name. */
-		csc_file(sp, csc, name, &dname, &dlen, &isnewer);
+		csc_file(sp, csc, name, &dname, &dlen, &isolder);
 
 		/*
-		 * If the file was modified more recently than the cscope
-		 * database, or there wasn't a search string, use the line
-		 * number.
+		 * If the file is older than the cscope database, that is,
+		 * the database was built since the file was last modified,
+		 * or there wasn't a search string, use the line number.
 		 */
-		if (isnewer || strcmp(search, "<unknown>") == 0) {
+		if (isolder || strcmp(search, "<unknown>") == 0) {
 			search = NULL;
 			slen = 0;
 		}
@@ -727,7 +735,7 @@ parse(sp, csc, tqp, matchesp)
 		 */
 		CALLOC_RET(sp, tp,
 		    TAG *, 1, sizeof(TAG) + dlen + 2 + nlen + 1 + slen + 1);
-		tp->fname = tp->buf;
+		tp->fname = (char *)tp->buf;
 		if (dlen != 0) {
 			memcpy(tp->fname, dname, dlen);
 			tp->fname[dlen] = '/';
@@ -737,16 +745,15 @@ parse(sp, csc, tqp, matchesp)
 		tp->fnlen = dlen + nlen;
 		tp->slno = slno;
 		if (slen != 0) {
-			tp->search = tp->fname + tp->fnlen + 1;
-			memcpy(tp->search, search, (tp->slen = slen) + 1);
+			tp->search = (CHAR_T*)(tp->fname + tp->fnlen + 1);
+			MEMCPYW(tp->search, search, (tp->slen = slen) + 1);
 		}
 		CIRCLEQ_INSERT_TAIL(&tqp->tagq, tp, q);
 
 		++*matchesp;
 	}
 
-	(void)read_prompt(sp, csc);
-	return (0);
+	return read_prompt(sp, csc);
 
 io_err:	if (feof(csc->from_fp))
 		errno = EIO;
@@ -760,12 +767,7 @@ io_err:	if (feof(csc->from_fp))
  *	Search for the right path to this file.
  */
 static void
-csc_file(sp, csc, name, dirp, dlenp, isnewerp)
-	SCR *sp;
-	CSC *csc;
-	char *name, **dirp;
-	size_t *dlenp;
-	int *isnewerp;
+csc_file(SCR *sp, CSC *csc, char *name, char **dirp, size_t *dlenp, int *isolderp)
 {
 	struct stat sb;
 	char **pp, buf[MAXPATHLEN];
@@ -782,7 +784,7 @@ csc_file(sp, csc, name, dirp, dlenp, isnewerp)
 		if (stat(buf, &sb) == 0) {
 			*dirp = *pp;
 			*dlenp = strlen(*pp);
-			*isnewerp = sb.st_mtime > csc->mtime;
+			*isolderp = sb.st_mtime < csc->mtime;
 			return;
 		}
 	}
@@ -794,12 +796,13 @@ csc_file(sp, csc, name, dirp, dlenp, isnewerp)
  *	The cscope help command.
  */
 static int
-cscope_help(sp, cmdp, subcmd)
-	SCR *sp;
-	EXCMD *cmdp;
-	char *subcmd;
+cscope_help(SCR *sp, EXCMD *cmdp, CHAR_T *subcmd)
 {
-	return (csc_help(sp, subcmd));
+	char *np;
+	size_t nlen;
+
+	INT2CHAR(sp, subcmd, STRLEN(subcmd) + 1, np, nlen);
+	return (csc_help(sp, np));
 }
 
 /*
@@ -807,9 +810,7 @@ cscope_help(sp, cmdp, subcmd)
  *	Display help/usage messages.
  */
 static int
-csc_help(sp, cmd)
-	SCR *sp;
-	char *cmd;
+csc_help(SCR *sp, char *cmd)
 {
 	CC const *ccp;
 
@@ -836,12 +837,13 @@ csc_help(sp, cmd)
  *	The cscope kill command.
  */
 static int
-cscope_kill(sp, cmdp, cn)
-	SCR *sp;
-	EXCMD *cmdp;
-	char *cn;
+cscope_kill(SCR *sp, EXCMD *cmdp, CHAR_T *cn)
 {
-	return (terminate(sp, NULL, atoi(cn)));
+	char *np;
+	size_t nlen;
+
+	INT2CHAR(sp, cn, STRLEN(cn) + 1, np, nlen);
+	return (terminate(sp, NULL, atoi(np)));
 }
 
 /*
@@ -849,10 +851,7 @@ cscope_kill(sp, cmdp, cn)
  *	Detach from a cscope process.
  */
 static int
-terminate(sp, csc, n)
-	SCR *sp;
-	CSC *csc;
-	int n;
+terminate(SCR *sp, CSC *csc, int n)
 {
 	EX_PRIVATE *exp;
 	int i, pstat;
@@ -905,16 +904,15 @@ badno:			msgq(sp, M_ERR, "312|%d: no such cscope session", n);
  *	The cscope reset command.
  */
 static int
-cscope_reset(sp, cmdp, notusedp)
-	SCR *sp;
-	EXCMD *cmdp;
-	char *notusedp;
+cscope_reset(SCR *sp, EXCMD *cmdp, CHAR_T *notusedp)
 {
 	EX_PRIVATE *exp;
 
-	for (exp = EXP(sp); exp->cscq.lh_first != NULL;)
-		if (cscope_kill(sp, cmdp, "1"))
+	for (exp = EXP(sp); exp->cscq.lh_first != NULL;) {
+		static CHAR_T one[] = {'1', 0};
+		if (cscope_kill(sp, cmdp, one))
 			return (1);
+	}
 	return (0);
 }
 
@@ -925,8 +923,7 @@ cscope_reset(sp, cmdp, notusedp)
  * PUBLIC: int cscope_display __P((SCR *));
  */
 int
-cscope_display(sp)
-	SCR *sp;
+cscope_display(SCR *sp)
 {
 	EX_PRIVATE *exp;
 	CSC *csc;
@@ -951,10 +948,7 @@ cscope_display(sp)
  * PUBLIC: int cscope_search __P((SCR *, TAGQ *, TAG *));
  */
 int
-cscope_search(sp, tqp, tp)
-	SCR *sp;
-	TAGQ *tqp;
-	TAG *tp;
+cscope_search(SCR *sp, TAGQ *tqp, TAG *tp)
 {
 	MARK m;
 
@@ -973,7 +967,7 @@ cscope_search(sp, tqp, tp)
 		m.lno = 1;
 		m.cno = 0;
 		if (f_search(sp, &m, &m,
-		    tp->search, tp->slen, NULL, SEARCH_CSCOPE | SEARCH_FILE)) {
+		    tp->search, tp->slen, NULL, SEARCH_CSCOPE | SEARCH_FIRST)) {
 			tag_msg(sp, TAG_SEARCH, tqp->tag);
 			return (1);
 		}
@@ -1003,8 +997,7 @@ cscope_search(sp, tqp, tp)
  *	Return a pointer to the command structure.
  */
 static CC const *
-lookup_ccmd(name)
-	char *name;
+lookup_ccmd(char *name)
 {
 	CC const *ccp;
 	size_t len;
@@ -1021,9 +1014,7 @@ lookup_ccmd(name)
  *	Read a prompt from cscope.
  */
 static int
-read_prompt(sp, csc)
-	SCR *sp;
-	CSC *csc;
+read_prompt(SCR *sp, CSC *csc)
 {
 	int ch;
 
