@@ -10,7 +10,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)ex_delete.c	10.8 (Berkeley) 3/6/96";
+static const char sccsid[] = "$Id: ex_delete.c,v 10.11 2001/06/25 15:19:15 skimo Exp $ (Berkeley) $Date: 2001/06/25 15:19:15 $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -30,11 +30,9 @@ static const char sccsid[] = "@(#)ex_delete.c	10.8 (Berkeley) 3/6/96";
  * PUBLIC: int ex_delete __P((SCR *, EXCMD *));
  */
 int
-ex_delete(sp, cmdp)
-	SCR *sp;
-	EXCMD *cmdp;
+ex_delete(SCR *sp, EXCMD *cmdp)
 {
-	recno_t lno;
+	db_recno_t lno;
 
 	NEEDFILE(sp, cmdp);
 
@@ -50,7 +48,7 @@ ex_delete(sp, cmdp)
 		return (1);
 
 	/* Delete the lines. */
-	if (delete(sp, &cmdp->addr1, &cmdp->addr2, 1))
+	if (del(sp, &cmdp->addr1, &cmdp->addr2, 1))
 		return (1);
 
 	/* Set the cursor to the line after the last line deleted. */
