@@ -10,7 +10,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)ex_equal.c	10.10 (Berkeley) 3/6/96";
+static const char sccsid[] = "$Id: ex_equal.c,v 10.12 2001/06/25 15:19:15 skimo Exp $ (Berkeley) $Date: 2001/06/25 15:19:15 $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -28,11 +28,9 @@ static const char sccsid[] = "@(#)ex_equal.c	10.10 (Berkeley) 3/6/96";
  * PUBLIC: int ex_equal __P((SCR *, EXCMD *));
  */
 int
-ex_equal(sp, cmdp)
-	SCR *sp;
-	EXCMD *cmdp;
+ex_equal(SCR *sp, EXCMD *cmdp)
 {
-	recno_t lno;
+	db_recno_t lno;
 
 	NEEDFILE(sp, cmdp);
 
