@@ -10,7 +10,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)ex_print.c	10.18 (Berkeley) 5/12/96";
+static const char sccsid[] = "$Id: ex_print.c,v 10.24 2001/07/29 19:07:29 skimo Exp $ (Berkeley) $Date: 2001/07/29 19:07:29 $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -30,7 +30,8 @@ static const char sccsid[] = "@(#)ex_print.c	10.18 (Berkeley) 5/12/96";
 
 #include "../common/common.h"
 
-static int ex_prchars __P((SCR *, const char *, size_t *, size_t, u_int, int));
+static int ex_prchars __P((SCR *, const CHAR_T *, size_t *, size_t, 
+                           u_int, int));
 
 /*
  * ex_list -- :[line [,line]] l[ist] [count] [flags]
@@ -40,9 +41,7 @@ static int ex_prchars __P((SCR *, const char *, size_t *, size_t, u_int, int));
  * PUBLIC: int ex_list __P((SCR *, EXCMD *));
  */
 int
-ex_list(sp, cmdp)
-	SCR *sp;
-	EXCMD *cmdp;
+ex_list(SCR *sp, EXCMD *cmdp)
 {
 	if (ex_print(sp, cmdp,
 	    &cmdp->addr1, &cmdp->addr2, cmdp->iflags | E_C_LIST))
@@ -60,9 +59,7 @@ ex_list(sp, cmdp)
  * PUBLIC: int ex_number __P((SCR *, EXCMD *));
  */
 int
-ex_number(sp, cmdp)
-	SCR *sp;
-	EXCMD *cmdp;
+ex_number(SCR *sp, EXCMD *cmdp)
 {
 	if (ex_print(sp, cmdp,
 	    &cmdp->addr1, &cmdp->addr2, cmdp->iflags | E_C_HASH))
@@ -80,9 +77,7 @@ ex_number(sp, cmdp)
  * PUBLIC: int ex_pr __P((SCR *, EXCMD *));
  */
 int
-ex_pr(sp, cmdp)
-	SCR *sp;
-	EXCMD *cmdp;
+ex_pr(SCR *sp, EXCMD *cmdp)
 {
 	if (ex_print(sp, cmdp, &cmdp->addr1, &cmdp->addr2, cmdp->iflags))
 		return (1);
@@ -98,16 +93,15 @@ ex_pr(sp, cmdp)
  * PUBLIC: int ex_print __P((SCR *, EXCMD *, MARK *, MARK *, u_int32_t));
  */
 int
-ex_print(sp, cmdp, fp, tp, flags)
-	SCR *sp;
-	EXCMD *cmdp;
-	MARK *fp, *tp;
-	u_int32_t flags;
+ex_print(SCR *sp, EXCMD *cmdp, MARK *fp, MARK *tp, u_int32_t flags)
 {
 	GS *gp;
-	recno_t from, to;
+	db_recno_t from, to;
 	size_t col, len;
-	char *p, buf[10];
+	CHAR_T *p;
+	CHAR_T buf[10];
+	CHAR_T *wp;
+	size_t wlen;
 
 	NEEDFILE(sp, cmdp);
 
@@ -122,10 +116,10 @@ ex_print(sp, cmdp, fp, tp, flags)
 		 */
 		if (LF_ISSET(E_C_HASH)) {
 			if (from <= 999999) {
-				snprintf(buf, sizeof(buf), "%6ld  ", from);
+				SPRINTF(buf, SIZE(buf), L("%6ld  "), from);
 				p = buf;
 			} else
-				p = "TOOBIG  ";
+				p = L("TOOBIG  ");
 			if (ex_prchars(sp, p, &col, 8, 0, 0))
 				return (1);
 		}
@@ -153,19 +147,15 @@ ex_print(sp, cmdp, fp, tp, flags)
  * ex_ldisplay --
  *	Display a line without any preceding number.
  *
- * PUBLIC: int ex_ldisplay __P((SCR *, const char *, size_t, size_t, u_int));
+ * PUBLIC: int ex_ldisplay __P((SCR *, const CHAR_T *, size_t, size_t, u_int));
  */
 int
-ex_ldisplay(sp, p, len, col, flags)
-	SCR *sp;
-	const char *p;
-	size_t len, col;
-	u_int flags;
+ex_ldisplay(SCR *sp, const CHAR_T *p, size_t len, size_t col, u_int flags)
 {
 	if (len > 0 && ex_prchars(sp, p, &col, len, LF_ISSET(E_C_LIST), 0))
 		return (1);
 	if (!INTERRUPTED(sp) && LF_ISSET(E_C_LIST)) {
-		p = "$";
+		p = L("$");
 		if (ex_prchars(sp, p, &col, 1, LF_ISSET(E_C_LIST), 0))
 			return (1);
 	}
@@ -181,21 +171,19 @@ ex_ldisplay(sp, p, len, col, flags)
  * PUBLIC: int ex_scprint __P((SCR *, MARK *, MARK *));
  */
 int
-ex_scprint(sp, fp, tp)
-	SCR *sp;
-	MARK *fp, *tp;
+ex_scprint(SCR *sp, MARK *fp, MARK *tp)
 {
-	const char *p;
+	CHAR_T *p;
 	size_t col, len;
 
 	col = 0;
 	if (O_ISSET(sp, O_NUMBER)) {
-		p = "        ";
+		p = L("        ");
 		if (ex_prchars(sp, p, &col, 8, 0, 0))
 			return (1);
 	}
 
-	if (db_get(sp, fp->lno, DBG_FATAL, (char **)&p, &len))
+	if (db_get(sp, fp->lno, DBG_FATAL, &p, &len))
 		return (1);
 
 	if (ex_prchars(sp, p, &col, fp->cno, 0, ' '))
@@ -206,7 +194,7 @@ ex_scprint(sp, fp, tp)
 		return (1);
 	if (INTERRUPTED(sp))
 		return (1);
-	p = "[ynq]";		/* XXX: should be msg_cat. */
+	p = L("[ynq]");		/* XXX: should be msg_cat. */
 	if (ex_prchars(sp, p, &col, 5, 0, 0))
 		return (1);
 	(void)ex_fflush(sp);
@@ -218,14 +206,11 @@ ex_scprint(sp, fp, tp)
  *	Local routine to dump characters to the screen.
  */
 static int
-ex_prchars(sp, p, colp, len, flags, repeatc)
-	SCR *sp;
-	const char *p;
-	size_t *colp, len;
-	u_int flags;
-	int repeatc;
+ex_prchars(SCR *sp, const CHAR_T *p, size_t *colp, size_t len, 
+	    u_int flags, int repeatc)
 {
-	CHAR_T ch, *kp;
+	CHAR_T ch;
+	char *kp;
 	GS *gp;
 	size_t col, tlen, ts;
 
@@ -234,7 +219,7 @@ ex_prchars(sp, p, colp, len, flags, repeatc)
 	gp = sp->gp;
 	ts = O_VAL(sp, O_TABSTOP);
 	for (col = *colp; len--;)
-		if ((ch = *p++) == '\t' && !LF_ISSET(E_C_LIST))
+		if ((ch = *p++) == L('\t') && !LF_ISSET(E_C_LIST))
 			for (tlen = ts - col % ts;
 			    col < sp->cols && tlen--; ++col) {
 				(void)ex_printf(sp,
@@ -243,8 +228,15 @@ ex_prchars(sp, p, colp, len, flags, repeatc)
 					goto intr;
 			}
 		else {
-			kp = KEY_NAME(sp, ch);
-			tlen = KEY_LEN(sp, ch);
+			/* XXXX */
+			if (INTISWIDE(ch)) {
+			    CHAR_T str[2] = {0, 0};
+			    str[0] = ch;
+			    INT2CHAR(sp, str, 2, kp, tlen);
+			} else {
+			    kp = KEY_NAME(sp, ch);
+			    tlen = KEY_LEN(sp, ch);
+			}
 			if (!repeatc  && col + tlen < sp->cols) {
 				(void)ex_puts(sp, kp);
 				col += tlen;
@@ -283,6 +275,7 @@ ex_printf(sp, fmt, va_alist)
 	EX_PRIVATE *exp;
 	va_list ap;
 	size_t n;
+	CHAR_T *kp;
 
 	exp = EXP(sp);
 
@@ -309,9 +302,7 @@ ex_printf(sp, fmt, va_alist)
  * PUBLIC: int ex_puts __P((SCR *, const char *));
  */
 int
-ex_puts(sp, str)
-	SCR *sp;
-	const char *str;
+ex_puts(SCR *sp, const char *str)
 {
 	EX_PRIVATE *exp;
 	int doflush, n;
@@ -337,15 +328,14 @@ ex_puts(sp, str)
  * PUBLIC: int ex_fflush __P((SCR *sp));
  */
 int
-ex_fflush(sp)
-	SCR *sp;
+ex_fflush(SCR *sp)
 {
 	EX_PRIVATE *exp;
 
 	exp = EXP(sp);
 
 	if (exp->obp_len != 0) {
-		sp->gp->scr_msg(sp, M_NONE, exp->obp, exp->obp_len);
+		sp->wp->scr_msg(sp, M_NONE, exp->obp, exp->obp_len);
 		exp->obp_len = 0;
 	}
 	return (0);
