@@ -10,7 +10,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)v_search.c	10.17 (Berkeley) 6/30/96";
+static const char sccsid[] = "$Id: v_search.c,v 10.30 2001/09/11 20:52:46 skimo Exp $ (Berkeley) $Date: 2001/09/11 20:52:46 $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -27,9 +27,10 @@ static const char sccsid[] = "@(#)v_search.c	10.17 (Berkeley) 6/30/96";
 
 #include "../common/common.h"
 #include "vi.h"
+#include "../ipc/ip.h"
 
 static int v_exaddr __P((SCR *, VICMD *, dir_t));
-static int v_search __P((SCR *, VICMD *, char *, size_t, u_int, dir_t));
+static int v_search __P((SCR *, VICMD *, CHAR_T *, size_t, u_int, dir_t));
 
 /*
  * v_srch -- [count]?RE[? offset]
@@ -38,9 +39,7 @@ static int v_search __P((SCR *, VICMD *, char *, size_t, u_int, dir_t));
  * PUBLIC: int v_searchb __P((SCR *, VICMD *));
  */
 int
-v_searchb(sp, vp)
-	SCR *sp;
-	VICMD *vp;
+v_searchb(SCR *sp, VICMD *vp)
 {
 	return (v_exaddr(sp, vp, BACKWARD));
 }
@@ -52,9 +51,7 @@ v_searchb(sp, vp)
  * PUBLIC: int v_searchf __P((SCR *, VICMD *));
  */
 int
-v_searchf(sp, vp)
-	SCR *sp;
-	VICMD *vp;
+v_searchf(SCR *sp, VICMD *vp)
 {
 	return (v_exaddr(sp, vp, FORWARD));
 }
@@ -64,19 +61,19 @@ v_searchf(sp, vp)
  *	Do a vi search (which is really an ex address).
  */
 static int
-v_exaddr(sp, vp, dir)
-	SCR *sp;
-	VICMD *vp;
-	dir_t dir;
+v_exaddr(SCR *sp, VICMD *vp, dir_t dir)
 {
-	static EXCMDLIST fake = { "search" };
+	static EXCMDLIST fake = { L("search") };
 	EXCMD *cmdp;
-	GS *gp;
+	WIN *wp;
 	TEXT *tp;
-	recno_t s_lno;
+	db_recno_t s_lno;
 	size_t len, s_cno, tlen;
 	int err, nb, type;
-	char *cmd, *t, buf[20];
+	char buf[20];
+	CHAR_T *cmd, *t;
+	CHAR_T *w;
+	size_t wlen;
 
 	/*
 	 * !!!
@@ -120,10 +117,10 @@ v_exaddr(sp, vp, dir)
 	 *
 	 * Build a fake ex command structure.
 	 */
-	gp = sp->gp;
-	gp->excmd.cp = tp->lb;
-	gp->excmd.clen = tp->len;
-	F_INIT(&gp->excmd, E_VISEARCH);
+	wp = sp->wp;
+	wp->excmd.cp = tp->lb;
+	wp->excmd.clen = tp->len;
+	F_INIT(&wp->excmd, E_VISEARCH);
 
 	/*
 	 * XXX
@@ -145,7 +142,7 @@ v_exaddr(sp, vp, dir)
 	 * messages, it's a real problem for us.
 	 */
 	if (!KEYS_WAITING(sp))
-		F_SET(&gp->excmd, E_SEARCH_WMSG);
+		F_SET(&wp->excmd, E_SEARCH_WMSG);
 		
 	/* Save the current line/column. */
 	s_lno = sp->lno;
@@ -175,7 +172,7 @@ v_exaddr(sp, vp, dir)
 	 * Historically, the command "/STRING/;   " failed, apparently it
 	 * confused the parser.  We're not that compatible.
 	 */
-	cmdp = &gp->excmd;
+	cmdp = &wp->excmd;
 	if (ex_range(sp, cmdp, &err))
 		return (1);
 	
@@ -185,7 +182,7 @@ v_exaddr(sp, vp, dir)
 	 */
 	cmd = cmdp->cp;
 	len = cmdp->clen;
-	gp->excmd.clen = 0;
+	wp->excmd.clen = 0;
 
 	if (err)
 		goto err2;
@@ -245,7 +242,7 @@ v_exaddr(sp, vp, dir)
 
 		/* Default to z+. */
 		if (!type &&
-		    v_event_push(sp, NULL, "+", 1, CH_NOMAP | CH_QUOTED))
+		    v_event_push(sp, NULL, L("+"), 1, CH_NOMAP | CH_QUOTED))
 			return (1);
 
 		/* Push the user's command. */
@@ -255,7 +252,8 @@ v_exaddr(sp, vp, dir)
 		/* Push line number so get correct z display. */
 		tlen = snprintf(buf,
 		    sizeof(buf), "%lu", (u_long)vp->m_stop.lno);
-		if (v_event_push(sp, NULL, buf, tlen, CH_NOMAP | CH_QUOTED))
+		CHAR2INT(sp, buf, tlen, w, wlen);
+		if (v_event_push(sp, NULL, w, wlen, CH_NOMAP | CH_QUOTED))
 			return (1);
 		 
 		/* Don't refresh until after 'z' happens. */
@@ -284,9 +282,7 @@ err2:	vp->m_final.lno = s_lno;
  * PUBLIC: int v_searchN __P((SCR *, VICMD *));
  */
 int
-v_searchN(sp, vp)
-	SCR *sp;
-	VICMD *vp;
+v_searchN(SCR *sp, VICMD *vp)
 {
 	dir_t dir;
 
@@ -311,9 +307,7 @@ v_searchN(sp, vp)
  * PUBLIC: int v_searchn __P((SCR *, VICMD *));
  */
 int
-v_searchn(sp, vp)
-	SCR *sp;
-	VICMD *vp;
+v_searchn(SCR *sp, VICMD *vp)
 {
 	return (v_search(sp, vp, NULL, 0, SEARCH_PARSE, sp->searchdir));
 }
@@ -325,36 +319,64 @@ v_searchn(sp, vp)
  * PUBLIC: int v_searchw __P((SCR *, VICMD *));
  */
 int
-v_searchw(sp, vp)
-	SCR *sp;
-	VICMD *vp;
+v_searchw(SCR *sp, VICMD *vp)
 {
 	size_t blen, len;
 	int rval;
-	char *bp;
+	CHAR_T *bp, *p;
 
-	len = VIP(sp)->klen + sizeof(RE_WSTART) + sizeof(RE_WSTOP);
-	GET_SPACE_RET(sp, bp, blen, len);
-	len = snprintf(bp, blen, "%s%s%s", RE_WSTART, VIP(sp)->keyw, RE_WSTOP);
+	len = VIP(sp)->klen + RE_WSTART_LEN + RE_WSTOP_LEN;
+	GET_SPACE_RETW(sp, bp, blen, len);
+	MEMCPY(bp, RE_WSTART, RE_WSTART_LEN); 
+	p = bp + RE_WSTART_LEN;
+	MEMCPY(p, VIP(sp)->keyw, VIP(sp)->klen);
+	p += VIP(sp)->klen;
+	MEMCPY(p, RE_WSTOP, RE_WSTOP_LEN); 
 
 	rval = v_search(sp, vp, bp, len, SEARCH_SET, FORWARD);
 
-	FREE_SPACE(sp, bp, blen);
+	FREE_SPACEW(sp, bp, blen);
 	return (rval);
 }
 
+/*
+ * v_esearch -- <dialog box>
+ *	Search command from the screen.
+ *
+ * PUBLIC: int v_esearch __P((SCR *, VICMD *));
+ */
+int
+v_esearch(SCR *sp, VICMD *vp)
+{
+	MARK m;
+	int flags;
+
+	m.lno = sp->lno;
+	m.cno = sp->cno;
+
+	LF_INIT(SEARCH_NOOPT);
+	if (FL_ISSET(vp->ev.e_flags, VI_SEARCH_EXT))
+		LF_SET(SEARCH_EXTEND);
+	if (FL_ISSET(vp->ev.e_flags, VI_SEARCH_IC))
+		LF_SET(SEARCH_IC);
+	if (FL_ISSET(vp->ev.e_flags, VI_SEARCH_ICL))
+		LF_SET(SEARCH_ICL);
+	if (FL_ISSET(vp->ev.e_flags, VI_SEARCH_INCR))
+		LF_SET(SEARCH_INCR);
+	if (FL_ISSET(vp->ev.e_flags, VI_SEARCH_LIT))
+		LF_SET(SEARCH_LITERAL);
+	if (FL_ISSET(vp->ev.e_flags, VI_SEARCH_WR))
+		LF_SET(SEARCH_WRAP);
+	return (v_search(sp, vp, vp->ev.e_csp, vp->ev.e_len, flags,
+	    FL_ISSET(vp->ev.e_flags, VI_SEARCH_REV) ? BACKWARD : FORWARD));
+}
+
 /*
  * v_search --
  *	The search commands.
  */
 static int
-v_search(sp, vp, ptrn, plen, flags, dir)
-	SCR *sp;
-	VICMD *vp;
-	u_int flags;
-	char *ptrn;
-	size_t plen;
-	dir_t dir;
+v_search(SCR *sp, VICMD *vp, CHAR_T *ptrn, size_t plen, u_int flags, dir_t dir)
 {
 	/* Display messages. */
 	LF_SET(SEARCH_MSG);
@@ -417,10 +439,7 @@ v_search(sp, vp, ptrn, plen, flags, dir)
  * PUBLIC: int v_correct __P((SCR *, VICMD *, int));
  */
 int
-v_correct(sp, vp, isdelta)
-	SCR *sp;
-	VICMD *vp;
-	int isdelta;
+v_correct(SCR *sp, VICMD *vp, int isdelta)
 {
 	dir_t dir;
 	MARK m;
@@ -496,15 +515,18 @@ v_correct(sp, vp, isdelta)
 	 * the current one (this is safe because we know the search had to move
 	 * to succeed).
 	 *
-	 * Searches become line mode operations if they start at column 0 and
-	 * end at column 0 of another line.
+	 * Searches become line mode operations if they start at the first
+	 * nonblank and end at column 0 of another line.
 	 */
 	if (vp->m_start.lno < vp->m_stop.lno && vp->m_stop.cno == 0) {
 		if (db_get(sp, --vp->m_stop.lno, DBG_FATAL, NULL, &len))
 			return (1);
-		if (vp->m_start.cno == 0)
-			F_SET(vp, VM_LMODE);
 		vp->m_stop.cno = len ? len - 1 : 0;
+		len = 0;
+		if (nonblank(sp, vp->m_start.lno, &len))
+			return (1);
+		if (vp->m_start.cno <= len)
+			F_SET(vp, VM_LMODE);
 	} else
 		--vp->m_stop.cno;
 
