@@ -10,7 +10,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)v_txt.c	10.83 (Berkeley) 9/15/96";
+static const char sccsid[] = "$Id: v_txt.c,v 10.108 2003/07/18 21:27:42 skimo Exp $ (Berkeley) $Date: 2003/07/18 21:27:42 $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -63,11 +63,7 @@ static void	 txt_unmap __P((SCR *, TEXT *, u_int32_t *));
  * PUBLIC: int v_tcmd __P((SCR *, VICMD *, ARG_CHAR_T, u_int));
  */
 int
-v_tcmd(sp, vp, prompt, flags)
-	SCR *sp;
-	VICMD *vp;
-	ARG_CHAR_T prompt;
-	u_int flags;
+v_tcmd(SCR *sp, VICMD *vp, ARG_CHAR_T prompt, u_int flags)
 {
 	/* Normally, we end up where we started. */
 	vp->m_final.lno = sp->lno;
@@ -118,8 +114,7 @@ v_tcmd(sp, vp, prompt, flags)
  *	Initialize the screen map for colon command-line input.
  */
 static int
-txt_map_init(sp)
-	SCR *sp;
+txt_map_init(SCR *sp)
 {
 	SMAP *esmp;
 	VI_PRIVATE *vip;
@@ -171,8 +166,7 @@ txt_map_init(sp)
  *	Reset the screen map for colon command-line input.
  */
 static int
-txt_map_end(sp)
-	SCR *sp;
+txt_map_end(SCR *sp)
 {
 	VI_PRIVATE *vip;
 	size_t cnt;
@@ -242,19 +236,19 @@ txt_map_end(sp)
  *	Vi text input.
  *
  * PUBLIC: int v_txt __P((SCR *, VICMD *, MARK *,
- * PUBLIC:    const char *, size_t, ARG_CHAR_T, recno_t, u_long, u_int32_t));
+ * PUBLIC:    const CHAR_T *, size_t, ARG_CHAR_T, db_recno_t, u_long, u_int32_t));
  */
 int
-v_txt(sp, vp, tm, lp, len, prompt, ai_line, rcount, flags)
-	SCR *sp;
-	VICMD *vp;
-	MARK *tm;		/* To MARK. */
-	const char *lp;		/* Input line. */
-	size_t len;		/* Input line length. */
-	ARG_CHAR_T prompt;	/* Prompt to display. */
-	recno_t ai_line;	/* Line number to use for autoindent count. */
-	u_long rcount;		/* Replay count. */
-	u_int32_t flags;	/* TXT_* flags. */
+v_txt(SCR *sp, VICMD *vp, MARK *tm, const CHAR_T *lp, size_t len, ARG_CHAR_T prompt, db_recno_t ai_line, u_long rcount, u_int32_t flags)
+	        
+	          
+	         		/* To MARK. */
+	                 	/* Input line. */
+	           		/* Input line length. */
+	                  	/* Prompt to display. */
+	                   	/* Line number to use for autoindent count. */
+	              		/* Replay count. */
+	                	/* TXT_* flags. */
 {
 	EVENT ev, *evp;		/* Current event. */
 	EVENT fc;		/* File name completion event. */
@@ -281,7 +275,7 @@ v_txt(sp, vp, tm, lp, len, prompt, ai_line, rcount, flags)
 	int showmatch;		/* Showmatch set on this character. */
 	int wm_set, wm_skip;	/* Wrapmargin happened, blank skip flags. */
 	int max, tmp;
-	char *p;
+	CHAR_T *p;
 
 	gp = sp->gp;
 	vip = VIP(sp);
@@ -308,7 +302,8 @@ v_txt(sp, vp, tm, lp, len, prompt, ai_line, rcount, flags)
 		tp->ai = tp->insert = tp->offset = tp->owrite = 0;
 		if (lp != NULL) {
 			tp->len = len;
-			memmove(tp->lb, lp, len);
+			BINC_RETW(sp, tp->lb, tp->lb_len, len);
+			MEMMOVEW(tp->lb, lp, len);
 		} else
 			tp->len = 0;
 	} else {
@@ -498,7 +493,7 @@ next:	if (v_event_get(sp, evp, 0, ec_flags))
 		    sp->rows ? 1 : sp->rows - vip->totalcount;
 		fc.e_tlno = sp->rows;
 		vip->linecount = vip->lcontinue = vip->totalcount = 0;
-		(void)vs_repaint(sp, &fc);
+		(void)v_erepaint(sp, &fc);
 		(void)vs_refresh(sp, 1);
 	}
 
@@ -520,7 +515,7 @@ next:	if (v_event_get(sp, evp, 0, ec_flags))
 		 */
 		goto k_escape;
 	case E_REPAINT:
-		if (vs_repaint(sp, &ev))
+		if (v_erepaint(sp, &ev))
 			return (1);
 		goto next;
 	case E_WRESIZE:
@@ -535,14 +530,14 @@ next:	if (v_event_get(sp, evp, 0, ec_flags))
 	/*
 	 * !!!
 	 * If the first character of the input is a nul, replay the previous
-	 * input.  Note, it was okay to replay non-existent input.  This was
-	 * not documented as far as I know, and is a great test of vi clones.
+	 * input.  (Historically, it's okay to replay non-existent input.)
+	 * This was not documented as far as I know, and is a great test of vi
+	 * clones.
 	 */
-	if (!LF_ISSET(TXT_REPLAY) && evp->e_c == '\0') {
+	if (rcol == 0 && !LF_ISSET(TXT_REPLAY) && evp->e_c == '\0') {
 		if (vip->rep == NULL)
 			goto done;
 
-		rcol = 0;
 		abb = AB_NOTSET;
 		LF_CLR(TXT_RECORD);
 		LF_SET(TXT_REPLAY);
@@ -579,7 +574,7 @@ next:	if (v_event_get(sp, evp, 0, ec_flags))
 
 	/* Abbreviation overflow check.  See comment in txt_abbrev(). */
 #define	MAX_ABBREVIATION_EXPANSION	256
-	if (F_ISSET(&evp->e_ch, CH_ABBREVIATED)) {
+	if (FL_ISSET(evp->e_flags, CH_ABBREVIATED)) {
 		if (++abcnt > MAX_ABBREVIATION_EXPANSION) {
 			if (v_event_flush(sp, CH_ABBREVIATED))
 				msgq(sp, M_ERR,
@@ -594,13 +589,16 @@ next:	if (v_event_get(sp, evp, 0, ec_flags))
 
 	/* Check to see if the character fits into the replay buffers. */
 	if (LF_ISSET(TXT_RECORD)) {
-		BINC_GOTO(sp, vip->rep,
+		BINC_GOTO(sp, EVENT, vip->rep,
 		    vip->rep_len, (rcol + 1) * sizeof(EVENT));
 		vip->rep[rcol++] = *evp;
 	}
 
-replay:	if (LF_ISSET(TXT_REPLAY))
+replay:	if (LF_ISSET(TXT_REPLAY)) {
+		if (rcol == vip->rep_cnt)
+			goto k_escape;
 		evp = vip->rep + rcol++;
+	}
 
 	/* Wrapmargin check for leading space. */
 	if (wm_skip) {
@@ -610,7 +608,7 @@ replay:	if (LF_ISSET(TXT_REPLAY))
 	}
 
 	/* If quoted by someone else, simply insert the character. */
-	if (F_ISSET(&evp->e_ch, CH_QUOTED))
+	if (FL_ISSET(evp->e_flags, CH_QUOTED))
 		goto insq_ch;
 
 	/*
@@ -662,6 +660,7 @@ replay:	if (LF_ISSET(TXT_REPLAY))
 	case K_NL:				/* New line. */
 		/* Return in script windows and the command line. */
 k_cr:		if (LF_ISSET(TXT_CR)) {
+			static CHAR_T cr[] = { '\r', 0 };
 			/*
 			 * If this was a map, we may have not displayed
 			 * the line.  Display it, just in case.
@@ -673,7 +672,7 @@ k_cr:		if (LF_ISSET(TXT_CR)) {
 				if (vs_change(sp, tp->lno, LINE_RESET))
 					goto err;
 			} else if (F_ISSET(sp, SC_SCRIPT))
-				(void)v_event_push(sp, NULL, "\r", 1, CH_NOMAP);
+				(void)v_event_push(sp, NULL, cr, 1, CH_NOMAP);
 
 			/* Set term condition: if empty. */
 			if (tp->cno <= tp->offset)
@@ -789,7 +788,7 @@ k_cr:		if (LF_ISSET(TXT_CR)) {
 			if (carat == C_NOCHANGE) {
 				if (v_txt_auto(sp, OOBLNO, &ait, ait.ai, ntp))
 					goto err;
-				FREE_SPACE(sp, ait.lb, ait.lb_len);
+				FREE_SPACEW(sp, ait.lb, ait.lb_len);
 			} else
 				if (v_txt_auto(sp, OOBLNO, tp, tp->cno, ntp))
 					goto err;
@@ -808,9 +807,9 @@ k_cr:		if (LF_ISSET(TXT_CR)) {
 			if (wmt.offset != 0 ||
 			    wmt.owrite != 0 || wmt.insert != 0) {
 #define	WMTSPACE	wmt.offset + wmt.owrite + wmt.insert
-				BINC_GOTO(sp, ntp->lb,
+				BINC_GOTOW(sp, ntp->lb,
 				    ntp->lb_len, ntp->len + WMTSPACE + 32);
-				memmove(ntp->lb + ntp->cno, wmt.lb, WMTSPACE);
+				MEMMOVEW(ntp->lb + ntp->cno, wmt.lb, WMTSPACE);
 				ntp->len += WMTSPACE;
 				ntp->cno += wmt.offset;
 				ntp->owrite = wmt.owrite;
@@ -821,7 +820,7 @@ k_cr:		if (LF_ISSET(TXT_CR)) {
 
 		/* New lines are TXT_APPENDEOL. */
 		if (ntp->owrite == 0 && ntp->insert == 0) {
-			BINC_GOTO(sp, ntp->lb, ntp->lb_len, ntp->len + 1);
+			BINC_GOTOW(sp, ntp->lb, ntp->lb_len, ntp->len + 1);
 			LF_SET(TXT_APPENDEOL);
 			ntp->lb[ntp->cno] = CH_CURSOR;
 			++ntp->insert;
@@ -842,6 +841,7 @@ k_cr:		if (LF_ISSET(TXT_CR)) {
 		if (rcount > 1) {
 			--rcount;
 
+			vip->rep_cnt = rcol;
 			rcol = 0;
 			abb = AB_NOTSET;
 			LF_CLR(TXT_RECORD);
@@ -892,7 +892,7 @@ k_escape:	LINE_RESOLVE;
 		 */
 		if (tp->owrite) {
 			if (tp->insert)
-				memmove(tp->lb + tp->cno,
+				MEMMOVEW(tp->lb + tp->cno,
 				    tp->lb + tp->cno + tp->owrite, tp->insert);
 			tp->len -= tp->owrite;
 		}
@@ -910,7 +910,7 @@ k_escape:	LINE_RESOLVE;
 			if (txt_resolve(sp, &sp->tiq, flags))
 				goto err;
 		} else {
-			BINC_GOTO(sp, tp->lb, tp->lb_len, tp->len + 1);
+			BINC_GOTOW(sp, tp->lb, tp->lb_len, tp->len + 1);
 			tp->lb[tp->len] = '\0';
 		}
 
@@ -954,8 +954,8 @@ k_escape:	LINE_RESOLVE;
 			/* Save the ai string for later. */
 			ait.lb = NULL;
 			ait.lb_len = 0;
-			BINC_GOTO(sp, ait.lb, ait.lb_len, tp->ai);
-			memmove(ait.lb, tp->lb, tp->ai);
+			BINC_GOTOW(sp, ait.lb, ait.lb_len, tp->ai);
+			MEMMOVEW(ait.lb, tp->lb, tp->ai);
 			ait.ai = ait.len = tp->ai;
 
 			carat = C_NOCHANGE;
@@ -1260,7 +1260,7 @@ ins_ch:		/*
 		 * wasn't a replay and wasn't handled specially, except
 		 * <tab> or <ff>.
 		 */
-		if (LF_ISSET(TXT_BEAUTIFY) && iscntrl(evp->e_c) &&
+		if (LF_ISSET(TXT_BEAUTIFY) && ISCNTRL(evp->e_c) &&
 		    evp->e_value != K_FORMFEED && evp->e_value != K_TAB) {
 			msgq(sp, M_BERR,
 			    "192|Illegal character; quote to enter");
@@ -1352,7 +1352,7 @@ insl_ch:	if (txt_insch(sp, tp, &evp->e_c, flags))
 		 * the length of the motion.
 		 */
 ebuf_chk:	if (tp->cno >= tp->len) {
-			BINC_GOTO(sp, tp->lb, tp->lb_len, tp->len + 1);
+			BINC_GOTOW(sp, tp->lb, tp->lb_len, tp->len + 1);
 			LF_SET(TXT_APPENDEOL);
 
 			tp->lb[tp->cno] = CH_CURSOR;
@@ -1414,7 +1414,8 @@ resolve:/*
 	 *    command line or doing file name completion, resolve them.
 	 */
 	if ((vip->totalcount != 0 || F_ISSET(gp, G_BELLSCHED)) &&
-	    !F_ISSET(sp, SC_TINPUT_INFO) && !filec_redraw && vs_resolve(sp, 0))
+	    !F_ISSET(sp, SC_TINPUT_INFO) && !filec_redraw &&
+	    vs_resolve(sp, NULL, 0))
 		return (1);
 
 	/*
@@ -1455,6 +1456,7 @@ done:	/* Leave input mode. */
 
 err:
 alloc_err:
+	F_CLR(sp, SC_TINPUT);
 	txt_err(sp, &sp->tiq);
 	return (1);
 }
@@ -1464,11 +1466,7 @@ alloc_err:
  *	Handle abbreviations.
  */
 static int
-txt_abbrev(sp, tp, pushcp, isinfoline, didsubp, turnoffp)
-	SCR *sp;
-	TEXT *tp;
-	CHAR_T *pushcp;
-	int isinfoline, *didsubp, *turnoffp;
+txt_abbrev(SCR *sp, TEXT *tp, CHAR_T *pushcp, int isinfoline, int *didsubp, int *turnoffp)
 {
 	VI_PRIVATE *vip;
 	CHAR_T ch, *p;
@@ -1561,7 +1559,7 @@ txt_abbrev(sp, tp, pushcp, isinfoline, didsubp, turnoffp)
 	 */
 search:	if (isinfoline)
 		if (off == tp->ai || off == tp->offset)
-			if (ex_is_abbrev(p, len)) {
+			if (ex_is_abbrev(sp, p, len)) {
 				*turnoffp = 1;
 				return (0);
 			} else
@@ -1612,7 +1610,7 @@ search:	if (isinfoline)
 		tp->owrite += len;
 	else {
 		if (tp->insert)
-			memmove(tp->lb + tp->cno + qp->olen,
+			MEMMOVEW(tp->lb + tp->cno + qp->olen,
 			    tp->lb + tp->cno + tp->owrite + len, tp->insert);
 		tp->owrite += qp->olen;
 		tp->len -= len - qp->olen;
@@ -1634,13 +1632,10 @@ search:	if (isinfoline)
  *	Handle the unmap command.
  */
 static void
-txt_unmap(sp, tp, ec_flagsp)
-	SCR *sp;
-	TEXT *tp;
-	u_int32_t *ec_flagsp;
+txt_unmap(SCR *sp, TEXT *tp, u_int32_t *ec_flagsp)
 {
 	size_t len, off;
-	char *p;
+	CHAR_T *p;
 
 	/* Find the beginning of this "word". */
 	for (off = tp->cno - 1, p = tp->lb + off, len = 0;; --p, --off) {
@@ -1666,7 +1661,7 @@ txt_unmap(sp, tp, ec_flagsp)
 	 * if the user erases the line and starts another command, we go ahead
 	 * an turn mapping back on.
 	 */
-	if ((off == tp->ai || off == tp->offset) && ex_is_unmap(p, len))
+	if ((off == tp->ai || off == tp->offset) && ex_is_unmap(sp, p, len))
 		FL_CLR(*ec_flagsp, EC_MAPINPUT);
 	else
 		FL_SET(*ec_flagsp, EC_MAPINPUT);
@@ -1677,15 +1672,12 @@ txt_unmap(sp, tp, ec_flagsp)
  *	When a line is resolved by <esc>, review autoindent characters.
  */
 static void
-txt_ai_resolve(sp, tp, changedp)
-	SCR *sp;
-	TEXT *tp;
-	int *changedp;
+txt_ai_resolve(SCR *sp, TEXT *tp, int *changedp)
 {
 	u_long ts;
 	int del;
 	size_t cno, len, new, old, scno, spaces, tab_after_sp, tabs;
-	char *p;
+	CHAR_T *p;
 
 	*changedp = 0;
 
@@ -1747,7 +1739,7 @@ txt_ai_resolve(sp, tp, changedp)
 
 	/* Shift the rest of the characters down, adjust the counts. */
 	del = old - new;
-	memmove(p - del, p, tp->len - old);
+	MEMMOVEW(p - del, p, tp->len - old);
 	tp->len -= del;
 	tp->cno -= del;
 
@@ -1764,17 +1756,13 @@ txt_ai_resolve(sp, tp, changedp)
  *	Handle autoindent.  If aitp isn't NULL, use it, otherwise,
  *	retrieve the line.
  *
- * PUBLIC: int v_txt_auto __P((SCR *, recno_t, TEXT *, size_t, TEXT *));
+ * PUBLIC: int v_txt_auto __P((SCR *, db_recno_t, TEXT *, size_t, TEXT *));
  */
 int
-v_txt_auto(sp, lno, aitp, len, tp)
-	SCR *sp;
-	recno_t lno;
-	TEXT *aitp, *tp;
-	size_t len;
+v_txt_auto(SCR *sp, db_recno_t lno, TEXT *aitp, size_t len, TEXT *tp)
 {
 	size_t nlen;
-	char *p, *t;
+	CHAR_T *p, *t;
 
 	if (aitp == NULL) {
 		/*
@@ -1801,15 +1789,15 @@ v_txt_auto(sp, lno, aitp, len, tp)
 		return (0);
 
 	/* Make sure the buffer's big enough. */
-	BINC_RET(sp, tp->lb, tp->lb_len, tp->len + nlen);
+	BINC_RETW(sp, tp->lb, tp->lb_len, tp->len + nlen);
 
 	/* Copy the buffer's current contents up. */
 	if (tp->len != 0)
-		memmove(tp->lb + nlen, tp->lb, tp->len);
+		MEMMOVEW(tp->lb + nlen, tp->lb, tp->len);
 	tp->len += nlen;
 
 	/* Copy the indentation into the new buffer. */
-	memmove(tp->lb, t, nlen);
+	MEMMOVEW(tp->lb, t, nlen);
 
 	/* Set the autoindent count. */
 	tp->ai = nlen;
@@ -1821,11 +1809,7 @@ v_txt_auto(sp, lno, aitp, len, tp)
  *	Back up to the previously edited line.
  */
 static TEXT *
-txt_backup(sp, tiqh, tp, flagsp)
-	SCR *sp;
-	TEXTH *tiqh;
-	TEXT *tp;
-	u_int32_t *flagsp;
+txt_backup(SCR *sp, TEXTH *tiqh, TEXT *tp, u_int32_t *flagsp)
 {
 	VI_PRIVATE *vip;
 	TEXT *ntp;
@@ -1900,10 +1884,7 @@ txt_backup(sp, tiqh, tp, flagsp)
  * changes.
  */
 static int
-txt_dent(sp, tp, isindent)
-	SCR *sp;
-	TEXT *tp;
-	int isindent;
+txt_dent(SCR *sp, TEXT *tp, int isindent)
 {
 	CHAR_T ch;
 	u_long sw, ts;
@@ -1931,7 +1912,7 @@ txt_dent(sp, tp, isindent)
 	 */
 	for (current = cno = 0; cno < tp->cno; ++cno)
 		current += tp->lb[cno] == '\t' ?
-		    COL_OFF(current, ts) : KEY_LEN(sp, tp->lb[cno]);
+		    COL_OFF(current, ts) : KEY_COL(sp, tp->lb[cno]);
 
 	target = current;
 	if (isindent)
@@ -1951,24 +1932,20 @@ txt_dent(sp, tp, isindent)
 
 	/*
 	 * Back up over any previous <blank> characters, changing them into
-	 * overwrite characters (including any ai characters).
+	 * overwrite characters (including any ai characters).  Then figure
+	 * out the current screen column.
 	 */
-	for (; tp->cno > tp->offset; --tp->cno, ++tp->owrite)
-		if (tp->lb[tp->cno - 1] == ' ')
-			--current;
-		else if (tp->lb[tp->cno - 1] == '\t') {
-			off = COL_OFF(current, ts);
-			if (current > off)
-				current -= off;
-			else
-				current = 0;
-		} else
-			break;
+	for (; tp->cno > tp->offset &&
+	    (tp->lb[tp->cno - 1] == ' ' || tp->lb[tp->cno - 1] == '\t');
+	    --tp->cno, ++tp->owrite);
+	for (current = cno = 0; cno < tp->cno; ++cno)
+		current += tp->lb[cno] == '\t' ?
+		    COL_OFF(current, ts) : KEY_COL(sp, tp->lb[cno]);
 
 	/*
 	 * If we didn't move up to or past the target, it's because there
 	 * weren't enough characters to delete, e.g. the first character
-	 * of the line was an tp->offset character, and the user entered
+	 * of the line was a tp->offset character, and the user entered
 	 * ^D to move to the beginning of a line.  An example of this is:
 	 *
 	 *	:set ai sw=4<cr>i<space>a<esc>i^T^D
@@ -2006,10 +1983,7 @@ txt_dent(sp, tp, isindent)
  *	File name completion.
  */
 static int
-txt_fc(sp, tp, redrawp)
-	SCR *sp;
-	TEXT *tp;
-	int *redrawp;
+txt_fc(SCR *sp, TEXT *tp, int *redrawp)
 {
 	struct stat sb;
 	ARGS **argv;
@@ -2017,7 +1991,9 @@ txt_fc(sp, tp, redrawp)
 	EXCMD cmd;
 	size_t indx, len, nlen, off;
 	int argc, trydir;
-	char *p, *t;
+	CHAR_T *p, *t;
+	char *np;
+	size_t nplen;
 
 	trydir = 0;
 	*redrawp = 0;
@@ -2049,16 +2025,14 @@ retry:		for (len = 0,
 	 * character.  I'm not sure if that's a bug or not...
 	 */
 	off = p - tp->lb;
-	BINC_RET(sp, tp->lb, tp->lb_len, tp->len + 1);
+	BINC_RETW(sp, tp->lb, tp->lb_len, tp->len + 1);
 	p = tp->lb + off;
 
 	s_ch = p[len];
 	p[len] = '*';
 
 	/* Build an ex command, and call the ex expansion routines. */
-	ex_cinit(&cmd, 0, 0, OOBLNO, OOBLNO, 0, NULL);
-	if (argv_init(sp, &cmd))
-		return (1);
+	ex_cinit(sp, &cmd, 0, 0, OOBLNO, OOBLNO, 0);
 	if (argv_exp2(sp, &cmd, p, len + 1)) {
 		p[len] = s_ch;
 		return (0);
@@ -2075,13 +2049,15 @@ retry:		for (len = 0,
 		return (0);
 	case 1:				/* One match. */
 		/* If something changed, do the exchange. */
-		nlen = strlen(cmd.argv[0]->bp);
-		if (len != nlen || memcmp(cmd.argv[0]->bp, p, len))
+		nlen = STRLEN(cmd.argv[0]->bp);
+		if (len != nlen || MEMCMP(cmd.argv[0]->bp, p, len))
 			break;
 
 		/* If haven't done a directory test, do it now. */
+		INT2CHAR(sp, cmd.argv[0]->bp, cmd.argv[0]->len + 1,
+			 np, nplen);
 		if (!trydir &&
-		    !stat(cmd.argv[0]->bp, &sb) && S_ISDIR(sb.st_mode)) {
+		    !stat(np, &sb) && S_ISDIR(sb.st_mode)) {
 			p += len;
 			goto isdir;
 		}
@@ -2124,26 +2100,27 @@ retry:		for (len = 0,
 	/* Shift remaining text up, and move the cursor to the end. */
 	if (nlen) {
 		off = p - tp->lb;
-		BINC_RET(sp, tp->lb, tp->lb_len, tp->len + nlen);
+		BINC_RETW(sp, tp->lb, tp->lb_len, tp->len + nlen);
 		p = tp->lb + off;
 
 		tp->cno += nlen;
 		tp->len += nlen;
 
 		if (tp->insert != 0)
-			(void)memmove(p + nlen, p, tp->insert);
+			(void)MEMMOVEW(p + nlen, p, tp->insert);
 		while (nlen--)
 			*p++ = *t++;
 	}
 
 	/* If a single match and it's a directory, retry it. */
-	if (argc == 1 && !stat(cmd.argv[0]->bp, &sb) && S_ISDIR(sb.st_mode)) {
+	INT2CHAR(sp, cmd.argv[0]->bp, cmd.argv[0]->len + 1, np, nplen);
+	if (argc == 1 && !stat(np, &sb) && S_ISDIR(sb.st_mode)) {
 isdir:		if (tp->owrite == 0) {
 			off = p - tp->lb;
-			BINC_RET(sp, tp->lb, tp->lb_len, tp->len + 1);
+			BINC_RETW(sp, tp->lb, tp->lb_len, tp->len + 1);
 			p = tp->lb + off;
 			if (tp->insert != 0)
-				(void)memmove(p + 1, p, tp->insert);
+				(void)MEMMOVEW(p + 1, p, tp->insert);
 			++tp->len;
 		} else
 			--tp->owrite;
@@ -2162,27 +2139,28 @@ isdir:		if (tp->owrite == 0) {
  *	Display file names for file name completion.
  */
 static int
-txt_fc_col(sp, argc, argv)
-	SCR *sp;
-	int argc;
-	ARGS **argv;
+txt_fc_col(SCR *sp, int argc, ARGS **argv)
 {
 	ARGS **av;
 	CHAR_T *p;
 	GS *gp;
 	size_t base, cnt, col, colwidth, numrows, numcols, prefix, row;
 	int ac, nf, reset;
+	char *np, *pp;
+	size_t nlen;
 
 	gp = sp->gp;
 
 	/* Trim any directory prefix common to all of the files. */
-	if ((p = strrchr(argv[0]->bp, '/')) == NULL)
+	INT2CHAR(sp, argv[0]->bp, argv[0]->len + 1, np, nlen);
+	if ((pp = strrchr(np, '/')) == NULL)
 		prefix = 0;
 	else {
-		prefix = (p - argv[0]->bp) + 1;
+		prefix = (pp - np) + 1;
 		for (ac = argc - 1, av = argv + 1; ac > 0; --ac, ++av)
 			if (av[0]->len < prefix ||
-			    memcmp(av[0]->bp, argv[0]->bp, prefix)) {
+			    MEMCMP(av[0]->bp, argv[0]->bp, 
+				   prefix)) {
 				prefix = 0;
 				break;
 			}
@@ -2219,14 +2197,16 @@ txt_fc_col(sp, argc, argv)
 
 	/* If the largest file name is too large, just print them. */
 	if (colwidth > sp->cols) {
-		p = msg_print(sp, av[0]->bp + prefix, &nf);
 		for (ac = argc, av = argv; ac > 0; --ac, ++av) {
-			(void)ex_printf(sp, "%s\n", p);
+			INT2CHAR(sp, av[0]->bp+prefix, av[0]->len+1-prefix,
+				 np, nlen);
+			pp = msg_print(sp, np, &nf);
+			(void)ex_printf(sp, "%s\n", pp);
 			if (F_ISSET(gp, G_INTERRUPTED))
 				break;
 		}
 		if (nf)
-			FREE_SPACE(sp, p, 0);
+			FREE_SPACE(sp, pp, 0);
 		CHK_INTR;
 	} else {
 		/* Figure out the number of columns. */
@@ -2241,10 +2221,12 @@ txt_fc_col(sp, argc, argv)
 		/* Display the files in sorted order. */
 		for (row = 0; row < numrows; ++row) {
 			for (base = row, col = 0; col < numcols; ++col) {
-				p = msg_print(sp, argv[base]->bp + prefix, &nf);
-				cnt = ex_printf(sp, "%s", p);
+				INT2CHAR(sp, argv[base]->bp+prefix, 
+					argv[base]->len+1-prefix, np, nlen);
+				pp = msg_print(sp, np, &nf);
+				cnt = ex_printf(sp, "%s", pp);
 				if (nf)
-					FREE_SPACE(sp, p, 0);
+					FREE_SPACE(sp, pp, 0);
 				CHK_INTR;
 				if ((base += numrows) >= argc)
 					break;
@@ -2274,14 +2256,12 @@ intr:		F_CLR(gp, G_INTERRUPTED);
  *	Set the end mark on the line.
  */
 static int
-txt_emark(sp, tp, cno)
-	SCR *sp;
-	TEXT *tp;
-	size_t cno;
+txt_emark(SCR *sp, TEXT *tp, size_t cno)
 {
-	CHAR_T ch, *kp;
+	CHAR_T ch;
+	char *kp;
 	size_t chlen, nlen, olen;
-	char *p;
+	CHAR_T *p;
 
 	ch = CH_ENDMARK;
 
@@ -2301,10 +2281,10 @@ txt_emark(sp, tp, cno)
 	 * to fix it up.
 	 */
 	if (olen > nlen) {
-		BINC_RET(sp, tp->lb, tp->lb_len, tp->len + olen);
+		BINC_RETW(sp, tp->lb, tp->lb_len, tp->len + olen);
 		chlen = olen - nlen;
 		if (tp->insert != 0)
-			memmove(tp->lb + cno + 1 + chlen,
+			MEMMOVEW(tp->lb + cno + 1 + chlen,
 			    tp->lb + cno + 1, tp->insert);
 
 		tp->len += chlen;
@@ -2327,11 +2307,9 @@ txt_emark(sp, tp, cno)
  *	Handle an error during input processing.
  */
 static void
-txt_err(sp, tiqh)
-	SCR *sp;
-	TEXTH *tiqh;
+txt_err(SCR *sp, TEXTH *tiqh)
 {
-	recno_t lno;
+	db_recno_t lno;
 
 	/*
 	 * The problem with input processing is that the cursor is at an
@@ -2362,14 +2340,12 @@ txt_err(sp, tiqh)
  * may not be able to enter.
  */
 static int
-txt_hex(sp, tp)
-	SCR *sp;
-	TEXT *tp;
+txt_hex(SCR *sp, TEXT *tp)
 {
 	CHAR_T savec;
 	size_t len, off;
 	u_long value;
-	char *p, *wp;
+	CHAR_T *p, *wp;
 
 	/*
 	 * Null-terminate the string.  Since nul isn't a legal hex value,
@@ -2396,7 +2372,7 @@ txt_hex(sp, tp)
 
 	/* Get the value. */
 	errno = 0;
-	value = strtol(wp, NULL, 16);
+	value = STRTOL(wp, NULL, 16);
 	if (errno || value > MAX_CHAR_T) {
 nothex:		tp->lb[tp->cno] = savec;
 		return (0);
@@ -2412,12 +2388,14 @@ nothex:		tp->lb[tp->cno] = savec;
 
 	/* Copy down any overwrite characters. */
 	if (tp->owrite)
-		memmove(tp->lb + tp->cno, tp->lb + tp->cno + len, tp->owrite);
+		MEMMOVEW(tp->lb + tp->cno, tp->lb + tp->cno + len, 
+		    tp->owrite);
 
 	/* Copy down any insert characters. */
 	if (tp->insert)
-		memmove(tp->lb + tp->cno + tp->owrite,
-		    tp->lb + tp->cno + tp->owrite + len, tp->insert);
+		MEMMOVEW(tp->lb + tp->cno + tp->owrite,
+		    tp->lb + tp->cno + tp->owrite + len, 
+		    tp->insert);
 
 	return (0);
 }
@@ -2442,15 +2420,12 @@ nothex:		tp->lb[tp->cno] = savec;
  * of the screen space they require, but that it not overwrite other characters.
  */
 static int
-txt_insch(sp, tp, chp, flags)
-	SCR *sp;
-	TEXT *tp;
-	CHAR_T *chp;
-	u_int flags;
+txt_insch(SCR *sp, TEXT *tp, CHAR_T *chp, u_int flags)
 {
-	CHAR_T *kp, savech;
+	char *kp;
+	CHAR_T savech;
 	size_t chlen, cno, copydown, olen, nlen;
-	char *p;
+	CHAR_T *p;
 
 	/*
 	 * The 'R' command does one-for-one replacement, because there's
@@ -2501,11 +2476,12 @@ txt_insch(sp, tp, chp, flags)
 				++copydown;
 				nlen -= olen;
 			} else {
-				BINC_RET(sp,
+				BINC_RETW(sp,
 				    tp->lb, tp->lb_len, tp->len + olen);
 				chlen = olen - nlen;
-				memmove(tp->lb + cno + 1 + chlen,
-				    tp->lb + cno + 1, tp->owrite + tp->insert);
+				MEMMOVEW(tp->lb + cno + 1 + chlen,
+				    tp->lb + cno + 1, 
+				    tp->owrite + tp->insert);
 
 				tp->len += chlen;
 				tp->owrite += chlen;
@@ -2528,7 +2504,7 @@ txt_insch(sp, tp, chp, flags)
 		 * into position.
 		 */
 		if (copydown != 0 && (tp->len -= copydown) != 0)
-			memmove(tp->lb + cno, tp->lb + cno + copydown,
+			MEMMOVEW(tp->lb + cno, tp->lb + cno + copydown,
 			    tp->owrite + tp->insert + copydown);
 
 		/* If we had enough overwrite characters, we're done. */
@@ -2539,14 +2515,14 @@ txt_insch(sp, tp, chp, flags)
 	}
 
 	/* Check to see if the character fits into the input buffer. */
-	BINC_RET(sp, tp->lb, tp->lb_len, tp->len + 1);
+	BINC_RETW(sp, tp->lb, tp->lb_len, tp->len + 1);
 
 	++tp->len;
 	if (tp->insert) {			/* Insert a character. */
 		if (tp->insert == 1)
 			tp->lb[tp->cno + 1] = tp->lb[tp->cno];
 		else
-			memmove(tp->lb + tp->cno + 1,
+			MEMMOVEW(tp->lb + tp->cno + 1,
 			    tp->lb + tp->cno, tp->owrite + tp->insert);
 	}
 	tp->lb[tp->cno++] = *chp;
@@ -2558,14 +2534,10 @@ txt_insch(sp, tp, chp, flags)
  *	Do an incremental search.
  */
 static int
-txt_isrch(sp, vp, tp, is_flagsp)
-	SCR *sp;
-	VICMD *vp;
-	TEXT *tp;
-	u_int8_t *is_flagsp;
+txt_isrch(SCR *sp, VICMD *vp, TEXT *tp, u_int8_t *is_flagsp)
 {
 	MARK start;
-	recno_t lno;
+	db_recno_t lno;
 	u_int sf;
 
 	/* If it's a one-line screen, we don't do incrementals. */
@@ -2682,14 +2654,11 @@ txt_isrch(sp, vp, tp, is_flagsp)
  *	Resolve the input text chain into the file.
  */
 static int
-txt_resolve(sp, tiqh, flags)
-	SCR *sp;
-	TEXTH *tiqh;
-	u_int32_t flags;
+txt_resolve(SCR *sp, TEXTH *tiqh, u_int32_t flags)
 {
 	VI_PRIVATE *vip;
 	TEXT *tp;
-	recno_t lno;
+	db_recno_t lno;
 	int changed;
 
 	/*
@@ -2739,9 +2708,7 @@ txt_resolve(sp, tiqh, flags)
  * I think not.
  */
 static int
-txt_showmatch(sp, tp)
-	SCR *sp;
-	TEXT *tp;
+txt_showmatch(SCR *sp, TEXT *tp)
 {
 	GS *gp;
 	VCS cs;
@@ -2808,15 +2775,11 @@ txt_showmatch(sp, tp)
  *	Handle margin wrap.
  */
 static int
-txt_margin(sp, tp, wmtp, didbreak, flags)
-	SCR *sp;
-	TEXT *tp, *wmtp;
-	int *didbreak;
-	u_int32_t flags;
+txt_margin(SCR *sp, TEXT *tp, TEXT *wmtp, int *didbreak, u_int32_t flags)
 {
 	VI_PRIVATE *vip;
 	size_t len, off;
-	char *p, *wp;
+	CHAR_T *p, *wp;
 
 	/* Find the nearest previous blank. */
 	for (off = tp->cno - 1, p = tp->lb + off, len = 0;; --off, --p, ++len) {
@@ -2888,15 +2851,11 @@ txt_margin(sp, tp, wmtp, didbreak, flags)
  *	Resolve the input line for the 'R' command.
  */
 static void
-txt_Rresolve(sp, tiqh, tp, orig_len)
-	SCR *sp;
-	TEXTH *tiqh;
-	TEXT *tp;
-	const size_t orig_len;
+txt_Rresolve(SCR *sp, TEXTH *tiqh, TEXT *tp, const size_t orig_len)
 {
 	TEXT *ttp;
 	size_t input_len, retain;
-	char *p;
+	CHAR_T *p;
 
 	/*
 	 * Check to make sure that the cursor hasn't moved beyond
@@ -2934,7 +2893,7 @@ txt_Rresolve(sp, tiqh, tp, orig_len)
 		if (db_get(sp,
 		    tiqh->cqh_first->lno, DBG_FATAL | DBG_NOCACHE, &p, NULL))
 			return;
-		memcpy(tp->lb + tp->cno, p + input_len, retain);
+		MEMCPYW(tp->lb + tp->cno, p + input_len, retain);
 		tp->len -= tp->owrite - retain;
 		tp->owrite = 0;
 		tp->insert += retain;
@@ -2946,8 +2905,7 @@ txt_Rresolve(sp, tiqh, tp, orig_len)
  *	No more characters message.
  */
 static void
-txt_nomorech(sp)
-	SCR *sp;
+txt_nomorech(SCR *sp)
 {
 	msgq(sp, M_BERR, "194|No more characters to erase");
 }
