@@ -10,7 +10,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)vs_msg.c	10.75 (Berkeley) 8/17/96";
+static const char sccsid[] = "$Id: vs_msg.c,v 10.85 2001/07/29 19:07:31 skimo Exp $ (Berkeley) $Date: 2001/07/29 19:07:31 $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -56,10 +56,7 @@ static void	vs_wait __P((SCR *, int *, sw_t));
  * PUBLIC: void vs_busy __P((SCR *, const char *, busy_t));
  */
 void
-vs_busy(sp, msg, btype)
-	SCR *sp;
-	const char *msg;
-	busy_t btype;
+vs_busy(SCR *sp, const char *msg, busy_t btype)
 {
 	GS *gp;
 	VI_PRIVATE *vip;
@@ -146,8 +143,7 @@ vs_busy(sp, msg, btype)
  * PUBLIC: void vs_home __P((SCR *));
  */
 void
-vs_home(sp)
-	SCR *sp;
+vs_home(SCR *sp)
 {
 	(void)sp->gp->scr_move(sp, LASTLINE(sp), 0);
 	(void)sp->gp->scr_refresh(sp, 0);
@@ -157,15 +153,15 @@ vs_home(sp)
  * vs_update --
  *	Update a command.
  *
- * PUBLIC: void vs_update __P((SCR *, const char *, const char *));
+ * PUBLIC: void vs_update __P((SCR *, const char *, const CHAR_T *));
  */
 void
-vs_update(sp, m1, m2)
-	SCR *sp;
-	const char *m1, *m2;
+vs_update(SCR *sp, const char *m1, const CHAR_T *m2)
 {
 	GS *gp;
 	size_t len, mlen, oldx, oldy;
+	CONST char *np;
+	size_t nlen;
 
 	gp = sp->gp;
 
@@ -178,8 +174,10 @@ vs_update(sp, m1, m2)
 	 * expanded, and by the ex substitution confirmation prompt.
 	 */
 	if (F_ISSET(sp, SC_SCR_EXWROTE)) {
+		if (m2 != NULL)
+			INT2CHAR(sp, m2, STRLEN(m2) + 1, np, nlen);
 		(void)ex_printf(sp,
-		    "%s\n", m1 == NULL? "" : m1, m2 == NULL ? "" : m2);
+		    "%s\n", m1 == NULL? "" : m1, m2 == NULL ? "" : np);
 		(void)ex_fflush(sp);
 	}
 
@@ -205,10 +203,10 @@ vs_update(sp, m1, m2)
 	} else
 		len = 0;
 	if (m2 != NULL) {
-		mlen = strlen(m2);
+		mlen = STRLEN(m2);
 		if (len + mlen > sp->cols - 2)
 			mlen = (sp->cols - 2) - len;
-		(void)gp->scr_addstr(sp, m2, mlen);
+		(void)gp->scr_waddstr(sp, m2, mlen);
 	}
 
 	(void)gp->scr_move(sp, oldy, oldx);
@@ -228,11 +226,7 @@ vs_update(sp, m1, m2)
  * PUBLIC: void vs_msg __P((SCR *, mtype_t, char *, size_t));
  */
 void
-vs_msg(sp, mtype, line, len)
-	SCR *sp;
-	mtype_t mtype;
-	char *line;
-	size_t len;
+vs_msg(SCR *sp, mtype_t mtype, char *line, size_t len)
 {
 	GS *gp;
 	VI_PRIVATE *vip;
@@ -394,13 +388,9 @@ ret:	(void)gp->scr_move(sp, oldy, oldx);
  *	Output the text to the screen.
  */
 static void
-vs_output(sp, mtype, line, llen)
-	SCR *sp;
-	mtype_t mtype;
-	const char *line;
-	int llen;
+vs_output(SCR *sp, mtype_t mtype, const char *line, int llen)
 {
-	CHAR_T *kp;
+	char *kp;
 	GS *gp;
 	VI_PRIVATE *vip;
 	size_t chlen, notused;
@@ -523,9 +513,7 @@ vs_output(sp, mtype, line, llen)
  * PUBLIC: int vs_ex_resolve __P((SCR *, int *));
  */
 int
-vs_ex_resolve(sp, continuep)
-	SCR *sp;
-	int *continuep;
+vs_ex_resolve(SCR *sp, int *continuep)
 {
 	EVENT ev;
 	GS *gp;
@@ -598,7 +586,7 @@ vs_ex_resolve(sp, continuep)
 	 * If we're not the bottom of the split screen stack, the screen
 	 * image itself is wrong, so redraw everything.
 	 */
-	if (sp->q.cqe_next != (void *)&sp->gp->dq)
+	if (sp->q.cqe_next != (void *)&sp->wp->scrq)
 		F_SET(sp, SC_SCR_REDRAW);
 
 	/* If ex changed the underlying file, the map itself is wrong. */
@@ -634,7 +622,7 @@ vs_ex_resolve(sp, continuep)
 		vip->linecount = vip->lcontinue = vip->totalcount = 0;
 
 		/* Redraw. */
-		(void)vs_repaint(sp, &ev);
+		(void)v_erepaint(sp, &ev);
 	} else
 		/* Reset the count of overwriting lines. */
 		vip->linecount = vip->lcontinue = vip->totalcount = 0;
@@ -646,26 +634,34 @@ vs_ex_resolve(sp, continuep)
  * vs_resolve --
  *	Deal with message output.
  *
- * This routine is called from the main vi loop to periodically ensure that
- * the user has seen any messages that have been displayed.
+ * PUBLIC: int vs_resolve __P((SCR *, SCR *, int));
  */
 int
-vs_resolve(sp, forcewait)
-	SCR *sp;
-	int forcewait;
+vs_resolve(SCR *sp, SCR *csp, int forcewait)
 {
 	EVENT ev;
 	GS *gp;
+	WIN *wp;
 	MSGS *mp;
 	VI_PRIVATE *vip;
 	size_t oldy, oldx;
 	int redraw;
 
+	/*
+	 * Vs_resolve is called from the main vi loop and the refresh function
+	 * to periodically ensure that the user has seen any messages that have
+	 * been displayed and that any status lines are correct.  The sp screen
+	 * is the screen we're checking, usually the current screen.  When it's
+	 * not, csp is the current screen, used for final cursor positioning.
+	 */
 	gp = sp->gp;
+	wp = sp->wp;
 	vip = VIP(sp);
+	if (csp == NULL)
+		csp = sp;
 
 	/* Save the cursor position. */
-	(void)gp->scr_cursor(sp, &oldy, &oldx);
+	(void)gp->scr_cursor(csp, &oldy, &oldx);
 
 	/* Ring the bell if it's scheduled. */
 	if (F_ISSET(gp, G_BELLSCHED)) {
@@ -692,7 +688,7 @@ vs_resolve(sp, forcewait)
 		if (!F_ISSET(sp, SC_SCR_VI) && vs_refresh(sp, 1))
 			return (1);
 		while ((mp = gp->msgq.lh_first) != NULL) {
-			gp->scr_msg(sp, mp->mtype, mp->buf, mp->len);
+			wp->scr_msg(sp, mp->mtype, mp->buf, mp->len);
 			LIST_REMOVE(mp, q);
 			free(mp->buf);
 			free(mp);
@@ -737,10 +733,10 @@ vs_resolve(sp, forcewait)
 
 	/* Redraw. */
 	if (redraw)
-		(void)vs_repaint(sp, &ev);
+		(void)v_erepaint(sp, &ev);
 
 	/* Restore the cursor position. */
-	(void)gp->scr_move(sp, oldy, oldx);
+	(void)gp->scr_move(csp, oldy, oldx);
 
 	return (0);
 }
@@ -750,10 +746,7 @@ vs_resolve(sp, forcewait)
  *	Scroll the screen for output.
  */
 static void
-vs_scroll(sp, continuep, wtype)
-	SCR *sp;
-	int *continuep;
-	sw_t wtype;
+vs_scroll(SCR *sp, int *continuep, sw_t wtype)
 {
 	GS *gp;
 	VI_PRIVATE *vip;
@@ -771,7 +764,7 @@ vs_scroll(sp, continuep, wtype)
 		(void)gp->scr_deleteln(sp);
 
 		/* If there are screens below us, push them back into place. */
-		if (sp->q.cqe_next != (void *)&sp->gp->dq) {
+		if (sp->q.cqe_next != (void *)&sp->wp->scrq) {
 			(void)gp->scr_move(sp, LASTLINE(sp), 0);
 			(void)gp->scr_insertln(sp);
 		}
@@ -786,10 +779,7 @@ vs_scroll(sp, continuep, wtype)
  *	Prompt the user to continue.
  */
 static void
-vs_wait(sp, continuep, wtype)
-	SCR *sp;
-	int *continuep;
-	sw_t wtype;
+vs_wait(SCR *sp, int *continuep, sw_t wtype)
 {
 	EVENT ev;
 	VI_PRIVATE *vip;
@@ -860,8 +850,7 @@ vs_wait(sp, continuep, wtype)
  *	Draw a dividing line between the screen and the output.
  */
 static void
-vs_divider(sp)
-	SCR *sp;
+vs_divider(SCR *sp)
 {
 	GS *gp;
 	size_t len;
@@ -880,11 +869,7 @@ vs_divider(sp)
  *	Save a message for later display.
  */
 static void
-vs_msgsave(sp, mt, p, len)
-	SCR *sp;
-	mtype_t mt;
-	char *p;
-	size_t len;
+vs_msgsave(SCR *sp, mtype_t mt, char *p, size_t len)
 {
 	GS *gp;
 	MSGS *mp_c, *mp_n;
