@@ -10,7 +10,7 @@
 #include "config.h"
 
 #ifndef lint
-static const char sccsid[] = "@(#)vs_refresh.c	10.44 (Berkeley) 10/13/96";
+static const char sccsid[] = "$Id: vs_refresh.c,v 10.50 2001/06/25 15:19:37 skimo Exp $ (Berkeley) $Date: 2001/06/25 15:19:37 $";
 #endif /* not lint */
 
 #include <sys/types.h>
@@ -33,28 +33,6 @@ static const char sccsid[] = "@(#)vs_refresh.c	10.44 (Berkeley) 10/13/96";
 static void	vs_modeline __P((SCR *));
 static int	vs_paint __P((SCR *, u_int));
 
-/*
- * v_repaint --
- *	Repaint selected lines from the screen.
- *
- * PUBLIC: int vs_repaint __P((SCR *, EVENT *));
- */
-int
-vs_repaint(sp, evp)
-	SCR *sp;
-	EVENT *evp;
-{
-	SMAP *smp;
-
-	for (; evp->e_flno <= evp->e_tlno; ++evp->e_flno) {
-		smp = HMAP + evp->e_flno - 1;
-		SMAP_FLUSH(smp);
-		if (vs_line(sp, smp, NULL, NULL))
-			return (1);
-	}
-	return (0);
-}
-
 /*
  * vs_refresh --
  *	Refresh all screens.
@@ -62,9 +40,7 @@ vs_repaint(sp, evp)
  * PUBLIC: int vs_refresh __P((SCR *, int));
  */
 int
-vs_refresh(sp, forcepaint)
-	SCR *sp;
-	int forcepaint;
+vs_refresh(SCR *sp, int forcepaint)
 {
 	GS *gp;
 	SCR *tsp;
@@ -80,8 +56,8 @@ vs_refresh(sp, forcepaint)
 	 * that we can find, including status lines.
 	 */
 	if (F_ISSET(sp, SC_SCR_REDRAW))
-		for (tsp = gp->dq.cqh_first;
-		    tsp != (void *)&gp->dq; tsp = tsp->q.cqe_next)
+		for (tsp = sp->wp->scrq.cqh_first;
+		    tsp != (void *)&sp->wp->scrq; tsp = tsp->q.cqe_next)
 			if (tsp != sp)
 				F_SET(tsp, SC_SCR_REDRAW | SC_STATUS);
 
@@ -98,8 +74,8 @@ vs_refresh(sp, forcepaint)
 	priv_paint = VIP_CUR_INVALID | VIP_N_REFRESH;
 	if (O_ISSET(sp, O_NUMBER))
 		priv_paint |= VIP_N_RENUMBER;
-	for (tsp = gp->dq.cqh_first;
-	    tsp != (void *)&gp->dq; tsp = tsp->q.cqe_next)
+	for (tsp = sp->wp->scrq.cqh_first;
+	    tsp != (void *)&sp->wp->scrq; tsp = tsp->q.cqe_next)
 		if (tsp != sp && !F_ISSET(tsp, SC_EXIT | SC_EXIT_FORCE) &&
 		    (F_ISSET(tsp, pub_paint) ||
 		    F_ISSET(VIP(tsp), priv_paint))) {
@@ -134,8 +110,8 @@ vs_refresh(sp, forcepaint)
 	 * And, finally, if we updated any status lines, make sure the cursor
 	 * gets back to where it belongs.
 	 */
-	for (need_refresh = 0, tsp = gp->dq.cqh_first;
-	    tsp != (void *)&gp->dq; tsp = tsp->q.cqe_next)
+	for (need_refresh = 0, tsp = sp->wp->scrq.cqh_first;
+	    tsp != (void *)&sp->wp->scrq; tsp = tsp->q.cqe_next)
 		if (F_ISSET(tsp, SC_STATUS)) {
 			need_refresh = 1;
 			vs_resolve(tsp, sp, 0);
@@ -162,17 +138,15 @@ vs_refresh(sp, forcepaint)
  *	what you're doing.  It's subtle and quick to anger.
  */
 static int
-vs_paint(sp, flags)
-	SCR *sp;
-	u_int flags;
+vs_paint(SCR *sp, u_int flags)
 {
 	GS *gp;
 	SMAP *smp, tmp;
 	VI_PRIVATE *vip;
-	recno_t lastline, lcnt;
+	db_recno_t lastline, lcnt;
 	size_t cwtotal, cnt, len, notused, off, y;
 	int ch, didpaint, isempty, leftright_warp;
-	char *p;
+	CHAR_T *p;
 
 #define	 LNO	sp->lno			/* Current file line. */
 #define	OLNO	vip->olno		/* Remembered file line. */
@@ -345,7 +319,7 @@ bottom:		if (db_last(sp, &lastline))
 		tmp.lno = LNO;
 		tmp.coff = HMAP->coff;
 		tmp.soff = 1;
-		lcnt = vs_sm_nlines(sp, &tmp, lastline, sp->t_rows);
+		lcnt = vs_sm_nlines(sp, &tmp, lastline+1, sp->t_rows);
 		if (lcnt < HALFTEXT(sp)) {
 			if (vs_sm_fill(sp, lastline, P_BOTTOM))
 				return (1);
@@ -507,8 +481,8 @@ adjust:	if (!O_ISSET(sp, O_LEFTRIGHT) &&
 		 * Count up the widths of the characters.  If it's a tab
 		 * character, go do it the the slow way.
 		 */
-		for (cwtotal = 0; cnt--; cwtotal += KEY_LEN(sp, ch))
-			if ((ch = *(u_char *)p--) == '\t')
+		for (cwtotal = 0; cnt--; cwtotal += KEY_COL(sp, ch))
+			if ((ch = *(UCHAR_T *)p--) == '\t')
 				goto slow;
 
 		/*
@@ -521,8 +495,8 @@ adjust:	if (!O_ISSET(sp, O_LEFTRIGHT) &&
 		 * If we're moving left, and there's a wide character in the
 		 * current position, go to the end of the character.
 		 */
-		if (KEY_LEN(sp, ch) > 1)
-			cwtotal -= KEY_LEN(sp, ch) - 1;
+		if (KEY_COL(sp, ch) > 1)
+			cwtotal -= KEY_COL(sp, ch) - 1;
 
 		/*
 		 * If the new column moved us off of the current logical line,
@@ -547,9 +521,9 @@ adjust:	if (!O_ISSET(sp, O_LEFTRIGHT) &&
 		 * screen boundary, we can quit.
 		 */
 		for (cwtotal = SCNO; cnt--;) {
-			if ((ch = *(u_char *)p++) == '\t')
+			if ((ch = *(UCHAR_T *)p++) == '\t')
 				goto slow;
-			if ((cwtotal += KEY_LEN(sp, ch)) >= SCREEN_COLS(sp))
+			if ((cwtotal += KEY_COL(sp, ch)) >= SCREEN_COLS(sp))
 				break;
 		}
 
@@ -688,8 +662,11 @@ done_cursor:
 	 * NULL.  If we're debugging, die, otherwise restart from scratch.
 	 */
 #ifdef DEBUG
-	if (vip->sc_smap == NULL)
+	if (vip->sc_smap == NULL) {
+		fprintf(stderr, "smap error\n");
+		sleep(100);
 		abort();
+	}
 #else
 	if (vip->sc_smap == NULL) {
 		F_SET(sp, SC_SCR_REFORMAT);
@@ -763,8 +740,7 @@ number:	if (O_ISSET(sp, O_NUMBER) &&
  *	Update the mode line.
  */
 static void
-vs_modeline(sp)
-	SCR *sp;
+vs_modeline(SCR *sp)
 {
 	static char * const modes[] = {
 		"215|Append",			/* SM_APPEND */
